package sugr;

import sugr.linalg.*;
/**
 * <p>
 * This class contains constructors to build a Affinity_2D. Its matrix needs 6
 * entries to build it:
 * A Affinity_2D consists of:
 * <ul>
 * 		<li> Transformation(2) </li> 
 * 		<li> Rotation(1),   </li>
 * 		<li> Scale (2),  </li>
 * 		<li> Shear (1),  </li>
 * </ul>
 * @author Norbert Fischer
 */
public class Affinity_2D extends Homography_2D 
{
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * standard constructor. creates an identity matrix
	 */
	public Affinity_2D()  {}
	
	/**
	 * sets the matrix by hand. positions are:
	 * <pre>
	 *           |a  c  e|
	 *           |b  d  f|
	 *           |0  0  1|
	 * </pre>
	 */
	public Affinity_2D( double a, double b, 
						double c, double d, 
						double e, double f)
	{
		this(a,b,c,d,e,f, LinearAlgebraFactory.getMatrix(DIM*DIM,DIM*DIM));
	}

	/**
	 * sets the matrix with covariance by hand. positions are:
	 * <pre>
	 *           |a  c  e|
	 *           |b  d  f|
	 *           |0  0  1|
	 * </pre>
	 * @param cov   9x9 matrix
	 */
	public Affinity_2D( double a, double b, 
						double c, double d, 
						double e, double f,
						Matrix cov)
	{
		Vector tmpV = LinearAlgebraFactory.getVector(DIM*DIM);
		tmpV.set(0, a); tmpV.set(1, b);
		tmpV.set(3, c); tmpV.set(4, d);
		tmpV.set(6, e); tmpV.set(7, f);
		tmpV.set(8, 1);
		
		setV(tmpV);
		
		checkCovDim(cov, DIM*DIM);
		setCov(cov);
	}

	
	/**
	 * Creates an affinity_2D by 6 paramaters:
	 * <ul>
	 * 		<li> Transformation(2) </li> 
	 * 		<li> Rotation(1),   </li>
	 * 		<li> Scale (2), in x and y different values possible  </li>
	 * 		<li> Shear (1),  </li>
	 * </ul>
	 * Error propagation is done automatically.
	 * @param rot   contains the rotation matrix
	 * @param trans   contains the translation matrix
	 * @param scale   a vector with dimension 2 
	 * @param shear   just as the name says
	 * @param shearSymmetric   boolean Flag. Set it true to get symmetric shear
	 * 						   otherwise unsymmetric
	 */
	public Affinity_2D (Rotation_2D rot, Translation_2D trans,
						Vector scale, double shear, boolean shearSymmetric)
	{
		this(rot, trans, scale, shear, shearSymmetric, null);
	}

	/**
	 * Creates an affinity_2D with covariance by 6 paramaters:
	 * <ul>
	 * 		<li> Transformation(2) </li> 
	 * 		<li> Rotation(1),   </li>
	 * 		<li> Scale (2), in x and y different values possible  </li>
	 * 		<li> Shear (1),  </li>
	 * </ul>
	 * No error propagation will be done.
	 * @param rot   contains the rotation matrix
	 * @param trans   contains the translation matrix
	 * @param scale   a vector with dimension 2 
	 * @param shear   just as the name says
	 * @param shearSymmetric   boolean Flag. Set it true to get symmetric shear
	 * 						   otherwise unsymmetric
	 * @param cov  9x9-matrix
	 */
	public Affinity_2D (Rotation_2D rot, Translation_2D trans,
						Vector scale, double shear, boolean shearSymmetric,
						Matrix cov)
	{
		//check dimension
		if (scale.getDim()!=DIM-1) 
		{
			throw new IllegalArgumentException("scale vector must " +
				"have two entries!");
		}
		
		//set Rotation & Transformation
		Motion_2D mot = new Motion_2D(trans, rot); 
		Matrix aff = mot.getTransformationMatrix();
		
		//Set scaleMat
		//changed to Matrix multiply by Bastian Oehler because of errors
		//aff.set(0,0, aff.get(0,0)*scale.get(0));
		//aff.set(1,1, aff.get(1,1)*scale.get(1));
		Matrix scaleMat = LinearAlgebraFactory.getMatrixId(DIM);
		scaleMat.set(0,0,scale.get(0));
		scaleMat.set(1,1,scale.get(1));
		aff = aff.mult(scaleMat);
		
		//Set shear
		Matrix shearMat = LinearAlgebraFactory.getMatrixId(DIM);
		if (shearSymmetric)
		{	//symmetric shear
			shearMat.set(0,1, shear/2.0);
			shearMat.set(1,0, shear/2.0);
		}
		else
		{	//unsymmetric shear
			shearMat.set(0,1, shear);
		}
		
		aff = aff.mult(shearMat);
		
		setV(aff.vec());
		
		//FFG
		//null is a kind of flag to show, that FFG should be performed
		if (cov==null)	
		{
			//Because this homography CONSISTS of mot scale and shear, no
			//error propagation could done (between what, if so?)			
			setCov(mot.getCov());
		}
		//if not null, then cov must be given by user und therefore be checked ;-)
		else
		{
			checkCovDim(cov, DIM*DIM);
			setCov(cov);
		}
	}
	
	/**
	 * Returns the number of elements to specify this transformation
	 */
	public int getDegreesOfFreedom()
	{
		return 6;
	}	
	
} // end Affinity_2D



