/** Java class "Element.java" generated from Poseidon for UML. 
 *  Poseidon for UML is developed by <A HREF="http://www.gentleware.com">Gentleware</A>.
 *  Generated with <A HREF="http://jakarta.apache.org/velocity/">velocity</A> template engine.
 */
package sugr;

import java.io.Serializable;

import sugr.linalg.*;

/**
 * This class is the "mother" of all elements like like Point_2D, Line_3D, 
 * Plane_3D, Transformation and so on.
 * It's abstract and it is a place where non specific method may be placed
 * This class knowns nothing about a homogenous and euclidean part. This
 * distinction is done in the subclass Entity.
 * 
 * @author Norbert Fischer
 * @version 1.1 (25.1.2004)
 * 		Added calcCov() as protected method to this class, because it is used 
 * 		in some Constructors of Lines and Points.
 * 
 */
public abstract class Element implements Cloneable,Serializable {

	///////////////////////////////////////
	// attributes

	/**
	 * Represents the Covariance Matrix
	 */
	protected Matrix cov;

	/**
	 * Represents the coordinates of an Entitiy like Point_3D. It is also used to
	 * store a transformation from Transformation-classes. 
	 */
	protected Vector v;

	///////////////////////////////////////
	// operations

	/**
	 * returns the Covariance Matrix. 
	 * BE CAREFUL:
	 * You get the reference of the covariance matrix,
	 * so a change of the reference results in a change of the 
	 * Element !
     * It may be insecure in future versions to modify the matrix
     * without a final setV().
	 * @return a Matrix, containing the covariances
	 */
	public Matrix getCov() {
		return cov;
	} // end getCov        

	/**
	 * Sets the covariance matrix of this element. 
	 * BE CAREFUL: The reference is set, so no copy is made !
	 * @param C containing a suitable matrix
	 */
	public void setCov(Matrix C) {
		cov = C;
	} // end setCov        

	/**
	 * returns coordinates of the element in a Vector
 	 * BE CAREFUL:
	 * You get the reference of the vector,
	 * so a change of the reference results in a change of the 
	 * Element !
     * It may be insecure in future versions to modify the vector
     * without a final setV().
	 * @return a Vector
	 */
	public Vector getV() {
		// your code here
		return v;
	} // end getV        

	/**
	 * Sets the coordinates of the element
 	 * BE CAREFUL: The reference is set, so no copy is made !
	 * @param v contains the coordinates
	 */
	public void setV(Vector v) {
		this.v = v;
	} // end setV        

	/**
	 * returns the minimum number of parameters to construct an element 
	 * @return a int
	 */
	public abstract int getDegreesOfFreedom();

	/**
	 * returns the dimension of an element. E.g. Line_3D.getDimension()
	 * returns 6. For Transformations this is the dimension of the
	 * transformation matrix.
	 */
	public abstract int getDimension();
	
	/** Condition that element with a set of conditioning factors.
	 * @param f conditioning factors.
	 */
	public abstract void doCondition(double[] f);

	/**
	 *	Print out element. If you
	 *  want to have a better control of the output, use 
	 *  print(width, digits)
	 * @return a String with element vector and covariance.
	 */
	public String toString() {
		// your code here
		String s="Vector:\n"+v.toString()+"Covariance:\n"+cov.toString();
		return s;
	} // end toString        

	/**
	 * Normalize this element to unit-vector length
	 */
	public void normalizeSphere() {
		double xLength = getV().getLength();

		Matrix x = getV().getMatrix();
		Matrix Id = LinearAlgebraFactory.getMatrixId(getV().getDim());
		Matrix jacobian_S =
			Id.minus(x.mult(x.trans()).mult(1.0 / (xLength * xLength)));
		jacobian_S = jacobian_S.mult(1.0 / xLength);

		setCov(jacobian_S.mult(getCov()).mult(jacobian_S.trans()));
		setV(getV().mult(1.0 / xLength));

	} // end normalizeSphere     
	
	/**
	 * Sherical normalization with comparable result: To
	 * avoid a sign ambiguity the largest element |xi| of v is
	 * positive after this normalization. This is slightly
	 * more expensive than normalizeShere
	 */
	public void normalizeSphereComparable() {
		normalizeSphere();
		double max = 0;
		for (int i = 0; i < v.getDim(); i++)
			// 1e-16 to avoid ambiguities when values are identical
			if (Math.abs(v.get(i)) + 1e-16 > Math.abs(max))
				max = v.get(i);
		if (max < 0)
			v = v.mult(-1.0);
	}
   


	/**
	 *	The covariance matrix is changed in that way: An orthogonal projection onto
	 *	{y}^{orthogonal} will be done. Vector ist not changed
	 */
	public void changeNullspace() {	
		// Now use JacobianOfConstraint to work for Line_3D as well (ChB, 20.5.2005)
		Matrix x = getJacobianOfConstraint().trans();
//		Matrix x = getV().getMatrix();
		Matrix Id = LinearAlgebraFactory.getMatrixId(getV().getDim());
		Matrix Psi = Id.minus(x.mult((x.trans().mult(x)).invert()).mult(x.trans()));
		setCov(Psi.mult(getCov()).mult(Psi.trans()));
	} // end changeNullspace        

	/**
	 *	The covariance matrix is changed in that way: An orthogonal projection onto
	 *	{y}^{orthogonal} will be done. Afterwards a small extension epsilon is added to the
	 *  nullspace, so that the covariance matrix has full rank. Vector ist not changed.
	 * 
	 * 	@param epsilon amount, that is added to the null-direction
	 */
	public void changeNullspace(double epsilon) {	
		Matrix G = getJacobianOfConstraint().trans();
		Matrix NullSpace = G.mult((G.trans().mult(G)).invert()).mult(G.trans());		
		Matrix Id = LinearAlgebraFactory.getMatrixId(getV().getDim());
		Matrix Psi = Id.minus(NullSpace);
		setCov(Psi.mult(getCov()).mult(Psi.trans()).plus(NullSpace.mult(epsilon)));
	} // end changeNullspace        

	/**
	 * prints this element to stdout. <br>
	 * A common value is w=5, d=3
	 * 
	 * @param width describes the space a value has to show itself
	 * @param digits how many places after the comma should be shown.
	 * 		   it is not clever to choose digits >= width
	 */
	public void print(int width, int digits) {
		// width means colums-width, digits is for #digits after the decimal
		System.out.print("Vector");
		getV().print(width, digits);
		System.out.print("Covariance-Matrix");
		getCov().print(width, digits);
	} // end print     

	/**
	 * prints this element to stdout. <br>
	 * 
	 */
	public void print() {
		System.out.print("Vector");
		getV().print();
		System.out.print("Covariance-Matrix");
		getCov().print();
	} // end print     

	/**
	 * Checks the dimension of a given covariance matrix if it matches
	 * the dimension of the element
	 * @param cov the covariance matrix to be checked
	 * @param dim the dimension of the element
	 */
	public void checkCovDim(Matrix cov, int dim) {
		if ((cov.getRowDimension() != dim)
			&& (cov.getColumnDimension() != dim)) {
			throw new IllegalArgumentException(
				"Covariance-Matrix dimensions must be "
					+ dim
					+ "x"
					+ dim
					+ ".");
		}
	} // end checkCovDim()

	/**
	 * Checks if not all values are = 0. If so => error!
	 * @param vec the coordinates to be checked
	 * @param dim the dimension of the element
	 */
	public static void checkElement(Vector vec, int dim) {
		if (vec.getDim() != dim) {
			throw new IllegalArgumentException(
				"Vector dimension must be " + dim + ".");
		}

		for (int i = 0; i < vec.getDim(); i++) {
			if (vec.get(i) != 0)
				return;
		}
		throw new IllegalArgumentException("Not all parameters must be Null");
	} // end checkEntity()

	/** Clone an element. The vector v and matrix c are cloned.
	 *
	 * @return a copy of the element. 
	 */
	public Object clone() {
		try {
			Element e = (Element) super.clone();
			e.v = (Vector) v.clone();
			e.cov = (Matrix) cov.clone();
			return e;

		} catch (CloneNotSupportedException e) {
			// this shouldn't happen 
			throw new InternalError();
		}

	}
	
	/**
	 * Compute an approximation of an entity. The result of this
	 * method approximates the optimal solution which fullfills all
	 * the given relational constraints in an optimal way. The old values
	 * of the entity are overwritten. Implementation
	 * according to Table 3.12 on page 89 from dissertation from
	 * S. Heuel. No statistics are used, so the
	 * covariance matrix are set to 0.
	 * No conditioning is done !
	 * @param rprops a set of relational Properties which should be fullfilled.
	 */
	public void approximate(RelationalProperty[] rprops) {
		Matrix Ai = rprops[0].getJacobian();
		Matrix A  = Ai.trans().mult(Ai);
			
		for (int i = 1; i < rprops.length; i++) {
			Ai = rprops[i].getJacobian();
			A = A.plus(Ai.trans().mult(Ai));
		}
		Matrix[] svd = A.svd();

		Vector x = svd[2].getColumn(svd[2].getColumnDimension() - 1);
		v = x;
		cov = LinearAlgebraFactory.getMatrix(x.getDim(), x.getDim());
	}

	/** Estimation of that entity out of given relational properties.
	 * To get Information about the estimation, call estimate with
	 * more parameters.
	 * @param rprops the observations
	 */
	public boolean estimate(RelationalProperty[] rprops) {
		RelationalProperty[] yhat = new RelationalProperty[rprops.length];
		double[] sigma0 = new double[1];
		int[] iterations = new int[1];
		return estimate(rprops, true, MAXITER, yhat, sigma0, iterations, true);
	}

	/** Estimation of that entity out of given relational properties.
	  * Hints: Change your coordinate system in that way, that
	  *        the entities lie around the origin. ! This seems 
	  *        important especially for 3D. Tests have shown
	  *        sensitivity to such a transformation if estimating
	  *        3D points from 3D points and 3D lines from 3D lines.
	  * 	   If distance between center of gravity and origin 
	  * 	   exceeds 1 conditioning is needed! If distance 
	  *        exceeds 10*10^14 conditioning can't help, you 
	  *        have to change coordinate system! 
	  *        See also: sugr.test.EstimationTest.conditioningTest()
	  * 					and its comments.
	  * @param rprops the observations
	  * @param doConditioning if true, a conditioning is done.
	  * @param maxiter maximum number of iterations.
	  * @param yhat corrected observations, spherically normalized. yhat=y-ehat.
	  * @param sigma0 sigma0 of the last iterations (sqrt(sigma2)) as
	  *        an output parameter (array with 1 element)
	  * @param no_of_iterations number of iterations computed (output
	  *        parameter: array with 1 element)
	  * @return true if convergence reached otherwise false. Convergence
	  *         is not reached if the maximum number of iterations is
	  *         reached and the convergence criteria is not reached.
	  */
	public boolean estimate(
		RelationalProperty[] rprops,
		boolean doConditioning,
		int maxiter,
		RelationalProperty[] yhat,
		double[] sigma0,
		int[] no_of_iterations,
		boolean estimate_sigma0) {

		RelationalProperty[] rpropscond = new RelationalProperty[rprops.length];
		for (int i = 0; i < rprops.length; i++)
			rpropscond[i] = (RelationalProperty) rprops[i].clone();

		double[] f= new double[1];

		if (doConditioning) {
			if (rprops[0] instanceof TriRelationalProperty) {
				f = TriRelationalProperty.doCondition(rpropscond);
			} else
				f = BiRelationalProperty.doCondition(rpropscond);
		}

		// normalize spherecally (this also changes Nullspace !)
		for (int i = 0; i < rprops.length; i++) {
			if (rpropscond[i] instanceof BiRelationalProperty) {
				((BiRelationalProperty) rpropscond[i]).getFirstEntity().normalizeSphere();
				((BiRelationalProperty) rpropscond[i]).getFirstEntity().changeNullspace();
			}
			if (rpropscond[i] instanceof TriRelationalProperty) {
				((TriRelationalProperty) rpropscond[i]).getFirstEntity().normalizeSphere();
				((TriRelationalProperty) rpropscond[i]).getFirstEntity().changeNullspace();
				((TriRelationalProperty) rpropscond[i]).getSecondEntity().normalizeSphere();
				((TriRelationalProperty) rpropscond[i]).getSecondEntity().changeNullspace();
			}
		}

		// Get initial values for the estimation.
		approximate(rpropscond);


		// and now do iterations
		boolean reachedconvergence =
			gaussHelmertIterations(
				rpropscond,
				maxiter,
				yhat,
				sigma0,
				no_of_iterations,
				estimate_sigma0);

		// reconditioning
		if (doConditioning) {
			boolean neccessary = false;
			double[] ref = new double[f.length];
			for (int i = 0; i < ref.length; i++) {
				ref[i] = 1.0 / f[i];
				if (f[i] != 1)
					neccessary = true;
			}

			if (neccessary) {
				// Recondition estimate
				doCondition(ref);

				// Reconditiong of corrected observations
				if (rprops[0] instanceof TriRelationalProperty)
					TriRelationalProperty.doCondition(yhat, ref);

				else
					BiRelationalProperty.doCondition(yhat, ref);
			}
		}

		return reachedconvergence;
	}

	public boolean gaussHelmertIterations(
			RelationalProperty[] rprops,
			int maxiter,
			RelationalProperty[] yhat,
			double[] sigma0,
			int[] no_of_iterations,
			boolean estimate_sigma0) {
		
		// Clone the observations, so that adjustments have no impact on the original data
		int W=0;
		Vector[] l = new Vector[yhat.length]; 
		for (int i=0; i<rprops.length; i++) {
			yhat[i] = (RelationalProperty)rprops[i].clone();
			l[i] = rprops[i].getObservations();  // This could be optimized and put out of the iteration...
			W+=rprops[i].getDof();
		}

		boolean stop=false;
		int iteration;
		double sigma2=0;
		
		for (iteration = 0; !stop && iteration < maxiter; iteration++) {		
			// Collect all required ingredients
			Matrix[] Cll = new Matrix[yhat.length]; 
			Matrix[] A = new Matrix[yhat.length]; 
			Matrix[] B = new Matrix[yhat.length];
			Matrix[] G = new Matrix[yhat.length]; 
			Vector[] g0 = new Vector[yhat.length]; 
			Vector[] f0 = new Vector[yhat.length]; 
			Vector[] l0 = new Vector[yhat.length]; 
			Matrix[] Cll_plus = new Matrix[yhat.length]; 
			Matrix[] invSigma = new Matrix[yhat.length];
			Vector[] cw = new Vector[yhat.length];
			for (int i = 0; i < yhat.length; i++) {
				Cll[i] = yhat[i].getCovarianceOfObservations();
				Matrix[] AB = getReducedAB(yhat[i]);
				A[i] = AB[0];
				B[i] = AB[1];
				G[i] = yhat[i].getJacobianOfObservationConstraints();			
				g0[i] = A[i].mult(v);
				f0[i] = yhat[i].getValueOfObservationConstraints();
				
				l0[i] = yhat[i].getObservations();
				Cll_plus[i] = Cll[i].pseudoInverse(G[i]);
				invSigma[i] = (B[i].mult(Cll[i]).mult(B[i].trans())).invert();
				cw[i] = invSigma[i].mult(((B[i].mult(Cll[i]).mult(Cll_plus[i]).mult(l0[i].minus(l[i]))).plus(B[i].mult(G[i].trans().mult((G[i].mult(G[i].trans())).invert()).mult(f0[i]))).minus(g0[i])));
			}
			Matrix H = getJacobianOfConstraint();
			Vector h0 = getConstraintValue();
					
			// Build normal equation system
			Matrix Npart = LinearAlgebraFactory.getMatrix (v.getDim(), v.getDim());
			Vector bpart = LinearAlgebraFactory.getVector (v.getDim());
			for (int i=0; i<yhat.length; i++) {
				Npart = Npart.plus(A[i].trans().mult(invSigma[i]).mult(A[i]));
				bpart = bpart.plus(A[i].trans().mult(cw[i]));
			}
			// Improve condition
			double npartsum=0;
			for(int i=0; i<Npart.getRowDimension(); i++)
				npartsum+=Npart.get(i,i);
			npartsum/=Npart.getRowDimension();
			Matrix HH=H.trans().mult(H);
			double hsum=0;
			for(int i=0; i<HH.getRowDimension(); i++)
				hsum+=HH.get(i,i);
			hsum/=HH.getRowDimension();
			double hcondfactor=npartsum/hsum;
			H=H.mult(hcondfactor);
			h0=h0.mult(hcondfactor);
			Matrix N = LinearAlgebraFactory.getMatrix(v.getDim()+h0.getDim(), v.getDim()+h0.getDim());
			N.slice (0,v.getDim()-1,0,v.getDim()-1,Npart);
			N.slice (v.getDim(),v.getDim()+h0.getDim()-1,0,v.getDim()-1, H);
			N.slice (0,v.getDim()-1,v.getDim(),v.getDim()+h0.getDim()-1, H.trans());		
			Vector b = LinearAlgebraFactory.getVector(v.getDim()+h0.getDim());
			b.set(0,bpart);
			b.set(v.getDim(),h0.mult(-1));
			
			// Solve normal equation system 
			Matrix Ninv = N.invert();
			Vector solution = Ninv.mult(b);		
			Vector deltabeta = solution.get(0,v.getDim()-1);
						
			// Re-substitute and solve for lambda and deltal		
			Vector[] minusdeltal = new Vector[yhat.length]; 
			for (int i=0; i<yhat.length; i++) {
				Vector lambda = (invSigma[i].mult(A[i].mult(deltabeta))).minus(cw[i]);
				minusdeltal[i] = (Cll[i].mult(Cll_plus[i]).mult(l0[i].minus(l[i]))).plus(Cll[i].mult(B[i].trans().mult(lambda)).plus(G[i].trans().mult((G[i].mult(G[i].trans())).invert()).mult(f0[i])));
			}
			
			// Update unknwons and observations and compute weighted squared residuals
			v = v.plus(deltabeta);			
			ensureConstraints();
			double omega=0;
			for (int i=0; i<yhat.length; i++) {
				yhat[i].setObservations(l0[i].minus(minusdeltal[i]));
				yhat[i].ensureConstraints();
				yhat[i].changeNullspace();

				Vector deltal = yhat[i].getObservations().minus(l[i]);
				omega += deltal.scalarmult(Cll_plus[i].mult(deltal));
			}

			// Compute sigma0
			int redundance = W + H.getRowDimension() - v.getDim();
			if (redundance<0) throw new IllegalArgumentException("Redundance<0, not enough observations for estimating!");
			if ((redundance == 0) || (!estimate_sigma0))
				sigma2 = 1.0;
			else
				sigma2 = omega / redundance;

			// compute covariance matrix
			cov = Ninv.slice(0, v.getDim()-1, 0, v.getDim()-1);
			cov = cov.mult(sigma2);
			changeNullspace();
			
			// check, if iteration can be terminated			
			if (deltabeta.scalarmult(N.slice(0, v.getDim()-1, 0, v.getDim()-1).mult(deltabeta))/sigma2 < CHANGETHRESHOLD)
				stop=true;
		}
		
		// return some values
		no_of_iterations[0] = iteration;
		sigma0[0] = Math.sqrt(sigma2);
		return stop;
	}
	
	/** Iterations of a Gauss-Helmert model for estimating new
	 *  entities.
	 * @param rprops the (already conditioned) observations
	 * @param maxiter maximum number of iterations.
	 * @param yhaterg corrected observations, spherically normalized. yhat=y-ehat.
	 * @param sigma0 sigma0 of the last iterations (sqrt(sigma2)) as
	 *        an output parameter (array with 1 element)
	 * @param no_of_iterations number of iterations computed (output
	 *        parameter: array with 1 element)
	 * @return true if convergence reached otherwise false. Convergence
	 *         is not reached if the maximum number of iterations is
	 *         reached and the convergence criteria is not reached.
	 */
	public boolean gaussHelmertIterationsOld(
		RelationalProperty[] rprops,
		int maxiter,
		RelationalProperty[] yhaterg,
		double[] sigma0,
		int[] no_of_iterations,
		boolean estimate_sigma0) {

		// sum of DOFs from restrictions = sum of dim(wj)= sum of dim (cw) 
		// if wj is computed with reduced matrixes what is
		// the case here
		int W = 0;
		RelationalProperty[] yhat = new RelationalProperty[rprops.length];
		for (int i = 0; i < rprops.length; i++) {
			yhat[i] = (RelationalProperty) rprops[i].clone();
			W+=rprops[i].getDof();
		}

		// Gauss Helmert iteration
		boolean stop = false;
		int iteration;
		double sigma2 = 0;

		Vector[] cw = new Vector[rprops.length];

		Matrix[] invsigma = new Matrix[rprops.length];
		Matrix A[] = new Matrix[rprops.length];
		Matrix B[] = new Matrix[rprops.length];

		for (iteration = 0; !stop && iteration < maxiter; iteration++) {
			Vector ch = getConstraintValue();
			int constr_size = ch.getDim();
			int dim=v.getDim();
			Matrix N =
				LinearAlgebraFactory.getMatrix(
					dim + constr_size,
					dim + constr_size);
			Matrix Npart =
				LinearAlgebraFactory.getMatrix(dim, dim);

			Vector b =
				LinearAlgebraFactory.getVector(dim + constr_size);
			Vector bpart = LinearAlgebraFactory.getVector(dim);

			Matrix H = getJacobianOfConstraint();


			for (int j = 0; j < rprops.length; j++) {
				Matrix[] AB = getReducedAB(yhat[j]);
				A[j] = AB[0];
				B[j] = AB[1];		
//				System.out.println ("A"+j+" = "+A[j]);
//				System.out.println ("B"+j+" = "+B[j]);
			}

			//System.out.println("beta:");
			//v.printMatlab();

			
			// For all observations
			for (int j = 0; j < rprops.length; j++) {
				Vector y = yhat[j].getObservations();
				              				
				// cw has two possibilities:
				// the second one seems not to work 	
				//cw[j] =
				//	A[j].mult(v).plus(
				//	B[j].mult(rprops[j].getObservations().minus(y)));

				//cw[j] = B[j].mult(rprops[j].getObservations());
				cw[j] =
					A[j].mult(v).plus(
						B[j].mult(rprops[j].getObservations().minus(y)));

				// adjust cov of observations (SH)
				Matrix Syy = rprops[j].getReducedCovarianceOfObservations();

				invsigma[j] =  (B[j]
				                  .mult(Syy)
				                  .mult(B[j].trans())
								).invert();

				Npart = Npart.plus(A[j].trans().mult(invsigma[j]).mult(A[j]));
				bpart = bpart.minus(A[j].trans().mult(invsigma[j]).mult(cw[j]));

			}
			
			// scale restriction matrix, so that Npart and H have 
			// the same scale, so that N is well conditioned.
			
			// mean of diagonal elements of Npart
			double npartsum=0;
			for(int i=0; i<Npart.getRowDimension(); i++)
				npartsum+=Npart.get(i,i);
			npartsum/=Npart.getRowDimension();
			
			// mean of diagonal elements of H'*H
			Matrix HH=H.trans().mult(H);
			double hsum=0;
			for(int i=0; i<HH.getRowDimension(); i++)
				hsum+=HH.get(i,i);
			hsum/=HH.getRowDimension();
			
			// Now condition H and ch
			double hcondfactor=npartsum/hsum;
			//double hcondfactor=Npart.svd()[1].get(0,0)/HH.svd()[1].get(0,0);
//			System.out.println(hcondfactor);
			
			H=H.mult(hcondfactor);
			ch=ch.mult(hcondfactor);

			// 
			// Put together complete normal equation matrix^		
			N.slice(
				0,
				Npart.getColumnDimension() - 1,
				0,
				Npart.getColumnDimension() - 1,
				Npart);
			N.slice(
				Npart.getColumnDimension(),
				N.getColumnDimension() - 1,
				0,
				H.getColumnDimension() - 1,
				H);
			N.slice(
				0,
				H.getColumnDimension() - 1,
				Npart.getColumnDimension(),
				N.getColumnDimension() - 1,
				H.trans());
						

			// Put together righthand-side
			for (int i = 0; i < bpart.getDim(); i++)
				b.set(i, bpart.get(i));
			for (int i = 0; i < ch.getDim(); i++)
				b.set(i + bpart.getDim(), (-1.0) * ch.get(i));
			
			// Compute solution (deltabeta mue)
			Matrix Ninvert = N.invert();
			Vector solution = Ninvert.mult(b);

			// Get deltabeta out of solution
			Vector deltabeta = LinearAlgebraFactory.getVector(dim);
			for (int i = 0; i < deltabeta.getDim(); i++)
				deltabeta.set(i, solution.get(i));

			// Compute new unkown value
			v = v.plus(deltabeta);
			
			//v.printMatlab();
			
			//System.out.println("Deltabeta:"+deltabeta.normF());
			
			// Compute new corrected observations
			Vector[] lambda = new Vector[rprops.length];
			Vector[] ehat = new Vector[rprops.length];

			for (int j = 0; j < rprops.length; j++) {
				lambda[j] = invsigma[j].mult(cw[j].plus(A[j].mult(deltabeta)));
				ehat[j] =
					rprops[j].getReducedCovarianceOfObservations().mult(
									B[j].trans()).mult(
											lambda[j]);

				yhat[j].setObservations(
						rprops[j].getObservations().minus(ehat[j]));

//				((BiRelationalProperty)yhat[j]).getFirstEntity().changeNullspace();
			}

			double omega = 0;
			for (int j = 0; j < rprops.length; j++) {
				omega
					+= (
						ehat[j].getMatrix().trans().mult(B[j].trans()).mult(
							lambda[j])).get(
						0);
			}
			
			int redundance = W + H.getRowDimension() - v.getDim();
			if (redundance<0) throw new IllegalArgumentException("Redundance<0, not enough observations for estimating!");

			if ((redundance == 0) || (!estimate_sigma0))
				sigma2 = 1.0;
			else
				sigma2 = omega / redundance;
			
			// System.out.println("sigma 0: "+Math.sqrt(sigma2));

			// Covariance of unknowns
			//cov = Npart.pseudoInverse(getDegreesOfFreedom());
			//cov=Npart.invert();
			cov = Ninvert.slice(0, dim - 1, 0, dim - 1);
			cov = cov.mult(sigma2);
			// cov.print();
			
			// End criterion
			stop = true;
//			if(((deltabeta.getMatrix().
//					trans().
//					mult(cov.pseudoInverse(getDegreesOfFreedom()))). 
//					mult(deltabeta)).
//					get(0) //puts out the skalar value
//						> deltabeta.getDim()*CHANGETHRESHOLD)
//				stop = false;
			for (int i = 0; i < deltabeta.getDim(); i++){
				if(cov.get(i,i) == 0 || cov.get(i,i) == -0) continue;
				if (deltabeta.get(i) * deltabeta.get(i) / cov.get(i, i)
						> CHANGETHRESHOLD)
					stop = false;
			}
			
			if(!stop){
				// Applying the constraints on the observations
				for (int j = 0; j < rprops.length; j++) {
					if (yhat[j] instanceof BiRelationalProperty) {
						((BiRelationalProperty) yhat[j]).getFirstEntity().ensureConstraintsWithoutIteration();
						((BiRelationalProperty) yhat[j]).getFirstEntity().changeNullspace(EPSILON);
					}						
					else if (yhat[j] instanceof TriRelationalProperty) {
						((TriRelationalProperty) yhat[j]).getFirstEntity().ensureConstraintsWithoutIteration();
						((TriRelationalProperty) yhat[j]).getFirstEntity().changeNullspace(EPSILON);
						((TriRelationalProperty) yhat[j]).getSecondEntity().ensureConstraintsWithoutIteration();
						((TriRelationalProperty) yhat[j]).getSecondEntity().changeNullspace(EPSILON);
					}
				}
			}
		}
		
		for (int i = 0; i < rprops.length; i++)
			yhaterg[i] = yhat[i];

		no_of_iterations[0] = iteration;
		//System.out.println ("Iteration: "+iteration);
		//System.out.println ("Sigma0: "+Math.sqrt(sigma2));
		sigma0[0] = Math.sqrt(sigma2);
		return stop;
	}
	
	/** Get the value of the constraint (h(beta) or g(y))  if this entity 
	 * is an unknown or an observation.
	 * This is the length constraint and for 3D-Lines also the
	 * plcker condition. Here is the default implementation for non 3D-Lines.
	 * @return the value of the constraint as a vector.
	 */
	protected Vector getConstraintValue() {
		Vector vhelp = LinearAlgebraFactory.getVector(1);
		vhelp.set(0, 0.5 * (v.scalarmult(v) - 1.0));
		return vhelp;
	}
	
	/** Get the reduced Matrix A and B from a relational 
	 * property. A is the Jacobian of this Entity (the unknown),
	 * B is the Jacobian of the observation (the relational property).
	 * The Jacobian of the oberservation depends (only) on the unknown and
	 * the Jacobian of the unknown depends (only) of the oberservation !
	 * 
	 * This is the default implementation.
	 * 
	 * It is overwritten for Transformations 
 	 * and is overwritten in Line_3D to handle the special
	 * case Equal(Line_3D, Line_3D) !
	 * 
	 * @param r the relational Property
	 * @return A in array element 0, B in array Element 1.
	 */
	protected Matrix[] getReducedAB(RelationalProperty r) {
			Matrix A = r.getJacobian();
			Matrix B = getJacobianOfRelation(r);
			Matrix[] AB = reduceMatrix(A, B, r.getDof());
			return AB;
	}

	/** Reduce a Jacobian matrix. Only the rows with the largest
	 * norm are used for the reduced matrix.
	 * @param A the matrix to reduce
	 * @param dof the degrees of freedom=rank=number of rows of the
	 *        reduced matrix.
	 * @return the reduced matrix.
	 */
	public static Matrix reduceMatrix(Matrix A, int dof) {
		Vector[] rows = new Vector[A.getRowDimension()];
		for (int i = 0; i < A.getRowDimension(); i++)
			rows[i] = A.getRow(i);
		java.util.Arrays.sort(rows, new java.util.Comparator() {
			public int compare(Object o1, Object o2) {
				double n1 = ((Vector) o1).normM();
				double n2 = ((Vector) o2).normM();
				if (n1 > n2)
					return -1;
				else if (n1 < n2)
					return 1;
				else
					return 0;
			}
		});
		Matrix reducedA =
			LinearAlgebraFactory.getMatrix(dof, A.getColumnDimension());
		for (int i = 0; i < dof; i++)
			reducedA.setRow(i, rows[i]);
		return reducedA;
	}

	/** Reduce two Jacobian matrixes consistently. Only the rows with the largest
	 * norms |ui|*|vi| are used for the reduced matrix.
	 * @param A the first matrix to reduce
	 * @param B the second matrix to reduce
	 * @param dof the degrees of freedom=rank=number of rows of the
	 *        reduced matrixes.
	 * @return the reduced matrixes. Element 0 = A, Element 1 = B
	 */
	public static Matrix[] reduceMatrix(Matrix A, Matrix B, int dof) {
		Vector[][] rows = new Vector[A.getRowDimension()][];
		for (int i = 0; i < A.getRowDimension(); i++) {
			rows[i] = new Vector[2];
			rows[i][0] = A.getRow(i);
			rows[i][1] = B.getRow(i);
		}
		java.util.Arrays.sort(rows, new java.util.Comparator() {
			public int compare(Object o1, Object o2) {
				Vector a1 = ((Vector[]) o1)[0];
				Vector b1 = ((Vector[]) o1)[1];
				Vector a2 = ((Vector[]) o2)[0];
				Vector b2 = ((Vector[]) o2)[1];
				double n1 = a1.normM() * b1.normM();
				double n2 = a2.normM() * b2.normM();
				if (n1 > n2)
					return -1;
				else if (n1 < n2)
					return 1;
				else
					return 0;
			}
		});
		Matrix reducedA =
			LinearAlgebraFactory.getMatrix(dof, A.getColumnDimension());
		Matrix reducedB =
			LinearAlgebraFactory.getMatrix(dof, B.getColumnDimension());
		for (int i = 0; i < dof; i++) {
			reducedA.setRow(i, rows[i][0]);
			reducedB.setRow(i, rows[i][1]);
		}
		Matrix[] result = new Matrix[2];
		result[0] = reducedA;
		result[1] = reducedB;
		return result;
	}
	
	/** Reduce two Jacobian matrixes consistently based on solely the first matrix A. Only the rows with the largest
	 * norms |ui| are used for the reduced matrix.
	 * @param A the matrix to reduce
	 * @param dof the degrees of freedom=rank=number of rows of the
	 *        reduced matrixes.
	 * @return the reduced matrixes. Element 0 = A, Element 1 = B
	 */
	public static Matrix[] reduceFirstMatrix(Matrix A, Matrix B, int dof) {
		Vector[][] rows = new Vector[A.getRowDimension()][];
		for (int i = 0; i < A.getRowDimension(); i++) {
			rows[i] = new Vector[2];
			rows[i][0] = A.getRow(i);
			rows[i][1] = B.getRow(i);
		}
//		java.util.Arrays.sort(rows, new java.util.Comparator() {
//			public int compare(Object o1, Object o2) {
//				Vector a1 = ((Vector[]) o1)[0];
//				Vector a2 = ((Vector[]) o2)[0];
//				double n1 = a1.normM();
//				double n2 = a2.normM();
//				if (n1 > n2)
//					return -1;
//				else if (n1 < n2)
//					return 1;
//				else
//					return 0;
//			}
//		});
		Matrix reducedA =
			LinearAlgebraFactory.getMatrix(dof, A.getColumnDimension());
		Matrix reducedB =
			LinearAlgebraFactory.getMatrix(dof, B.getColumnDimension());
		for (int i = 0; i < dof; i++) {
			reducedA.setRow(i, rows[i][0]);
			reducedB.setRow(i, rows[i][1]);
		}
		Matrix[] result = new Matrix[2];
		result[0] = reducedA;
		result[1] = reducedB;
		return result;
	}
	
	/**
	 * Return Jacobian Matrix in combination with a 
	 * relation. This is the Jacobian after the observations (given
	 * in the relational property). *this plays the role of the unknown. 
	 * @param r a relational Property. 
	 * @return Jacobian matrix (This is a part of the constraint formula)
	 */
	public abstract Matrix getJacobianOfRelation(RelationalProperty r);



	/** Get the Jacobian of the constraint (H, G) if this entity 
	 *  is an unkown or an observation. This implementation is the 
	 *  Jacobian of the default constraint
	 *  0.5(v*vt-1)=0
	 * 
	 * @return the Jacobian as a matrix.
	 */
	public Matrix getJacobianOfConstraint() {
		return getV().getMatrix().trans();
	}


	/** Default maximal number of iterations for Gauss-Helmert iteration */
	protected static final int MAXITER = 2000;

	/** Factor between unknown and its accuracy for stopping iteration */
	private static final double CHANGETHRESHOLD = 0.0001;

	/** Variance in direction of Nullspace. This only depends on the curvature of the space in the estimation. */
	private static final double EPSILON = 1e-6;

	/**
	 * starts the error propagation of the uncertainty elements
	 * 
	 * @param U_x The first jacobian between element 1 and 2
	 * @param Sigma_x the covariance matrix of element 1
	 * @param V_y the second jacobian between element 2 and 1
	 * @param Sigma_y the covariance matrix of element 2
	 * @return the calculated covariance matrix of the new element
	 */
	protected static Matrix calcCov(Matrix U_x, Matrix Sigma_x, Matrix V_y, Matrix Sigma_y) {        
		Matrix tmp = U_x.mult(Sigma_y).mult(U_x.trans());
		tmp = tmp.plus   ( V_y.mult(Sigma_x).mult(V_y.trans()) );
		return tmp;
	} // end calcCov
	
	/** Change the entity (covariance not changed) in that way,
	 * that all constraints (given by getConstraintValue) are
	 * fulfilled. This is done by a Gauss-Helmert-Iterations without
	 * unknowns. Attention: A covariance matrix must be set !
	 * TODO: Check why G*cov*Gt sometimes gets singular
	 */
	public void ensureConstraints() {
		for (int iteration2 = 0; iteration2 < MAXITER2; iteration2++) {
			Matrix G = getJacobianOfConstraint();
			if (getConstraintValue().getLength() < CONSTRAINTZEROTHRESHOLD)
				break;
				v =
					v.minus(
						G.trans().mult(
							(G.mult(G.trans())).invert()).mult(
							getConstraintValue()));				
		}
	}
	
	/** Maximal number of iterations for applying constraints*/
	private static final int MAXITER2 = 200;
	
	/** Function ensureConstraints: Threshold for the constraint value to be zero. The iteration
	 *  is stopped if this threshold has been reached. */
	private static final double CONSTRAINTZEROTHRESHOLD = 1e-12;
	

} // end Element
