/*
 * Created on 13.03.2003
 */
package sugr;

import sugr.linalg.*;
/**
 * This matrix contains Translation (3 parameters) and Rotation(3 parameters) 
 * in 3D just as needed
 * for exterior orientation, for example.
 * @author Norbert Fischer
 *
 */
public class Motion_3D extends ScaledMotion_3D
{
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * standard constructor. Creates an identity matrix
	 *
	 */
	public Motion_3D()	{}
	
	/**
	 * creates a motion by a given Translation_3D and a Rotation_3D
	 * @param trans   contains the Translation-matrix
	 * @param rot   contains the rotation-matrix
	 */
	public Motion_3D (Translation_3D trans, Rotation_3D rot)
	{
		this(trans, rot, null);
	}

	/**
	 * creates a motion with covariance by a given Translation_3D 
	 * and a Rotation_3D
	 * @param trans   contains the Translation-matrix
	 * @param rot   contains the rotation-matrix
	 * @param cov   16x16 matrix
	 */
	public Motion_3D (Translation_3D trans, Rotation_3D rot,
					  Matrix cov)
	{
		setV(rot.getTransformationMatrix().
		  mult(trans.getTransformationMatrix()).vec());
		
		//FFG
		if (cov==null)	
		{
			setCov(errorProp(rot, trans));
		}
		//if not null, then cov must be given by user und therefore be checked ;-)
		else
		{
			checkCovDim(cov, DIM*DIM);
			setCov(cov);
		}
	}

	/**
	 * Returns the number of elements to specify this transformation
	 */
	public int getDegreesOfFreedom()
	{
		return 6;
	}	
	
	/** Estimation of that entity out of given relational properties.
	 *  Overwritten method of Element, because an estimation of
	 *  a Motion needs a special conditioning. The two sets of
	 *  Entities must be conditioned with the same factor to avoid
	 *  a scale in the transformation !
	  * @param rprops the observations
	  * @param doConditioning if true, a conditioning is done.
	  * @param maxiter maximum number of iterations.
	  * @param yhat corrected observations, spherically normalized. yhat=y-ehat.
	  * @param sigma0 sigma0 of the last iterations (sqrt(sigma2)) as
	  *        an output parameter (array with 1 element)
	  * @param no_of_iterations number of iterations computed (output
	  *        parameter: array with 1 element)
	  * @return true if convergence reached otherwise false. Convergence
	  *         is not reached if the maximum number of iterations is
	  *         reached and the convergence criteria is not reached.
	  */
	public boolean estimate(
		RelationalProperty[] rprops,
		boolean doConditioning,
		int maxiter,
		RelationalProperty[] yhat,
		double[] sigma0,
		int[] no_of_iterations,
		boolean estimate_sigma0) {

		RelationalProperty[] rpropscond = new RelationalProperty[rprops.length];
		for (int i = 0; i < rprops.length; i++)
			rpropscond[i] = (RelationalProperty) rprops[i].clone();

		double[] f= new double[1];

		if (doConditioning) {
				f = TriRelationalProperty.doConditionOneFactor(rpropscond);
		}

		// normalize spherecally (this also changes Nullspace !)
		for (int i = 0; i < rprops.length; i++) {
			if (rpropscond[i] instanceof BiRelationalProperty)
				((BiRelationalProperty) rpropscond[i])
					.getFirstEntity()
					.normalizeSphere();
			if (rpropscond[i] instanceof TriRelationalProperty) {
				((TriRelationalProperty) rpropscond[i])
					.getFirstEntity()
					.normalizeSphere();
				((TriRelationalProperty) rpropscond[i])
					.getSecondEntity()
					.normalizeSphere();
			}
		}

		// Get initial values for the estimation.
		approximate(rpropscond);

		// and now do iterations
		boolean reachedconvergence =
			gaussHelmertIterations(
				rpropscond,
				maxiter,
				yhat,
				sigma0,
				no_of_iterations,
				estimate_sigma0);

		// reconditioning
		if (doConditioning) {
			boolean neccessary = false;
			double[] ref = new double[f.length];
			for (int i = 0; i < ref.length; i++) {
				ref[i] = 1.0 / f[i];
				if (f[i] != 1)
					neccessary = true;
			}

			if (neccessary) {
				// Recondition estimate
				doCondition(ref);

				// Reconditiong of corrected observations
				if (rprops[0] instanceof TriRelationalProperty)
					TriRelationalProperty.doCondition(yhat, ref);

				else
					BiRelationalProperty.doCondition(yhat, ref);
			}
		}

		return reachedconvergence;
	}

	/** Get the value of the constraint (h(beta) or g(y))  if this entity 
	 * is an unknown or an observation.
	 * This is, that the last row is 0 0 0 x ,
	 * all rows are orthogonal to each other,
	 * the first three rows have length 1,
	 * and the whole vector has length 1
	 * @return the value of the constraint as a vector.
	 */
	protected Vector getConstraintValue() {
		Vector vhelp = LinearAlgebraFactory.getVector(10);
		Matrix trans = this.getTransformationMatrix();
		
		//zero entries
		vhelp.set(0, trans.get(3,0));
		vhelp.set(1, trans.get(3,1));
		vhelp.set(2, trans.get(3,2));
		
		//orthogonal constraint
		trans.set(3,0,0);
		trans.set(3,1,0);
		trans.set(3,2,0);
		vhelp.set(3,trans.getColumn(0)
				.scalarmult(trans.getColumn(1)));
		vhelp.set(4,trans.getColumn(1)
				.scalarmult(trans.getColumn(2)));
		vhelp.set(5,trans.getColumn(0)
				.scalarmult(trans.getColumn(2)));
		
		//length constraint
		double s = trans.get(3,3);
		s *= s;
		vhelp.set(6,0.5*(trans.getColumn(0)
				.scalarmult(trans.getColumn(0))-s));
		vhelp.set(7,0.5*(trans.getColumn(1)
				.scalarmult(trans.getColumn(1))-s));
		vhelp.set(8,0.5*(trans.getColumn(2)
				.scalarmult(trans.getColumn(2))-s));

		//length constraint for the whole vector
		vhelp.set(9,0.5*(v.scalarmult(v) - 1));
		
		return vhelp;
	}
	
	public void printConstraint(){
		System.out.println("Constraint: " + getConstraintValue());
	}
	
	/** Get the Jacobian of the constraint (H, G) if this entity 
	 *  is an unkown or an observation. 
	 * 
	 * @return the Jacobian as a matrix.
	 */
	public Matrix getJacobianOfConstraint() {
		//get Matrix filled with zeros
		Matrix jac = LinearAlgebraFactory.getMatrix(10,16);
		
		//zero entries, (identity)' = 1
		jac.set(0,3,1);
		jac.set(1,7,1);
		jac.set(2,11,1);
		
		//orthogonal constraint,
		//(0 * 4 + 1 * 5 + 2 * 6)
		jac.set(3,0,v.get(4));
		jac.set(3,1,v.get(5));
		jac.set(3,2,v.get(6));
		jac.set(3,4,v.get(0));
		jac.set(3,5,v.get(1));
		jac.set(3,6,v.get(2));
		//(4 * 8 + 5 * 9 + 6 * 10)
		jac.set(4,4,v.get(8));
		jac.set(4,5,v.get(9));
		jac.set(4,6,v.get(10));
		jac.set(4,8,v.get(4));
		jac.set(4,9,v.get(5));
		jac.set(4,10,v.get(6));
		//(0 * 8 + 1 * 9 + 2 * 10)
		jac.set(5,0,v.get(8));
		jac.set(5,1,v.get(9));
		jac.set(5,2,v.get(10));
		jac.set(5,8,v.get(0));
		jac.set(5,9,v.get(1));
		jac.set(5,10,v.get(2));
		
		//length constraint for three columns
		jac.set(6,0,v.get(0));
		jac.set(6,1,v.get(1));
		jac.set(6,2,v.get(2));
		jac.set(6,15,-v.get(15));
		//second column
		jac.set(7,4,v.get(4));
		jac.set(7,5,v.get(5));
		jac.set(7,6,v.get(6));
		jac.set(7,15,-v.get(15));
		//third column
		jac.set(8,8,v.get(8));
		jac.set(8,9,v.get(9));
		jac.set(8,10,v.get(10));
		jac.set(8,15,-v.get(15));
		
		//length constraint for whole vector
		jac.setRow(9,v);

		return jac;
	}
}
