/** Java class "ProjectiveCamera.java" generated from Poseidon for UML. 
 *  Poseidon for UML is developed by <A HREF="http://www.gentleware.com">Gentleware</A>.
 *  Generated with <A HREF="http://jakarta.apache.org/velocity/">velocity</A> template engine.
 */
package sugr;

import sugr.linalg.*;
import java.io.*;

/**
 * This class represents a projective camera. This camera consists of:
 * <ul>
 * 		<li> camera constant c </li> 
 * 		<li> shear s </li>
 * 		<li> scale m </li>
 * 		<li> principal point pp  (2 parameters)</li><br>
 * 		<li> Translation T  (3 parameters)</li>
 * 		<li> Rotation R  (3 parameters)</li>
 * </ul>
 * 
 * Internel the camera parameters are stored as a projection matrix P.
 * Like every transformation, P is stored as stacked columns in vector v.
 * 
 * Translation and Rotation are known as exterior orientation (camera to world-
 * coordinate-system), the parameters above specify inner orientation of the camera.
 * Both types together gives the projection matrix (3x4), to project a 3D-Point 
 * into a 2D-Point.
 * <pre>
 *         | c   cs   ppX |     | r11 r12 r13 t1 |         | p11 p12 p13 p14 |      
 *         | 0 c(1+m) ppY |  *  | r21 r22 r23 t2 |    =    | p21 p22 p23 p24 |
 *         | 0    0    1  |     | r31 r32 r33 t3 |         | p31 p32 p33 p34 |
 *
 *           inner orient.        exterior orient.          projection matrix
 *              (3x3)                 (3x4)                      (3x4)
 * </pre>
 * If you use getCov(), you get covariance for Point-Transformations. If you like
 * to transform 3D-Lines, you can get this transformation-matrix via getLineMatrix()
 * and its covariance via getLineMatrixCov().
 * @author Norbert Fischer
 */
public class ProjectiveCamera extends Transformation {
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;
	
	
	private static final int DIM = 12;
	/** Number of rows of projection matrix */
	private static final int DIMR = 3;
	/** Number of cols of projection matrix */
	private static final int DIMC = 4;

	private static final int DOF = 11;

	/** Stores the Point-Projection and Line-Projection matrices (for speed-optimisation only)
	 */
	private Matrix P, Q, Qdual;

	///////////////////////////////////////
	// operations

	/**
	 * Default constructor
	 */
	public ProjectiveCamera() {
		setV(LinearAlgebraFactory.getVector(DIM));
		setCov(LinearAlgebraFactory.getMatrix(DIM, DIM));

		calcProjectionMatrices();
	} // end ProjectiveCamera        

	/** Creates a ProjectiveCamera from a 3x4 point projection matrix
	 * @param pointMatrix The 3x4 point projection matrix 
	 */
	public ProjectiveCamera(Matrix pointMatrix) {
		Vector vec = LinearAlgebraFactory.getVector(DIM);

		for (int i = 0; i < 3; i++)
			for (int j = 0; j < 4; j++)
				vec.set(j * DIMR + i, pointMatrix.get(i, j));
		setV(vec);

		Matrix cov = LinearAlgebraFactory.getMatrix(DIM, DIM);
		setCov(cov);

		calcProjectionMatrices();
	}

	/** creates a PojectiveCamera from the 12 entries of the point projection matrix:
	 * <pre>
	 *    | p11  p12 p13 p14 |
	 *    | p21  p22 p23 p24 |
	 *    | p31  p32 p33 p34 |
	 * </pre>
	 * 
	 * @param p11 Entry of the 3x4 point projection matrix
	 * @param p12 Entry of the 3x4 point projection matrix
	 * @param p13 Entry of the 3x4 point projection matrix
	 * @param p14 Entry of the 3x4 point projection matrix
	 * @param p21 Entry of the 3x4 point projection matrix
	 * @param p22 Entry of the 3x4 point projection matrix
	 * @param p23 Entry of the 3x4 point projection matrix
	 * @param p24 Entry of the 3x4 point projection matrix
	 * @param p31 Entry of the 3x4 point projection matrix
	 * @param p32 Entry of the 3x4 point projection matrix
	 * @param p33 Entry of the 3x4 point projection matrix
	 * @param p34 Entry of the 3x4 point projection matrix
	 */
	public ProjectiveCamera(
		double p11,
		double p12,
		double p13,
		double p14,
		double p21,
		double p22,
		double p23,
		double p24,
		double p31,
		double p32,
		double p33,
		double p34) {
		Vector vec = LinearAlgebraFactory.getVector(DIM);

		vec.set(0, p11);
		vec.set(1, p21);
		vec.set(2, p31);
		vec.set(3, p12);
		vec.set(4, p22);
		vec.set(5, p32);
		vec.set(6, p13);
		vec.set(7, p23);
		vec.set(8, p33);
		vec.set(9, p14);
		vec.set(10, p24);
		vec.set(11, p34);

		setV(vec);

		Matrix cov = LinearAlgebraFactory.getMatrix(DIM, DIM);
		setCov(cov);

		calcProjectionMatrices();
	}

	/** 
	 * Reads a projection matrix from a Reader. The purpose of this constructor 
	 * is to read the camrea parameters from an input file.
	 * 
	 * @param r  stream Reader, that yields 12 double values for the point 
	 * 		  projection matrix
	 */
	public ProjectiveCamera(Reader r) throws IOException {
		int index = 0;
		double[] cam = new double[DIM];
		BufferedReader in = new BufferedReader(r);
		String str;
		while ((str = in.readLine()) != null) {
			str = str.trim();
			String[] sa = str.split("\\s+");
			for (int i = 0; i < sa.length; i++) {
				cam[index] = Double.valueOf(sa[i]).doubleValue();
				index++;
				if (index == DIM)
					break;
			}
			if (index == DIM)
				break;
		}
		index=0;
		double[] Sigma = new double[DIM*DIM];
		while ((str = in.readLine()) != null) {
			str = str.trim();
			String[] sa = str.split("\\s+");
			for (int i = 0; i < sa.length; i++) {
				Sigma[index] = Double.valueOf(sa[i]).doubleValue();
				index++;
				if (index == DIM*DIM)
					break;
			}
			if (index == DIM*DIM)
				break;
		}
		in.close();
	
		Vector vec = LinearAlgebraFactory.getVector(DIM);
		vec.set(0, cam[0]);
		vec.set(1, cam[4]);
		vec.set(2, cam[8]);
		vec.set(3, cam[1]);
		vec.set(4, cam[5]);
		vec.set(5, cam[9]);
		vec.set(6, cam[2]);
		vec.set(7, cam[6]);
		vec.set(8, cam[10]);
		vec.set(9, cam[3]);
		vec.set(10,cam[7]);
		vec.set(11,cam[11]);

		setV(vec);

		Matrix cov = LinearAlgebraFactory.getMatrix(DIM, DIM);
		for (int i=0; i<DIM; i++)
			for (int j=0; j<DIM; j++) {
				cov.set(i,j,Sigma[i*DIM+j]);
			}
		setCov(cov);
		
		calcProjectionMatrices();
	}

	/**
	 * Creates a ProjectiveCamera by setting inner and exterior orientation. 
	 * Matrices should have this form:
	 * <pre>
	 *         | c   cs   ppX |     | r11 r12 r13 t1 |
	 *         | 0 c(1+m) ppY |  *  | r21 r22 r23 t2 | 
	 *         | 0    0    1  |     | r31 r32 r33 t3 | 
	 *
	 *           inner orient.        exterior orient. 
	 *              (3x3)                 (3x4)        
	 * </pre>
	 * 
	 * @param inOrien 3x3-matrix
	 * @param extOrien 3x4 matrix
	 */
	public ProjectiveCamera(Matrix inOrien, Matrix extOrien) {
		this(inOrien, extOrien, LinearAlgebraFactory.getMatrix(DIM, DIM));
	}

	/**
	 * Creates a ProjectiveCamera by setting inner and exterior orientation with
	 * covariance. Matrices should have this form:
	 * <pre>
	 *         | c   cs   ppX |     | r11 r12 r13 t1 |
	 *         | 0 c(1+m) ppY |  *  | r21 r22 r23 t2 | 
	 *         | 0    0    1  |     | r31 r32 r33 t3 | 
	 *
	 *           inner orient.        exterior orient. 
	 *              (3x3)                 (3x4)        
	 * </pre>
	 * 
	 * @param inOrien   3x3-matrix
	 * @param extOrien   3x4 matrix
	 * @param cov   12x12 Matrix
	 */
	public ProjectiveCamera(Matrix inOrien, Matrix extOrien, Matrix cov) {
		//check matrix dimensions
		checkCovDim(inOrien, 3);
		checkCovDim(cov, DIM);
		if (extOrien.getRowDimension() != 3
			|| extOrien.getColumnDimension() != 4) {
			new IllegalArgumentException(
				"Exterior orientation matrix should" + "have dimensions: 3x4");
		}

		//multiply inOrien and extOrient to gain projective matrix
		setV(inOrien.mult(extOrien).vec());
		setCov(cov);

		calcProjectionMatrices();
	}

	/**
	 * Creates a ProjectiveCamera by setting exterior orientation. Inner orientation
	 * is set to identity
	 * 
	 * @param extOrien   a Motion_3D
	 */
	public ProjectiveCamera(Motion_3D extOrien) {
		this(extOrien, LinearAlgebraFactory.getMatrix(DIM, DIM));
	}

	/**
	 * Creates a ProjectiveCamera by setting exterior orientation with covariance. 
	 * Inner orientation is set to identity
	 * 
	 * @param extOrien   a Motion_3D
	 * @param cov    12x12 Matrix
	 */
	public ProjectiveCamera(Motion_3D extOrien, Matrix cov) {
		this(LinearAlgebraFactory.getMatrixId(3), extOrien, cov);
	}

	/**
	 * Creates a ProjectiveCamera by setting inner and exterior orientation. Matrix 
	 * should have this form:
	 * <pre>
	 *         | c   cs   ppX |        
	 *         | 0 c(1+m) ppY |  
	 *         | 0    0    1  |  
	 * </pre>
	 * Error propagation is done automatically
	 * 
	 * @param inOrien	3x3 Matrix
	 * @param extOrien	a Motion_3D
	 */
	public ProjectiveCamera(Matrix inOrien, Motion_3D extOrien) {
		this(inOrien, extOrien, LinearAlgebraFactory.getMatrix(DIM, DIM));
	}

	/**
	 * Creates a ProjectiveCamera by setting inner and exterior orientation. Matrix
	 * should have this form:
	 * <pre>
	 *         | c   cs   ppX |        
	 *         | 0 c(1+m) ppY |  
	 *         | 0    0    1  |  
	 * </pre>
	 * 
	 * @param inOrien	3x3 Matrix
	 * @param extOrien	a Motion_3D
	 * @param cov	12x12 Matrix
	 */
	public ProjectiveCamera(Matrix inOrien, Motion_3D extOrien, Matrix cov) {
		//check dimensions, set covariance
		checkCovDim(inOrien, 3);
		checkCovDim(cov, DIM);
		setCov(cov);

		//3x3 Matrix multiplicated with 3x4-Matrix (z-projection is thrown away
		setV(inOrien.mult(extOrien.getTransformationMatrix().delRow(3)).vec());

		calcProjectionMatrices();
	}

	/**
	 * Sets a ProjectiveCamera by setting inner orientation parameters directly.
	 * Exterior orientation is set to identity
	 * 
	 * @param c  		camera constant
	 * @param shear		as name says
	 * @param scale		as name says
	 * @param ppX		principal point x-axis
	 * @param ppY		principal point y-axis
	 */
	public ProjectiveCamera(
		double c,
		double shear,
		double scale,
		double ppX,
		double ppY) {
		this(
			c,
			shear,
			scale,
			ppX,
			ppY,
			LinearAlgebraFactory.getMatrix(DIM, DIM));
	}

	/**
	 * Sets a ProjectiveCamera with covariance by setting inner orientation 
	 * parameters directly.
	 * Exterior orientation is set to identity
	 * 
	 * @param c  		camera constant
	 * @param shear		as name says
	 * @param scale		as name says
	 * @param ppX		principal point x-axis
	 * @param ppY		principal point y-axis
	 * @param cov		12x12 Matrix
	 */
	public ProjectiveCamera(
		double c,
		double shear,
		double scale,
		double ppX,
		double ppY,
		Matrix cov) {
		checkCovDim(cov, DIM);
		setCov(cov);

		Matrix inOrien = LinearAlgebraFactory.getMatrixId(3);

		inOrien.set(0, 0, c);
		inOrien.set(0, 1, c * shear);
		inOrien.set(0, 2, ppX);

		inOrien.set(1, 1, c * (1 + scale));
		inOrien.set(1, 2, ppY);

		setV(
			inOrien
				.mult(LinearAlgebraFactory.getMatrixId(4).delRow(3))
				.vec());

		calcProjectionMatrices();
	}

	/**
	 * Creates a ProjectiveCamera by setting exterior orientation. Inner orientation
	 * is set to identity
	 * 
	 * @param trans		a Translation_3D
	 * @param rot		a Rotation_3D
	 */
	public ProjectiveCamera(Translation_3D trans, Rotation_3D rot) {
		this(new Motion_3D(trans, rot));
	}
	/**
	 * Creates a ProjectiveCamera by setting exterior orientation. Inner orientation
	 * is set to identity
	 * 
	 * @param trans		a Translation_3D
	 * @param rot		a Rotation_3D
	 * @param cov		12x12 Matrix
	 */
	public ProjectiveCamera(
		Translation_3D trans,
		Rotation_3D rot,
		Matrix cov) {
		this(new Motion_3D(trans, rot), cov);
	}

	/**
	 * Sets a ProjectiveCamera by setting whole projection matrix. It's built by:
	 * <pre>
	 *    | a[0] a[1] a[ 2] a[ 3] |
	 *    | a[4] a[5] a[ 6] a[ 7] |
	 *    | a[8] a[9] a[10] a[11] |
	 * </pre> 
	 * TODO: same row-column index order in homograpy and projective camera
	 * @param cam	double-Array with 12 entries
	 */
	public ProjectiveCamera(double cam[]) {
		this(cam, LinearAlgebraFactory.getMatrix(DIM, DIM));
	}

	/**
	 * Sets a ProjectiveCamera by setting whole projection matrix. It's build by:
	 * <pre>
	 *    | a[0] a[1] a[ 2] a[ 3] |
	 *    | a[4] a[5] a[ 6] a[ 7] |
	 *    | a[8] a[9] a[10] a[11] |
	 * </pre> 
	 * Covariance is also used here.
	 * 
	 * @param cam	double-Array with 12 entries
	 * @param cov	12x12 Matrix
	 */
	public ProjectiveCamera(double cam[], Matrix cov) {
		if (cam.length!=12) throw new IllegalArgumentException("Camera parameter array has not 12 Elements!");
		
		checkCovDim(cov, DIM);
		setCov(cov);

		Vector vec = LinearAlgebraFactory.getVector(DIM);
		vec.set(0, cam[0]);
		vec.set(1, cam[4]);
		vec.set(2, cam[8]);
		vec.set(3, cam[1]);
		vec.set(4, cam[5]);
		vec.set(5, cam[9]);
		vec.set(6, cam[2]);
		vec.set(7, cam[6]);
		vec.set(8, cam[10]);
		vec.set(9, cam[3]);
		vec.set(10,cam[7]);
		vec.set(11,cam[11]);

		setV(vec);

		calcProjectionMatrices();
	}

	/**
	 * Sets a ProjectiveCamera by only defining camera constant. Exterior orientation
	 * is set to identity.
	 * 
	 * @param c		camera constant
	 */
	public ProjectiveCamera(double c) {
		this(c, LinearAlgebraFactory.getMatrix(DIM, DIM));
	}

	/**
	 * Sets a ProjectiveCamera with covariance by only defining camera constant.
	 * (ideal camera) <br>
	 * Exterior orientation is set to identity.
	 *  
	 * @param c		camera constant
	 * @param cov	12x12 Matrix
	 */
	public ProjectiveCamera(double c, Matrix cov) {
		this(c, 0, 0, 0, 0, cov);
	}

	/**
	 * Sets a ProjectiveCamera by camera constant and principal point (camera with
	 * euclidean sensor) <br> 
	 * Exterior orientation is set to identity.
	 *  
	 * @param c		camera constant
	 * @param ppX	principal point x-axis
	 * @param ppY	principal point y-axis
	 */
	public ProjectiveCamera(double c, double ppX, double ppY) {
		this(c, 0, 0, ppX, ppY);
	}

	/**
	 * Sets a ProjectiveCamera by camera constant and principal point (camera with
	 * euclidean sensor) <br> 
	 * Exterior orientation is set to identity.
	 *  
	 * @param c		camera constant
	 * @param ppX	principal point x-axis
	 * @param ppY	principal point y-axis
	 * @param cov	12x12 Matrix
	 */
	public ProjectiveCamera(double c, double ppX, double ppY, Matrix cov) {
		this(c, 0, 0, ppX, ppY, cov);
	}

	/**
	 * Sets a ProjectiveCamera by setting whole projection matrix. It's build by:
	 * <pre>
	 *    | p11  p12 p13 p14 |
	 *    | p21  p22 p23 p24 |
	 *    | p31  p32 p33 p34 |
	 * </pre>
	 * Covariance is also used here. 
	 * 
	 * @param cov	12x12 Matrix
	 */
	public ProjectiveCamera(
		double p11,
		double p12,
		double p13,
		double p14,
		double p21,
		double p22,
		double p23,
		double p24,
		double p31,
		double p32,
		double p33,
		double p34,
		Matrix cov) {
		checkCovDim(cov, DIM);
		setCov(cov);

		Vector vec = LinearAlgebraFactory.getVector(DIM);
		vec.set(0, p11);
		vec.set(1, p21);
		vec.set(2, p31);
		vec.set(3, p12);
		vec.set(4, p22);
		vec.set(5, p32);
		vec.set(6, p13);
		vec.set(7, p23);
		vec.set(8, p33);
		vec.set(9, p14);
		vec.set(10, p24);
		vec.set(11, p34);

		setV(vec);

		calcProjectionMatrices();
	}

	/** Swaps the first two lines of the transformation matrix
	 *  This is needed to support RC- and XY-coordinate-systems
	 */
	public void swapRC() {
		int[] xxx = {1,0,2,4,3,5,7,6,8,10,9,11};
		Vector vv = LinearAlgebraFactory.getVector(12);
		for (int i=0; i<xxx.length; i++)
			vv.set(i,v.get(xxx[i]));
		v = vv;
		cov = cov.slice(xxx,xxx);
		
		calcProjectionMatrices();
	}
	
	/** Get the number of rows of the transformation matrix.
	 * @return the number of rows of the transformation matrix.
	 */
	public int getDimRow() {
		return DIMR;
	}

	/** Get the number of columns of the transformation matrix.
	 * @return the number of columns of the transformation matrix.
	 */
	public int getDimCol() {
		return DIMC;
	}

	/* Calculates the internal Point-Projection and Line-Projection matrices (called inside constructors, for speed-optimisation only)
	 */
	private void calcProjectionMatrices() {
		P = LinearAlgebraFactory.getMatrix(3, 4);

		for (int i = 0; i < DIMR; i++)
			for (int j = 0; j < DIMC; j++)
				P.set(i, j, v.get(j * DIMR + i));

		Q = LinearAlgebraFactory.getMatrix(3, 6);

		Q.setColumn(0, P.getColumn(3).crossProduct(P.getColumn(0)));
		Q.setColumn(1, P.getColumn(3).crossProduct(P.getColumn(1)));
		Q.setColumn(2, P.getColumn(3).crossProduct(P.getColumn(2)));
		Q.setColumn(3, P.getColumn(1).crossProduct(P.getColumn(2)));
		Q.setColumn(4, P.getColumn(2).crossProduct(P.getColumn(0)));
		Q.setColumn(5, P.getColumn(0).crossProduct(P.getColumn(1)));

		Qdual = LinearAlgebraFactory.getMatrix(3, 6);
		for (int i = 0; i < 3; i++)
			for (int j = 0; j < 2; j++)
				for (int k = 0; k < 3; k++)
					Qdual.set(i, (1 - j) * 3 + k, Q.get(i, j * 3 + k));
	}

	public Matrix getPointMatrix() {
		return P;
	} // end getPointMatrix        

	public Matrix getLineMatrix() {
		return Q;
	} // end getLineMatrix        

	public Matrix getDualLineMatrix() {
		return Qdual;
	} // end getLineMatrix
	
	
	/** Returns the principal plane of the camera
	 * @return last row of the PointProjectionMatrix
	 */
	public Plane_3D getPrincipalPlane() {
		int[] xxx = {2, DIMR+2, 2*DIMR+2, 3*DIMR+2};		
		return new Plane_3D (v.get(xxx[0]), v.get(xxx[1]), v.get(xxx[2]), v.get(xxx[3]), cov.slice(xxx,xxx));
	} // end getPrincipalPlane

	/** Returns the projection center of the camera
	 * @return projection center
	 */
	public Point_3D getProjectionCenter() {
		int[] row1 = {0, DIMR+0, 2*DIMR+0, 3*DIMR+0};				
		int[] row2 = {1, DIMR+1, 2*DIMR+1, 3*DIMR+1};				
		int[] row3 = {2, DIMR+2, 2*DIMR+2, 3*DIMR+2};
		Plane_3D p1 = new Plane_3D (v.get(row1[0]), v.get(row1[1]), v.get(row1[2]), v.get(row1[3]), cov.slice(row1,row1));
		Plane_3D p2 = new Plane_3D (v.get(row2[0]), v.get(row2[1]), v.get(row2[2]), v.get(row2[3]), cov.slice(row2,row2));
		Plane_3D p3 = new Plane_3D (v.get(row3[0]), v.get(row3[1]), v.get(row3[2]), v.get(row3[3]), cov.slice(row3,row3));
		return new Point_3D (p1, p2, p3);
	} // end getProjectionCenter
	
	/** Returns the rotation matrix from the world to the camera coordiante system
	 * @return world to camera rotation matrix
	 */
	public Matrix getRotationMatrix() {
		Matrix[] QR = P.slice(0,2,0,2).invert().qr();
		
		// Correct signs
	    Matrix K=QR[1].invert();

	    Matrix S=LinearAlgebraFactory.getMatrix(3,3);
	    for (int i=0; i<3; i++)
	    	if (K.get(i,i)>0)
	    		S.set(i,i,1);
	    	else
	    		S.set(i,i,-1);
	    
	    // Rotation Matrix
	    Matrix R=S.mult(QR[0].trans());
	    
	    if (R.det()<0)
	    	R=R.mult(-1.0);

		return R;
	}
	
	/** Returns the calibration matrix containing the interior orientation parameters
	 * @return calibration matrix
	 */
	public Matrix getCalibrationMatrix() {
		Matrix[] QR = P.slice(0,2,0,2).invert().qr();
		
	    Matrix K=QR[1].invert();
	    
		// Correct signs
	    Matrix S=LinearAlgebraFactory.getMatrix(3,3);
	    for (int i=0; i<3; i++)
	    	if (K.get(i,i)>0)
	    		S.set(i,i,1);
	    	else
	    		S.set(i,i,-1);
	    
	    K=K.mult(S);
	    
	    return K.mult(1/K.get(2,2));
	}
	
	/** Returns the Inverse Point Projection Matrix of the plane A. 
	 * Multiplying an image point with this matrix yields the 3D-Point lying on the plane A.
	 * @param A the plane
	 * @return Inverse Point Projection Matrix
	 */
	public Matrix getInversePointMatrix (Vector A) {
		Plane_3D U = new Plane_3D (P.getRow(0), LinearAlgebraFactory.getMatrix(4,4));
		Plane_3D V = new Plane_3D (P.getRow(1), LinearAlgebraFactory.getMatrix(4,4));
		Plane_3D W = new Plane_3D (P.getRow(2), LinearAlgebraFactory.getMatrix(4,4));
		
		Plane_3D PI = new Plane_3D (A, LinearAlgebraFactory.getMatrix(4,4));
		
		Point_3D X = new Point_3D (V, W, PI);
		Point_3D Y = new Point_3D (W, U, PI);
		Point_3D Z = new Point_3D (U, V, PI);
		
		Matrix PA = LinearAlgebraFactory.getMatrix(4,3);
		PA.setColumn(0, X.getV());
		PA.setColumn(1, Y.getV());
		PA.setColumn(2, Z.getV());
		
		return PA;
	}
	
	/** Performes the inverse projection of an 2D-point
	 * @param p the 2D point to be inverse projected
	 * @return the 3D-line, that is obtained by inverse projection of the 2D-point
	 * TODO: Check this
	 */
	public Line_3D transform(Point_2D p) {
		Vector x = p.getV();
		Matrix J_x = LinearAlgebraFactory.getMatrix(6, 12);

		double t3 = -P.get(2,2)*x.get(1)+P.get(1,2)*x.get(2);
		double t6 = P.get(2,2)*x.get(0)-P.get(0,2)*x.get(2);
		double t9 = -P.get(1,2)*x.get(0)+P.get(0,2)*x.get(1);
		double t12 = P.get(2,1)*x.get(1)-P.get(1,1)*x.get(2);
		double t15 = -P.get(2,1)*x.get(0)+P.get(0,1)*x.get(2);
		double t18 = P.get(1,1)*x.get(0)-P.get(0,1)*x.get(1);
		double t21 = -P.get(2,0)*x.get(1)+P.get(1,0)*x.get(2);
		double t24 = P.get(2,0)*x.get(0)-P.get(0,0)*x.get(2);
		double t27 = -P.get(1,0)*x.get(0)+P.get(0,0)*x.get(1);
		double t30 = P.get(2,3)*x.get(1)-P.get(1,3)*x.get(2);
		double t33 = -P.get(2,3)*x.get(0)+P.get(0,3)*x.get(2);
		double t36 = P.get(1,3)*x.get(0)-P.get(0,3)*x.get(1);
		J_x.set(0,3, t3/2.0);
		J_x.set(0,4, t6/2.0);
		J_x.set(0,5, t9/2.0);
		J_x.set(0,6, t12/2.0);
		J_x.set(0,7, t15/2.0);
		J_x.set(0,8, t18/2.0);
		J_x.set(1,0, -t3/2.0);
		J_x.set(1,1, -t6/2.0);
		J_x.set(1,2, -t9/2.0);
		J_x.set(1,6, t21/2.0);
		J_x.set(1,7, t24/2.0);
		J_x.set(1,8, t27/2.0);
		J_x.set(2,0, -t12/2.0);
		J_x.set(2,1, -t15/2.0);
		J_x.set(2,2, -t18/2.0);
		J_x.set(2,3, -t21/2.0);
		J_x.set(2,4, -t24/2.0);
		J_x.set(2,5, -t27/2.0);
		J_x.set(3,0, t30/2.0);
		J_x.set(3,1, t33/2.0);
		J_x.set(3,2, t36/2.0);
		J_x.set(3,9, t21/2.0);
		J_x.set(3,10, t24/2.0);
		J_x.set(3,11, t27/2.0);
		J_x.set(4,3, t30/2.0);
		J_x.set(4,4, t33/2.0);
		J_x.set(4,5, t36/2.0);
		J_x.set(4,9, -t12/2.0);
		J_x.set(4,10, -t15/2.0);
		J_x.set(4,11, -t18/2.0);
		J_x.set(5,6, t30/2.0);
		J_x.set(5,7, t33/2.0);
		J_x.set(5,8, t36/2.0);
		J_x.set(5,9, t3/2.0);
		J_x.set(5,10, t6/2.0);
		J_x.set(5,11, t9/2.0);

		return new Line_3D(
			Qdual.trans().mult(p.v),
			Qdual.trans().mult(p.cov).mult(Qdual).plus(
				J_x.mult(getCov()).mult(J_x.trans())));
	}

	/** Performes the inverse projection of an 2D-line
	 * @param l the 2D line to be inverse projected
	 * @return the 3D plane, that is obtained by inverse projection of the 2D line
	 * TODO: Check this
	 */
	public Plane_3D transform(Line_2D l) {
		Matrix J_l = LinearAlgebraFactory.getMatrix(4, 12);
		Vector v = l.getV();

		J_l.set(0,0, v.get(0));
		J_l.set(0,1, v.get(1));
		J_l.set(0,2, v.get(2));
		J_l.set(1,3, v.get(0));
		J_l.set(1,4, v.get(1));
		J_l.set(1,5, v.get(2));
		J_l.set(2,6, v.get(0));
		J_l.set(2,7, v.get(1));
		J_l.set(2,8, v.get(2));
		J_l.set(3,9, v.get(0));
		J_l.set(3,10, v.get(1));
		J_l.set(3,11, v.get(2));

		return new Plane_3D(
			P.trans().mult(l.v),
			P.trans().mult(l.cov).mult(P).plus(
				J_l.mult(getCov()).mult(J_l.trans())));
	}

	/** Performes the projection of a 3D point
	 * @param p the 3D point to be projected
	 * @return  the 2D point obtained by projecting the 3D point */
	public Point_2D transform(Point_3D p) {
		return (Point_2D) transform(P, getCov(), p, Point_2D.class);
	}

	/** Performes the projection of a 3D line
	 * @param l 3D line to be projected
	 * @return  the 2D line, that is obtained by projecting the 3D line 
	 * TODO: Check this
	 */
	public Line_2D transform(Line_3D l) {
		Matrix J_L = LinearAlgebraFactory.getMatrix(3, 12);
		Vector v = l.getV();

		double t4 = -P.get(2,3)*v.get(0)-P.get(2,2)*v.get(4)+P.get(2,1)*v.get(5);
		double t8 = P.get(1,3)*v.get(0)+P.get(1,2)*v.get(4)-P.get(1,1)*v.get(5);
		double t12 = -P.get(2,3)*v.get(1)+P.get(2,2)*v.get(3)-P.get(2,0)*v.get(5);
		double t16 = P.get(1,3)*v.get(1)-P.get(1,2)*v.get(3)+P.get(1,0)*v.get(5);
		double t20 = -P.get(2,3)*v.get(2)-P.get(2,1)*v.get(3)+P.get(2,0)*v.get(4);
		double t24 = P.get(1,3)*v.get(2)+P.get(1,1)*v.get(3)-P.get(1,0)*v.get(4);
		double t28 = P.get(2,0)*v.get(0)+P.get(2,1)*v.get(1)+P.get(2,2)*v.get(2);
		double t32 = -P.get(1,0)*v.get(0)-P.get(1,1)*v.get(1)-P.get(1,2)*v.get(2);
		double t36 = -P.get(0,3)*v.get(0)-P.get(0,2)*v.get(4)+P.get(0,1)*v.get(5);
		double t40 = -P.get(0,3)*v.get(1)+P.get(0,2)*v.get(3)-P.get(0,0)*v.get(5);
		double t44 = -P.get(0,3)*v.get(2)-P.get(0,1)*v.get(3)+P.get(0,0)*v.get(4);
		double t48 = P.get(0,0)*v.get(0)+P.get(0,1)*v.get(1)+P.get(0,2)*v.get(2);
		J_L.set(0,1, t4/2.0);
		J_L.set(0,2, t8/2.0);
		J_L.set(0,4, t12/2.0);
		J_L.set(0,5, t16/2.0);
		J_L.set(0,7, t20/2.0);
		J_L.set(0,8, t24/2.0);
		J_L.set(0,10, t28/2.0);
		J_L.set(0,11, t32/2.0);
		J_L.set(1,0, -t4/2.0);
		J_L.set(1,2, t36/2.0);
		J_L.set(1,3, -t12/2.0);
		J_L.set(1,5, t40/2.0);
		J_L.set(1,6, -t20/2.0);
		J_L.set(1,8, t44/2.0);
		J_L.set(1,9, -t28/2.0);
		J_L.set(1,11, t48/2.0);
		J_L.set(2,0, -t8/2.0);
		J_L.set(2,1, -t36/2.0);
		J_L.set(2,3, -t16/2.0);
		J_L.set(2,4, -t40/2.0);
		J_L.set(2,6, -t24/2.0);
		J_L.set(2,7, -t44/2.0);
		J_L.set(2,9, -t32/2.0);
		J_L.set(2,10, -t48/2.0);

		return new Line_2D(
			Q.mult(l.v),
			Q.mult(l.cov).mult(Q.trans()).plus(
				J_L.mult(getCov()).mult(J_L.trans())));
	}

	/** This method invokes the appropriate transform, if the Object has been downcasted
	 * @param e entity to be transformed
	 * @return  transformed entity or null, if entity is not transformable */
	public Entity transform(Entity e) {
		if (e instanceof Point_2D)
			return transform((Point_2D) e);
		else if (e instanceof Line_2D)
			return transform((Line_2D) e);
		else if (e instanceof Line_3D)
			return transform((Line_3D) e);
		else if (e instanceof Point_3D)
			return transform((Point_3D) e);
		else
			return null;
	}

	/** returns the degrees of freedom of the projective camera
	  * @return  always 11 */
	public int getDegreesOfFreedom() {
		return DOF;
	}

	/** Condition that element with a set of conditioning factors.
	 *  Because the stored homography matrix is the one for points,
	 *  the condition-factor-matrix for Point_3D and Point_2D is used.
	 * @param f conditioning factors ( two factors for transformations!)
	 */
	public void doCondition(double[] f) {
		Point_2D dummy2d = new Point_2D();
		Point_3D dummy3d = new Point_3D();
		Matrix W0 = dummy3d.getConditionMatrix(f[0]);
		Matrix W1 = dummy2d.getConditionMatrix(f[1]);
		Matrix Id1 = LinearAlgebraFactory.getMatrixId(getDimCol());
		Matrix Id2 = LinearAlgebraFactory.getMatrixId(getDimRow());
		
		// H0=W(fstrich)*H, vec(ABC)=(Ct kron A) vec(B) 
		Matrix A = Id1.kronecker(W1);
		v = A.mult(v);
		cov = A.mult(cov).mult(A.trans());

		// H1=H*W-1(f) 
		A = W0.invert().trans().kronecker(Id2);
		v = A.mult(v);
		cov = A.mult(cov).mult(A.trans());
		
		calcProjectionMatrices();
	}

	/** The the Jacobian after the first entity of a trilinear relation.
	 *  For this computation the second entity and the unkown=transformation is needed.
	 * @param e the second Entity of the relation.
	 */
	public Matrix getJacobianAfterFirst(Entity e) {
		if (e.getClass().equals(Point_2D.class)) {
			Point_2D p = (Point_2D) e;
			return p.SkewSym().mult(getTransformationMatrix());
		}

		if (e.getClass().equals(Line_2D.class)) {
			Line_2D l = (Line_2D) e;
			return Point_3D
				.Pi(getTransformationMatrix().trans().mult(l.getV()))
				.trans();
		}

		throw new IllegalArgumentException(
			"Illegal trilinear relation between "
				+ getClass()
				+ " and "
				+ e.getClass());

	}

	/** The the Jacobian after the second entity of a trilinear relation.
	 * For this computation the first entity and the unkown=transformation is needed.
	 * @param e the first Entity of the relation.
	 */
	public Matrix getJacobianAfterSecond(Entity e) {
		if (e.getClass().equals(Point_3D.class)) {
			return Point_2D.SkewSym(
				getTransformationMatrix().mult(e.getV())).mult(
				-1);
		}

		if (e.getClass().equals(Line_3D.class)) {
			Line_3D l = (Line_3D) e;
			return l.Gamma().trans().mult(getTransformationMatrix().trans());
		}

		throw new IllegalArgumentException(
			"Illegal trilinear relation between "
				+ getClass()
				+ " and "
				+ e.getClass());

	}

} // end ProjectiveCamera
