/*
 * Created on 25.09.2003
 *
 */
package sugr;

import sugr.linalg.*;

/**
 * This interface represents a relation between 1..n oberservations and a target
 * entity. The target entity is not yet known, therefore only the type of that
 * entity must be available. All relational properties should be cloneable.
 * @author Thomas Laebe, ipb
 *
 */
public interface RelationalProperty extends Cloneable {

	/** Get observations of that relation as a stacked vector.
	 *  Used for estimation.
	 * @return the observations for that relation
	 */
	public Vector getObservations();
	
	/** Set observations of that relation (as a stacked vector).
	 *  Used for estimation.
	 */
	public void setObservations(Vector v);
	
	/** Get the Jacobian of the constraint of the observation
	 * 
	 * @return Matrix G
	 */
	public Matrix getJacobianOfObservationConstraints();

	/** Get the value of the constraint of the observation
	 * 
	 * @return Matrix g0
	 */
	public Vector getValueOfObservationConstraints();
	
	/** Ensures, that all elements fulfill their constraints
	 * 
	 */
	public void ensureConstraints();

	/** Changes the nullspace for all entities
	 * 
	 */
	public void changeNullspace();

	/** The the covariance matrix of the observations of that relation.
	 *  This is the covariance of the vector given by getObservatios().
	 *  Used for estimation.
	 * @return the covariance of the observations.
	 */
	public Matrix getCovarianceOfObservations();
	
	/** The "reduced" covariance matrix of the observations which takes 
	 *  care of the homogeneous constraint. This is needed to obtain a 
	 *  coherent covariance of the corrected observation from the inner loop 
	 *  of the estimation, see note of WF from May 2005. 
	 *  	
	 * @return the covariance of the observations.
	 */
	public Matrix getReducedCovarianceOfObservations();
	
	
	/** Get the type of the target entity. For estimation, this is the
	 *  type to be estimated.
	 * @return the target type
	 */
	public Class getTargetType();

	/**
	 * <p>
	 * Return Jacobian Matrix A after the target type.
	 * </p><p>
	 * 
	 * @return Jacobian Matrix A
	 * </p>
	 */
	public Matrix getJacobian();
	
	/**
	 * Returns the Covariance Matrix of the row-vector of the Jacobian Matrix A 
	 *
	 * @param row Row vector, for which the Covariance matrix is to be returned
	 * @return covariance of the row vector of the jacobian
	 */
	public Matrix getCovarianceOfJacobianRow(int row);

	/**
	 * <p>
	 * Compute degrees of freedom of that Relation
	 * </p><p>
	 * 
	 * @return degrees of freedom of that Relation
	 */
	public int getDof();
	
	/** Public clone function, because all relational properties should be cloneable. */
	public Object clone();

}
