/** Java class "Transformation.java" generated from Poseidon for UML. 
 *  Poseidon for UML is developed by <A HREF="http://www.gentleware.com">Gentleware</A>.
 *  Generated with <A HREF="http://jakarta.apache.org/velocity/">velocity</A> template engine.
 */
package sugr;

import sugr.linalg.*;

/**
 * <p>
 * This class contains main methods for transformations and their
 * special cases. 
 * Important to know: 
 * <ul>
 *   <li> The transformation matrix is stored into vector v which
 *        contains vec(T) (stacked columns).
 * 		  <pre>
 *        if matrix A is given by :     | a  b |
 *                                      | c  d |
 *
 *        then the vector is given by:  | a |
 *                                      | c |  = vec(A)
 *                                      | b |
 *                                      | d |            </pre> </li> 
 *		  
 *   <li> coovariance matrix is stored in normal way. please note the 
 * 		  necessary higher dimension of this matrix: 
 * 		  if transformation-matrix is 4x4, then covariance matrix is 16x16! </li>
 * 
 *   <li> Please construct point-transformation-matrices. If an entity like 
 * 		  Plane_3D oder Line_xD should be transformed, the transformation-matrix
 * 		  will be formatted correctly </li>
 * </ul>
 * 
 * @author Norbert Fischer
 * @version 1.0.1 (25.3.2003): 
 * 		FIXED: defining a variable DIM makes no sense in this class. Subclasses
 * 				have their own values, they are not "brought" up into this class 
 * </p>
 */
public abstract class Transformation extends Element {

	/**
	 * Returns number of elements to specify a transformation 
	 * @return a int containing this number. <br>
	 * 		   e.g.: Affinity_2D needs 6 parameters to be set
	 */
	public abstract int getDegreesOfFreedom();

	/**
	 * Transforms an entity with this transformation. The new entity contains 
	 * error propagation of tranform and old entity
	 * @param e   Entity to transform  
	 * @return A new Entity with applied transformation and error propagation
	 */
	public abstract Entity transform(Entity e);

	/**
	 * If you want to get the Transformation-Matrix (Affinity, Motion,
	 * Rotation, Translation) use this method. 
	 * If you want to get Transformation-Matrix as Vector, use getV().
	 * Matrix is copied. 
	 * @return Matrix with dimension DIMxDIM
	 */
	public Matrix getTransformationMatrix() {
		Matrix mat =
			LinearAlgebraFactory.getMatrix(getDimRow(), getDimCol());
		for (int i = 0; i < getDimRow(); i++) {
			for (int j = 0; j < getDimCol(); j++) {
				mat.set(i, j, getV().get(j * getDimRow() + i));
			}
		}

		return mat;
	}

	/**
	 * DIM cannot be used in subclasses' way. Therefore we need a method
	 * to gain the dimension out of the matrix, exactly, out of the vector.
	 * This is the dimension of the underlying matrix as vector. For the
	 * dimension of the transformation matrix use getDimR, getDimC.
	 * @return an integer containing the dimension of this transformation
	 */
	public int getDimension() {
		return getV().getDim();
	}
	
	/** Get the number of rows of the transformation matrix.
	 * @return the number of rows of the transformation matrix.
	 */
	public abstract int getDimRow();
	
	/** Get the number of columns of the transformation matrix.
	 * @return the number of columns of the transformation matrix.
	 */
	public abstract int getDimCol();


    /** Do a transformation with given transformation matrix  and covariance
     *  of tranformation matrix. The transformation matrix is not always the
     *  one stored in vector v, so it?s given by parameters.
     * @param h tranformation matrix
     * @param covariance covariance matrix of transformation matrix. This
     *        is actually the covariance of vec(h).
     * @param newType type of the new transformed entity
     * @return the new entity of type newType
     */
	protected Entity transform(Matrix h, Matrix covariance, Entity e, Class newType) {
		//Create new entity
		Vector new_Entity = h.mult(e.getV());

		//create error propagation. See stephans diss, page 121
		// Note that we use vec(H), whereas in Diss it is vec(HT)
		Matrix Id_DIM = LinearAlgebraFactory.getMatrixId(h.getRowDimension());
		Matrix err_Prop = h.mult(e.getCov()).mult(h.trans());
		
		Matrix Jacobi=e.getV().getMatrix().trans().kronecker(Id_DIM);
		err_Prop = err_Prop.plus(Jacobi.mult(covariance).mult(Jacobi.trans()));

		//return new Entity
		Entity tmp = Factory.getEntity(new_Entity, err_Prop, newType);
		return tmp;
	}
	

	/**
	 * calculates the error propagation between two transformations <br>
	 * Note that there is only needed one method for 2D and 3D case
	 * @param a   first Transformation
	 * @param b   second Transformation
	 * @return  covariance Matrix containing the error propagation between a and b
	 */
	protected Matrix errorProp(Transformation a, Transformation b) {
		//less to write
		int dim = a.getDimRow(); //equal for a and b
		Matrix A = a.getTransformationMatrix();
		Matrix B = b.getTransformationMatrix();

		//fill matrices!
		// (these formulas were calculated with maple by M. Luxen
		Matrix jacobian_A =
			B.trans().kronecker(LinearAlgebraFactory.getMatrixId(dim));
		
		Matrix jacobian_B = LinearAlgebraFactory.getMatrixId(dim).kronecker(A);

		//error propagation
		Matrix covariance = jacobian_A.mult(a.getCov()).mult(jacobian_A.trans());
		covariance =
			covariance.plus(jacobian_B.mult(b.getCov()).mult(jacobian_B.trans()));

		return covariance;
	}

	/** Get the reduced Matrix A and B from a relational 
	 * property. A is the Jacobian of this Entity (the unknown),
	 * B is the Jacobian of the observation (the relational property).
	 * The Jacobian of the oberservation depends (only) on the unknown and
	 * the Jacobian of the unknown depends (only) of the oberservation !
	 * 
	 * This is an overwritten method from element. Due to the structure of
	 * the Jacobians for transformation, only the Matrix A is used for
	 * reduction. Matrix B includes the transformation matrix which
	 * destroys the structure given by PI and Gamma matrixes and therefore
	 * the reduction implemented in Element does not work any more.
	 * 
	 * @param r the relational Property
	 * @return A in array element 0, B in array Element 1.
	 */
	protected Matrix[] getReducedAB(RelationalProperty r) {
			Matrix A = r.getJacobian();
			Matrix B = getJacobianOfRelation(r);
			
			// Now reduce Matrixes. The lines chosen depend only
			// on Matrix A!
			int dof=r.getDof();
			Vector[][] rows = new Vector[A.getRowDimension()][];
			for (int i = 0; i < A.getRowDimension(); i++) {
				rows[i] = new Vector[2];
				rows[i][0] = A.getRow(i);
				rows[i][1] = B.getRow(i);
			}
			java.util.Arrays.sort(rows, new java.util.Comparator() {
				public int compare(Object o1, Object o2) {
					
					Vector a1 = ((Vector[]) o1)[0];
					Vector a2 = ((Vector[]) o2)[0];
					double n1 = a1.normM();
					double n2 = a2.normM();
					if (n1 > n2)
						return -1;
					else if (n1 < n2)
						return 1;
					else
						return 0;
				}
			});
			Matrix reducedA =
				LinearAlgebraFactory.getMatrix(dof, A.getColumnDimension());
			Matrix reducedB =
				LinearAlgebraFactory.getMatrix(dof, B.getColumnDimension());
			for (int i = 0; i < dof; i++) {
				reducedA.setRow(i, rows[i][0]);
				reducedB.setRow(i, rows[i][1]);
			}
			Matrix[] result = new Matrix[2];
			result[0] = reducedA;
			result[1] = reducedB;
			return result;
	}


	/** Get the (joint) Jacobian matrix after the observation of
	 *  a trilinear Relation between two entities and a tranformation.
	 *  This joint covariance has always two parts: The first part
	 *  is the Jacobian after the first entity and the second part
	 *  is the Jacobian after the second entity.
	 *  @param rp the relational property which is for transformations
	 *         always a TrirelationalProperty.
	 *
	 */
	public Matrix getJacobianOfRelation(RelationalProperty rp) {
		TriRelationalProperty r= (TriRelationalProperty) rp;
		Matrix B1 = getJacobianAfterFirst(r.getSecondEntity());
		Matrix B2 = getJacobianAfterSecond(r.getFirstEntity());
		int rdim=B1.getRowDimension();
		Matrix B=LinearAlgebraFactory.getMatrix(rdim,
		B1.getColumnDimension()+B2.getColumnDimension());
		B.slice(0, rdim-1, 0, B1.getColumnDimension()-1, B1);
		B.slice(0, rdim-1, B1.getColumnDimension(), B.getColumnDimension()-1, B2);
		return B;
	}

	/** The the Jacobian after the first entity of a trilinear relation.
	 *  second=Transformation*first
	 *  For this computation the second entity and the unkown=transformation is needed.
	 * @param e the second Entity of the relation.
	 *
	 */
	public abstract Matrix getJacobianAfterFirst(Entity e);

	/** The the Jacobian after the second entity of a trilinear relation.
	 *  second=Transformation*first
	 * For this computation the first entity and the unkown=transformation is needed.
	 * @param e the first Entity of the relation.
	 *
	 */
	public abstract Matrix getJacobianAfterSecond(Entity e);
	
	/** Get a transformation matrix A which transforms vec(Ht)=A*vec(H). Works for
	 *  square matrixes H
	 *  @param size number of rows (and columns) of H */
	public static Matrix getTransposeTransMatrix(int size) {
		Matrix A=LinearAlgebraFactory.getMatrix(size*size, size*size);
		for(int i=0; i<size; i++)
			for(int j=0; j<size; j++)
				A.set(i*size+j, j*size+i, 1);
		return A;
	}


} // end Transformation
