/*
 * Created on 13.03.2003
 */
package sugr;

import sugr.linalg.*;

/**
 * class containing transformation(3) constructors 
 * @author Norbert Fischer
 */
public class Translation_3D extends Motion_3D
{
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * standard constructor. creates an identity matrix
	 */
	public Translation_3D() {}
	
	/**
	 * Creates a translation matrix with single parameters
	 * @param x   moving along the x-Axis
	 * @param y   moving along the y-Axis
	 * @param z   moving along the z-Axis
	 */
	public Translation_3D(double x, double y, double z)
	{
		this(x,y,z, LinearAlgebraFactory.getMatrix(DIM*DIM, DIM*DIM));		
	}
	
	/**
	 * Creates a translation matrix with covariance and single parameters
	 * @param x   moving along the x-Axis
	 * @param y   moving along the y-Axis
	 * @param z   moving along the z-Axis
	 * @param cov   16x16 Matrix
	 */
	public Translation_3D(double x, double y, double z, Matrix cov)
	{
		Vector tmpV = LinearAlgebraFactory.getMatrixId(DIM).vec();
		tmpV.set(12, x);
		tmpV.set(13, y);
		tmpV.set(14, z);
		tmpV.set(15, 1);
		setV(tmpV);
		
		checkCovDim(cov, DIM*DIM);
		setCov(cov);		
	}


	/**
	 * Creates a translation matrix by a vector
	 * @param transVector   Dimension must be 4
	 */
	public Translation_3D(Vector transVector)
	{
		this(transVector, LinearAlgebraFactory.getMatrix(DIM*DIM, DIM*DIM));
	}
	
	/**
	 * Creates a translation matrix by a vector with covariance
	 * @param transVector   Dimension must be 4
	 * @param cov	16x16 matrix
	 */
	public Translation_3D(Vector transVector, Matrix cov)
	{
		if (transVector.getDim()!=3) 
		{
			throw new IllegalArgumentException("Translation vector must " +
				"have three entries!");
		}
		Vector tmpV = LinearAlgebraFactory.getMatrixId(DIM).vec();
		tmpV.set(12, transVector.get(0));
		tmpV.set(13, transVector.get(1));
		tmpV.set(14, transVector.get(2));
		tmpV.set(15, 1);
		setV(tmpV);		
		
		checkCovDim(cov, DIM*DIM);
		setCov(cov);
	}
	
	/**
	 * Creates a translation by a Point_3D
	 * @param x   an arbitrary Point_3D. Euclidean normalization will be
	 * 			 done automatically in this method
	 */
	public Translation_3D(Point_3D x)
	{
		this(x, LinearAlgebraFactory.getMatrix(DIM*DIM,DIM*DIM));
	}
	
	/**
	 * Creates a translation by a Point_3D
	 * @param x   an arbitrary Point_3D. Euclidean normalization will be
	 * 			 done automatically in this method
	 * @param cov   a 16x16 matrix
	 */
	public Translation_3D(Point_3D x, Matrix cov)
	{
		this(x.getV().get(0)/x.getV().get(3),
			 x.getV().get(1)/x.getV().get(3),
			 x.getV().get(2)/x.getV().get(3),
			 cov);
	}
	
	/**
	 * Returns the number of elements to specify this transformation
	 */
	public int getDegreesOfFreedom()
	{
		return 3;
	}	
	
}
