/*
 * Created on 19.09.2003
 *
 */
package sugr;

import sugr.linalg.*;

/** This class represents trilinear relations. This is used for
 *  modeling relations with transformations (projections and
 *  homographies). The relation includes two concrete entities
 *  (first, second) and the type of the transformation (e.g.
 *  a transformation). second=Transformation*first
 *  
 * 
 * @author Thomas Laebe, ipb
 *
 */
public class TriRelationalProperty implements RelationalProperty {

	private Entity first;
	private Entity second;
	private Class targetType;

	/** Construct a trilinear Relation e2= T*e1.
	 * 
	 * @param _first the first Entity of the relation
	 * @param _second the second Entity 
	 * @param _targetType the type of the element to be estimated e.g.
	 *        a transformation.
	 */
	public TriRelationalProperty(
		Entity _first,
		Entity _second,
		Class _targetType) {
		first = _first;
		second = _second;
		targetType = _targetType;
	}

	/** Get the type of the target entity. For estimation, this is the
	 *  type to be estimated.
	 * @return the target type
	 */
	public Class getTargetType() {
		return targetType;
	}

	/** Get the first Entity.
	 * 
	 */
	public Entity getFirstEntity() {
		return first;
	}

	/** Get the second Entity.
	 * 
	 */
	public Entity getSecondEntity() {
		return second;
	}

	/**
	 * <p>
	 * Return Jacobian Matrix A of the first Entity and the relation type.
	 * </p><p>
	 * 
	 * @return Jacobian Matrix A
	 * </p>
	 */
	public Matrix getJacobian() {
		if (targetType.equals(Homography_2D.class)) {
			Matrix Id=LinearAlgebraFactory.getMatrixId(3);
			
			if (first.getClass().equals(Point_2D.class)) {
				Point_2D x = (Point_2D) first;
				Point_2D xp = (Point_2D) second;
				Matrix Jt =
					xp.SkewSym().mult(
			Id.kronecker(
							x.getV().getMatrix().trans()));
				return ChangeOrderVecTrans(Jt);
			}
			if (first.getClass().equals(Line_2D.class)) {
				Line_2D l = (Line_2D) first;
				Line_2D lp= (Line_2D) second;
				Matrix Jt=l.SkewSym().mult(lp.getV().getMatrix().trans().kronecker(Id));
				return ChangeOrderVecTrans(Jt);
			}
		}
		 
		if (targetType.equals(Homography_3D.class)
				|| targetType.equals(Motion_3D.class)
				|| targetType.equals(ScaledMotion_3D.class)) {
			Matrix Id=LinearAlgebraFactory.getMatrixId(4);
			
			if (first.getClass().equals(Point_3D.class)) {
				Point_3D x = (Point_3D) first;
				Point_3D xp = (Point_3D) second;
				Matrix Jt = xp.Pi().mult(Id.kronecker(x.getV().getMatrix().trans()));
				return ChangeOrderVecTrans(Jt);
			}
			if (first.getClass().equals(Line_3D.class)) {
				Line_3D l = (Line_3D) first;
				Line_3D lp= (Line_3D) second;
				//Matrix Jt=lp.GammaDual().trans().kronecker(l.Gamma());
				//return ChangeOrderVecTrans(Jt);
				Matrix Jt=l.Gamma().trans().kronecker(lp.GammaDual());
				return Jt;
			}
			if (first.getClass().equals(Plane_3D.class)) {
				Plane_3D A = (Plane_3D) first;
				Plane_3D Ap= (Plane_3D) second;
				Matrix Jt=A.PiDual().mult(Ap.getV().getMatrix().trans().kronecker(Id));
				return ChangeOrderVecTrans(Jt);
			}
			
		}
		
		if (targetType.equals(ProjectiveCamera.class)) {
			
			
			if (first.getClass().equals(Point_3D.class)) {
				Matrix Id=LinearAlgebraFactory.getMatrixId(3);
				Point_3D x = (Point_3D) first;
				Point_2D xp = (Point_2D) second;
				//Matrix Jt = xp.SkewSym().mult(Id.kronecker(x.getV().getMatrix().trans()));
				Matrix Jt = xp.SkewSym().mult(x.getV().getMatrix().trans().kronecker(Id));
				return Jt;
				//return ChangeOrderVecTrans(Jt);
			}
			if (first.getClass().equals(Line_3D.class)) {
				Matrix Id=LinearAlgebraFactory.getMatrixId(4);
				Line_3D l = (Line_3D) first;
				Line_2D lp= (Line_2D) second;
				//Matrix Jt=l.Gamma().trans().mult(lp.getV().getMatrix().trans().kronecker(Id));
				//return ChangeOrderVecTrans(Jt);
				Matrix Jt=l.Gamma().trans().mult(Id.kronecker(lp.getV().getMatrix().trans()));
				return Jt;
			}
			
		}


		
		throw new IllegalArgumentException(
			"Illegal Trilinear Relation between "
				+ first.getClass()
				+ " and "
				+ second.getClass()
				+ " and as target "
				+ targetType);
	}

	/** Clone a relational property. The Entities and the target Type are cloned.
	 * @return a copy of the element. 
	 */
	public Object clone() {
		try {
			TriRelationalProperty r = (TriRelationalProperty) super.clone();
			r.first = (Entity) first.clone();
			r.second = (Entity) second.clone();
			r.targetType = targetType;
			return r;

		} catch (CloneNotSupportedException e) {
			// this shouldn't happen 
			throw new InternalError();
		}

	}

	/**
	 * Compute degrees of freedom of that relation.
	 * 
	 * @return degrees of freedom of that relation
	 */
	public int getDof() {
		if (targetType.equals(Homography_2D.class))
			return first.getDegreesOfFreedom();
		if (targetType.equals(Homography_3D.class)
				|| targetType.equals(Motion_3D.class)
				|| targetType.equals(ScaledMotion_3D.class))
			return first.getDegreesOfFreedom();
		if (targetType.equals(ProjectiveCamera.class))
			return 2;
			
		throw new IllegalArgumentException(
			"Illegal Trilinear Relation between "
				+ first.getClass()
				+ " and "
				+ second.getClass()
				+ " and as target "
				+ targetType);
	}

	/**
	 * <p>
	 * Returns the Covariance Matrix of the row-vector of the Jacobian Matrix A of the first Entity and the relation type.
	 *</p>
	 *
	 * @param row Row vector, for which the Covariance matrix is to be returned
	 * @return covariance of the row vector of the jacobian
	 * TODO: implement this
	 */
	public Matrix getCovarianceOfJacobianRow(int row) {
		throw new RuntimeException("Not yet implemented");
	}

	/** Get observation of that relation. 
		 *  The first part of the vector is the first entity, the second part is 
	 *  the second entity.
	 *  Used for estimation.
	 * @return the observation for that relation
	 */
	public Vector getObservations() {
		Vector v1 = first.getV();
		Vector v2 = second.getV();
		Vector v = LinearAlgebraFactory.getVector(v1.getDim() + v2.getDim());

		v.set(0, v1);
		v.set(v1.getDim(), v2);

		return v;
	}

	/** Set observations of that relation (as a stacked vector).
	 *  The first part of the vector is the first entity, the second part is 
	 *  the second entity.
	 */
	public void setObservations(Vector v) {
		Vector v1 = v.get(0, first.getV().getDim() - 1);
		Vector v2 = v.get(first.getV().getDim(), v.getDim() - 1);
		first.setV(v1);
		second.setV(v2);
	}

	public Matrix getJacobianOfObservationConstraints() {
		Matrix G1 = first.getJacobianOfConstraint();
		Matrix G2 = second.getJacobianOfConstraint();
		Matrix G = LinearAlgebraFactory.getMatrix(G1.getRowDimension()+G2.getRowDimension(), G1.getColumnDimension()+G2.getColumnDimension());
		G.slice(0,
				G1.getRowDimension()-1, 
				0, 
				G1.getColumnDimension()-1, 
				G1);
		G.slice(G1.getRowDimension(), 
				G1.getRowDimension()+G2.getRowDimension()-1, 
				G1.getColumnDimension(), 
				G1.getColumnDimension()+G2.getColumnDimension()-1, 
				G2);
		return G;
	}

	public Vector getValueOfObservationConstraints() {
		Vector v1 = first.getConstraintValue();
		Vector v2 = second.getConstraintValue();
		Vector v = LinearAlgebraFactory.getVector(v1.getDim() + v2.getDim());

		v.set(0, v1);
		v.set(v1.getDim(), v2);

		return v;
	}

	/** The the covariance matrix of the observation of that relation.
	 *  For trilinear observations this is the combinations for the
	 *  covariance matrixes of the first and third entity. Used for
	 * estimation.
	 * @return the covariance of the observations.
	 */
	public Matrix getCovarianceOfObservations() {
		int s1 = first.getDimension();
		int s2 = second.getDimension();
		Matrix cov = LinearAlgebraFactory.getMatrix(s1 + s2, s1 + s2);

		cov.slice(0, s1 - 1, 0, s1 - 1, first.getCov());
		cov.slice(s1, s1 + s2 - 1, s1, s1 + s2 - 1, second.getCov());
	
		return cov;
	}
	
	/** The "reduced" covariance matrix of the observations which takes 
	 *  care of the homogeneous constraint. This is needed to obtain a 
	 *  coherent covariance of the corrected observation from the inner loop 
	 *  of the estimation, see note of WF from May 2005. 
	 *  	
	 * @return the covariance of the observations.
	 */
	public Matrix getReducedCovarianceOfObservations() {
		Matrix Syy = getCovarianceOfObservations();
		Matrix B2_1 = getFirstEntity().getJacobianOfConstraint().trans();
		Matrix B2_2 = getSecondEntity().getJacobianOfConstraint().trans();
		Matrix B2 = LinearAlgebraFactory.getMatrix(B2_1.getRowDimension()+B2_2.getRowDimension()
				,Math.max(B2_2.getColumnDimension(),B2_1.getColumnDimension()));
		B2.slice(0,B2_1.getRowDimension()-1,0,B2_1.getColumnDimension()-1,B2_1); //TODO: index out of bounds exception for projectivecamera
		B2.slice(B2_1.getRowDimension(),B2.getRowDimension()-1,
				 0,B2_2.getColumnDimension()-1,B2_2);
		Matrix DeltaNSyy = Syy.mult(B2).mult(
                B2.trans().mult(Syy).mult(B2).invert()
				 ).mult(B2.trans()).mult(Syy);
		
		return Syy.minus(DeltaNSyy);
	}

	/** Do conditioning of the observations.Two factors for
	 *  all first entities and all second entities are returned.
	 * @param rprops a set of Tri(!)RelationalProperties. RelationalProperty is used
	 *        for compatibility reasons.
	 * @return the 2 conditioning factors
	 */
	public static double[] doCondition(RelationalProperty[] rprops) {
		Entity[] elist = new Entity[rprops.length];
		double[] f = new double[2];
		for (int i = 0; i < rprops.length; i++)
			elist[i] = ((TriRelationalProperty) rprops[i]).first;

		f[0] = Entity.doCondition(elist);
		for (int i = 0; i < rprops.length; i++)
			elist[i] = ((TriRelationalProperty) rprops[i]).second;

		f[1] = Entity.doCondition(elist);

		return f;
	}
	
	
	/** Do conditioning of the observations with one factor.
	 *  Only one common factor for both sets of Entities is used.
	 *  Used for estimation of Motion_3D.
	 * 
	 * @param rprops a set of Tri(!)RelationalProperties. RelationalProperty is used
	 *        for compatibility reasons.
	 * @return An array of two conditioning factors, which
	 *         hold the same factor (for compatibility reasons)
	 */
	public static double[] doConditionOneFactor(RelationalProperty[] rprops) {
		Entity[] elist = new Entity[rprops.length*2];
		double[] f = new double[2];
		for (int i = 0; i < rprops.length; i++)
			elist[i] = ((TriRelationalProperty) rprops[i]).first;

		for (int i = 0; i < rprops.length; i++)
			elist[i+rprops.length] = ((TriRelationalProperty) rprops[i]).second;

		f[0] = Entity.doCondition(elist);
		f[1] = f[0];

		return f;
	}


	/** Do conditioning of the observations. The first entity is conditioned with
	 * f[0], the second with f[1].
	 * 
	 * @param f the 2 conditioning factors
	 * @param rprops a set of Tri(!)RelationalProperties. RelationalProperty is used
	 *        for compatibility reasons.
	 */
	public static void doCondition(RelationalProperty[] rprops, double[] f) {
		for (int i = 0; i < rprops.length; i++) {
			TriRelationalProperty r = (TriRelationalProperty) rprops[i];
			r.first.doCondition(f[0]);
			r.second.doCondition(f[1]);
		}
	}

	/** Change the order of columns of a matrix in that way that the resulting
	 *  matrix is J with J*vec(H) =0 whereas the input matrix is J*vec(Ht).
	 * @param J a matrix
	 * @return J with changed order of colums.
	 */
	private static Matrix ChangeOrderVecTrans(Matrix J) {
		int n = J.getColumnDimension();
		int sizeH = (int) (Math.sqrt(n));
		Matrix res = LinearAlgebraFactory.getMatrix(J.getRowDimension(), n);
		for (int i = 0; i < n; i++) {
			int newindex = (i / sizeH) + sizeH * (i % sizeH);
			res.setColumn(newindex, J.getColumn(i));
		}
		return res;
	}

	public void ensureConstraints() {
		first.ensureConstraints();
		second.ensureConstraints();
	}

	public void changeNullspace() {
		first.changeNullspace();
		second.changeNullspace();
	}


}
