package sugr.test;
import junit.framework.TestCase;
import sugr.*;
import sugr.linalg.*;

/*
 * Created on 06.02.2004
 *
 */

/**
 * @author Bastian Oehler
 *
 */
public class ElementTest extends TestCase {

	/**
	 * Constructor for ElementTest.
	 * @param arg0
	 */
	public ElementTest(String arg0) {
		super(arg0);
	}

	/**
	 * int MAXITER determines how many iterations there are for calculation of
	 * the statstic parts. It should be around 1 000 000 for good results, but
	 * this may mean much work for your maschine and some coffee for you,
	 * if you are waiting for the results.
	 * If MAXITER is set to 1 000 000, it may happen in an 'Out of Memory'-Error!
	 */
	public static int MAXITER = 1000000;
	
	/**
	 * double PRECISION for testing statistics. 
	 */
	public static double PRECISION = 0.3;

	/**
	 * Test for non-statistic part of void normalizeSphere()
	 */
	public void testNormalizeSphere() {
		Point_3D point = new Point_3D(1,1,1,1);
		Point_3D point2 = new Point_3D(0.5,0.5,0.5,0.5);
		point.normalizeSphere();
		if (!TestStatistics.check(point.getV(),point2.getV(),PRECISION)) fail();
		
		point = new Point_3D(7,7,1,1);
		point2 = new Point_3D(0.7,0.7,0.1,0.1);
		point.normalizeSphere();
		if (!TestStatistics.check(point.getV(),point2.getV(),PRECISION)) fail();
	}

	/**
	 * Test for error propagation of void normalizeSphere()
	 */
	public void testNormalizeSphereStatistics() {
		double[][] covArray = {{0.1,0,0,0},{0,0.1,0,0},{0,0,0.1,0},{0,0,0,0}};
		Matrix cov = LinearAlgebraFactory.getMatrix(covArray);
		Vector vec = LinearAlgebraFactory.getVector(3,3,3,1);
		Point_3D point = new Point_3D(vec,cov);
		point.normalizeSphere();
		TestStatistics statistic = new TestStatistics(4);
		RandomVector rv = new RandomVector(vec,cov);
		Point_3D point2;
		for (int i = 0; i < MAXITER; i++) {
			point2 = new Point_3D(rv.getNextRandom(),cov);
			point2.normalizeSphere();
			statistic.addObservation(point2);
		}
		if (!TestStatistics.check(point.getV(),statistic.getMue(),
									point.getCov(),statistic.getSigma(),PRECISION))
			fail();
	}

	/**
	 * Test for non-statistic part of void normalizeSphereComparable()
	 */
	public void testNormalizeSphereComparable() {
		Point_3D point = new Point_3D(-1,-1,-1,-1);
		Point_3D point2 = new Point_3D(0.5,0.5,0.5,0.5);
		point.normalizeSphereComparable();
		if (!TestStatistics.check(point.getV(),point2.getV(),PRECISION)) fail();
		
		point = new Point_3D(-7,-7,-1,-1);
		point2 = new Point_3D(0.7,0.7,0.1,0.1);
		point.normalizeSphereComparable();
		if (!TestStatistics.check(point.getV(),point2.getV(),PRECISION)) fail();
	}

	/**
	 * Test for error propagation of void normalizeSphereComparable()
	 */
	public void testNormalizeSphereComparableStatistics() {
		double[][] covArray = {{0.1,0,0,0},{0,0.1,0,0},{0,0,0.1,0},{0,0,0,0}};
		Matrix cov = LinearAlgebraFactory.getMatrix(covArray);
		Vector vec = LinearAlgebraFactory.getVector(-3,-3,-3,-1);
		Point_3D point = new Point_3D(vec,cov);
		point.normalizeSphereComparable();
		TestStatistics statistic = new TestStatistics(4);
		RandomVector rv = new RandomVector(vec,cov);
		Point_3D point2;
		for (int i = 0; i < MAXITER; i++) {
			point2 = new Point_3D(rv.getNextRandom(),cov);
			point2.normalizeSphereComparable();
			statistic.addObservation(point2);
		}
		if (!TestStatistics.check(point.getV(),statistic.getMue(),
									point.getCov(),statistic.getSigma(),PRECISION))
			fail();
	}

	/**
	 * Test for void changeNullspace()
	 */
	public void testChangeNullspace() {
		double[][] covArray = {{1,2,3,4},{2,3,5,1},{3,5,2,2},{4,1,2,3}};
		Vector vec = LinearAlgebraFactory.getVector(2,4,5,4);
		Point_3D point = new Point_3D(vec,LinearAlgebraFactory.getMatrix(covArray));
		//Calculating the Matrix cov:
		double factor = 1/(vec.scalarmult(vec));
		Matrix mat = LinearAlgebraFactory.getDiagMatrix(4,0);
		for(int i=0;i<4;i++)mat.setRow(i,vec.mult(vec.get(i)));
		Matrix psi = LinearAlgebraFactory.getMatrixId(4).minus(mat.mult(factor));
		Matrix cov = psi.mult(point.getCov().mult(psi.trans()));
		Point_3D testPoint = new Point_3D(vec,cov);
		Point_3D pointCopy = point.copy();
		point.changeNullspace();
		if (!TestStatistics.check(point.getV(),testPoint.getV(),
									point.getCov(),testPoint.getCov(),PRECISION))
			fail();
		//Check if euclidean Norms are the same
		pointCopy.normalizeEuclidean();
		point.normalizeEuclidean();
		if (!TestStatistics.check(point.getV(),pointCopy.getV(),
									point.getCov(),pointCopy.getCov(),PRECISION))
			fail();
	}

	/**
	 * Test for void checkCovDim(Matrix,int)
	 */
	public void testCheckCovDim() {
		double[][] covArray = {{1,2,3,4},{2,3,5,1},{4,1,2,2},{2,4,1,3}};
		Vector vec = LinearAlgebraFactory.getVector(2,4,5,0);
		Matrix cov = LinearAlgebraFactory.getMatrix(covArray);
		Point_3D point = new Point_3D();
		point.setV(vec);
		try {
			point.checkCovDim(cov,point.getDimension());
		}
		catch(IllegalArgumentException illArg){
			fail();
		}
		//Matrix cov with reduced number of columns
		Matrix cov2 = cov.slice(0,0,3,2);
		try {
			point.checkCovDim(cov2,point.getDimension());
			fail();
		}
		catch(IllegalArgumentException illArg){
		}
		//Matrix cov with reduced number of rows
		cov2 = cov.slice(0,0,2,3);
		try {
			point.checkCovDim(cov2,point.getDimension());
			fail();
		}
		catch(IllegalArgumentException illArg){
		}
	}

	/**
	 * Test for void checkElement(Vector,int)
	 * try checkElement and throw an IllegalArgumentException
	 * if tests in checkElement fail.
	 */
	public void testCheckElement() {
		Vector vec2d = LinearAlgebraFactory.getVector(1,0,0);
		Vector vec3d = LinearAlgebraFactory.getVector(1,0,0,0);
		try{
			Point_2D.checkElement(vec2d,3);
			Line_2D.checkElement(vec2d,3);
			Point_3D.checkElement(vec3d,4);
			Plane_3D.checkElement(vec3d,4);
		}
		catch(IllegalArgumentException illArg){
			fail();
		}
		vec2d = LinearAlgebraFactory.getVector(0,0,0);
		vec3d = LinearAlgebraFactory.getVector(0,0,0,0);
		try{
			Point_2D.checkElement(vec2d,3);
			fail();
		}
		catch(IllegalArgumentException illArg){}
		try{
			Line_2D.checkElement(vec2d,3);
			fail();
		}
		catch(IllegalArgumentException illArg){}
		try{
			Point_3D.checkElement(vec3d,4);
			fail();
		}
		catch(IllegalArgumentException illArg){}
		try{
			Plane_3D.checkElement(vec3d,4);
			fail();
		}
		catch(IllegalArgumentException illArg){}
	}

	/**
	 * Test for Object clone()
	 */
	public void testClone() {
		Vector vec = LinearAlgebraFactory.getVector(1,2,3,4);
		double[][] covArray = {{1,2,3,4},{3,7,2,6},{0,9,8,7},{7,6,5,4}};
		Matrix cov = LinearAlgebraFactory.getMatrix(covArray);
		Point_3D point = new Point_3D(vec,cov);
		Point_3D pointClone = (Point_3D) point.clone();
		if ((!point.getV().equals(pointClone.getV()))&&
			(!point.getCov().equals(pointClone.getCov())))
			fail();
		//new init of point
		vec = LinearAlgebraFactory.getVector(5,4,3,2);
		point = new Point_3D(vec,cov.trans());
		//test if clone() only makes a reference (-->fail()) or a copy
		if ((point.getV().equals(pointClone.getV()))&&
			(point.getCov().equals(pointClone.getCov())))
			fail();
	}
}