/*
 * Created on 14.06.2004
 */
package sugr.test;

import junit.framework.TestCase;
import sugr.*;
import sugr.linalg.*;

/**
 *  Test Equal Relation Methods
 * 
 * @author Achim Knigs
 */
public class EqualTest extends TestCase {

	/**
	 * Tests if the right jacobians are delivered for points, 
	 * planes and lines in 2D and 3D
	 */
	public void testGetJacobian() {
		//Point_2D:
		Point_2D p2D = new Point_2D(1,2);
		Equal eq = new Equal(p2D);
		
		//the jacobian is just the skewsym matrix of the vector
		//mult with -1
		Matrix p2DJac = p2D.getV().SkewSym().mult(-1.0);
		double p2DZero = p2DJac.minus(eq.getJacobian()).vec().getLength();
		
		//fail if wrong jacobian
		if(p2DZero != 0) fail();
		
		//Line_2D:
		Line_2D l2D = new Line_2D(2,3,5);
		eq = new Equal(l2D);
		
		//the jacobian is just the skewsym matrix of the vector
		Matrix l2DJac = l2D.getV().SkewSym().mult(-1.0);
		double l2DZero = l2DJac.minus(eq.getJacobian()).vec().getLength();
		
		//fail if wrong jacobian
		if(l2DZero != 0) fail();
		
		//Point_3D
		Point_3D p3D = new Point_3D(5,6,7,8);
		eq = new Equal(p3D);
		
		//the jacobian is PI(X): (-1.0 same as above...)
		Matrix p3DJac = p3D.Pi().mult(-1.0);
		double p3DZero = p3DJac.minus(eq.getJacobian()).vec().getLength();
		
		//fail if jacobians differed
		if(p3DZero != 0) fail();
		
		//Plane_3D:
		Plane_3D plane = new Plane_3D(8,9,10,11);
		eq = new Equal(plane);
		
		//the jacobian is PiDual(plane): (-1.0 same as above...)
		Matrix planeJac = plane.PiDual().mult(-1.0);
		double planeZero = planeJac.minus(eq.getJacobian()).vec().getLength();
		
		//fail if jacobians differed
		if(planeZero != 0) fail();
		
		//Line_3D:
		Line_3D l3D = new Line_3D(1,2,3,4,5,6);
		eq = new Equal(l3D);
		
		//Line Line Jacobian tested in Line_3DTest.
		Matrix l3DJac = l3D.getLineLineJacobian().mult(-1.0);
		double l3DZero = l3DJac.minus(eq.getJacobian()).vec().getLength();
		
		//fail if jacobians differed
		if(l3DZero != 0) fail();
	}

	/**
	 * Tests the dofs of all possible entities
	 */
	public void testGetDof() {
		//construct some EqualProperties
		Equal p2D = new Equal(new Point_2D(1,2));
		Equal l2D = new Equal(new Line_2D(2,3,4));
		Equal p3D = new Equal(new Point_3D(4,5,6,7));
		Equal plane = new Equal(new Plane_3D(8,9,10,11));
		Equal l3D = new Equal(new Line_3D(1,2,3,4,5,6));
		
		//fail if wrong DOFs are delivered
		if(p2D.getDof() != 2) fail();
		if(l2D.getDof() != 2) fail();
		if(p3D.getDof() != 3) fail();
		if(plane.getDof() != 3) fail();
		if(l3D.getDof() != 4) fail();
	}

	/**
	 * Test for Equal(Entity)
	 * This tests a Constructor of Equal, which only stores
	 * the given Entity in a RelationalProperty.
	 * If you are looking for more tests of this Relation 
	 * look in RelationTests.testDecision()
	 * and the getJacobianOfRelation() tests.
	 */
	public void testEqualEntity() {
		//construct point and equal
		Point_2D p = new Point_2D(1,2);
		Equal test = new Equal(p);
	
		//fail if Equal has different point.
		if(!TestStatistics.check(test.getFirstEntity(),p,1e-17)) fail();
	}

	/**
	 * Test for Equal(Point_2D, Class)
	 * This tests a Constructor of Equal, which only stores
	 * the given Entity in a RelationalProperty.
	 * If you are looking for more tests of this Relation 
	 * look in RelationTests.testDecision()
	 * and the getJacobianOfRelation() tests.
	 */
	public void testEqualPoint_2DClass() {
		double cov[][] = 
			{{0.01,0,0},
			 {0,0.01,0},
			 {0,0,0}};
		//construct point
		Point_2D p = new Point_2D(1,2,
				LinearAlgebraFactory.getMatrix(cov));
		//new Equal, using constructor, second class has to be Point_2D.class
		Equal eq = new Equal(p,Point_2D.class);
		
		if(!TestStatistics.check(eq.getFirstEntity(),p,1e-17)) fail();
	}

	/**
	 * Test for Equal(Line_2D, Class)
	 * This tests a Constructor of Equal, which only stores
	 * the given Entity in a RelationalProperty.
	 * If you are looking for more tests of this Relation 
	 * look in RelationTests.testDecision()
	 * and the getJacobianOfRelation() tests.
	 */
	public void testEqualLine_2DClass() {
		double cov[][] =
			{{0.01,0,0},
			 {0,0.01,0},
			 {0,0,0.01}};
		
		//construct line
		Line_2D l = new Line_2D(1,2,3,
				LinearAlgebraFactory.getMatrix(cov));
		//new Equal. Second Class has to be Line_2D.class
		Equal eq = new Equal(l,Line_2D.class);
		
		if(!TestStatistics.check(eq.getFirstEntity(),l,1e-17)) fail();
	}

	/**
	 * Test for Equal(Point_3D, Class)
	 * This tests a Constructor of Equal, which only stores
	 * the given Entity in a RelationalProperty.
	 * If you are looking for more tests of this Relation 
	 * look in RelationTests.testDecision()
	 * and the getJacobianOfRelation() tests.
	 */
	public void testEqualPoint_3DClass() {
		double cov[][] =
			{{0.01,0,0,0},
			 {0,0.01,0,0},
			 {0,0,0.01,0},
			 {0,0,0,0.01}};
		
		//construct point
		Point_3D p = new Point_3D(1,2,3,
					LinearAlgebraFactory.getMatrix(cov));
		//new Equal, using constructor, second class has to be Point_3D.class
		Equal eq = new Equal(p,Point_3D.class);
		
		if(!TestStatistics.check(eq.getFirstEntity(),p,1e-17)) fail();
	}

	/**
	 * Test for Equal(Line_3D, Class)
	 * This tests a Constructor of Equal, which only stores
	 * the given Entity in a RelationalProperty.
	 * If you are looking for more tests of this Relation 
	 * look in RelationTests.testDecision()
	 * and the getJacobianOfRelation() tests.
	 */
	public void testEqualLine_3DClass() {
		double cov[][] = 
			{{0.01,0,0,0,0,0},
			 {0,0.01,0,0,0,0},
			 {0,0,0.01,0,0,0},
			 {0,0,0,0.01,0,0},
			 {0,0,0,0,0.01,0},
			 {0,0,0,0,0,0.01}};
	
		Matrix lineCov = LinearAlgebraFactory.getMatrix(cov);
		
		//construct line
		Line_3D l = new Line_3D(1,2,3,1,1,-1,lineCov);
		//new Equal. Second Class has to be Line_3D.class
		Equal eq = new Equal(l,Line_3D.class);
		
		if(!TestStatistics.check(eq.getFirstEntity(),l,1e-17)) fail();
	}

	/**
	 * Test for Equal(Plane_3D, Class)
	 * This tests a Constructor of Equal, which only stores
	 * the given Entity in a RelationalProperty.
	 * If you are looking for more tests of this Relation 
	 * look in RelationTests.testDecision()
	 * and the getJacobianOfRelation() tests.
	 */
	public void testEqualPlane_3DClass() {
		double cov [][] = 
			{{0.01,0,0,0},
		     {0,0.01,0,0},
			 {0,0,0.01,0},
			 {0,0,0,0.01}};
		//construct plane
		Plane_3D plane = new Plane_3D(1,3,4,5,
				LinearAlgebraFactory.getMatrix(cov));
		//new Equal. Second class has to be Plane_3D.class
		Equal eq = new Equal(plane,Plane_3D.class);
		
		if(!TestStatistics.check(eq.getFirstEntity(),plane,1e-17)) fail();
	}

}
