/*
 * Created on 15.06.2004
 */
package sugr.test;

import junit.framework.TestCase;
import sugr.*;
import sugr.linalg.*;

/**
 * Class to test Parallel Relation
 * 
 * @author Achim Knigs
 */
public class ParallelTest extends TestCase {

	/**
	 * Constructor for ParallelTest.
	 * @param arg0
	 */
	public ParallelTest(String arg0) {
		super(arg0);
	}

	/**
	 * Test for getJacobian for lines and planes
	 */
	public void testGetJacobian() {
		//Line_2D to Line_2D:
		Line_2D l2 = new Line_2D(1,2,3);
		Parallel test = new Parallel(l2,Line_2D.class);
		
		//build jacobian Matrix and compare to the one from sugr
		Vector lh = l2.getHomogeneousPart();
		Matrix l2Jac = LinearAlgebraFactory.getVector(lh.get(1) * (-1.0), lh.get(0),0).getMatrix().trans();
		double l2Zero = l2Jac.minus(test.getJacobian()).vec().getLength();
		
		//matrizes differed
		if(l2Zero != 0) fail();
		
		//Line_3D to Line_3D:
		Line_3D l3 = new Line_3D(1,2,3,1,1,-1);
		test = new Parallel(l3,Line_3D.class);
		
		//build jacobian and compare to the one in sugr, Jacobian is S(Lh)
		Matrix l3Jac = LinearAlgebraFactory.getMatrix(6,6);
		l3Jac.slice(0,2,0,2,l3.getHomogeneousPart().SkewSym());
		double l3Zero = l3Jac.minus(test.getJacobian().mult(-1.0)).vec().getLength();
		
		//fail if jacobians differ
		if(l3Zero != 0) fail();
		
		//Plane_3D to Plane_3D:
		Plane_3D p3 = new Plane_3D(1,2,3,4);
		test = new Parallel(p3,Plane_3D.class);
		
		//build jacobian and compare to the one in sugr, Jacobian is S(Ah)
		Matrix p3Jac = LinearAlgebraFactory.getMatrix(4,4);
		p3Jac.slice(0,2,0,2,p3.getHomogeneousPart().SkewSym());
		double p3Zero = p3Jac.minus(test.getJacobian().mult(-1.0)).vec().getLength();
		
		//fail if jacobians differ
		if(p3Zero != 0) fail();
		
		//Line_3D to Plane_3D:
		//reuse entities from above
		test = new Parallel(l3,Plane_3D.class);
		
		//build jacobian, that is Lh^T
		Matrix plJac = LinearAlgebraFactory.getMatrix(1,4);
		plJac.slice(0,0,0,2,l3.getHomogeneousPart().getMatrix().trans());
		double plZero = plJac.minus(test.getJacobian()).vec().getLength();
		
		//fail if jacobians differ
		if(plZero != 0) fail();
	}

	/**
	 * Test for getDof
	 */
	public void testGetDof() {
		//construct some entities
		Parallel l2 = new Parallel(new Line_2D(1,2,3),Line_2D.class);
		Parallel l3 = new Parallel(new Line_3D(5,3,2,-1,1,1),Line_3D.class);
		Parallel plane = new Parallel(new Plane_3D(1,2,34,4),Plane_3D.class);
		Parallel pl = new Parallel(new Line_3D(5,3,2,-1,1,1),Plane_3D.class);
		
		//test dofs:
		if(l2.getDof() != 1) fail();
		if(l3.getDof() != 2) fail();
		if(plane.getDof() != 2) fail();
		if(pl.getDof() != 1) fail();
	}

	/**
	 * Test for Parallel(Line_2D, Class)
	 * This tests a Constructor of Parallel, which only stores
	 * the given Entity in a RelationalProperty.
	 * If you are looking for more tests of this Relation 
	 * look in RelationTests.testDecision()
	 * and the getJacobianOfRelation() tests.
	 */
	public void testParallelLine_2DClass() {
		//construct Parallel
		Line_2D l = new Line_2D(1,2,4,
				LinearAlgebraFactory.getDiagMatrix(3,0.01));
		//second parameter has to be Line_2D.class
		Parallel par = new Parallel(l,Line_2D.class);
		
		if(!TestStatistics.check(par.getFirstEntity(),l,1e-17))
			fail();
	}

	/**
	 * Test for Parallel(Line_3D, Class)
	 * This tests a Constructor of Parallel, which only stores
	 * the given Entity in a RelationalProperty.
	 * If you are looking for more tests of this Relation 
	 * look in RelationTests.testDecision()
	 * and the getJacobianOfRelation() tests.
	 */
	public void testParallelLine_3DClass() {
		//construct Parallel
		Line_3D l = new Line_3D(2,2,4,1,1,-1,
				LinearAlgebraFactory.getDiagMatrix(6,0.01));
		//second parameter has to be Line_3D.class or Plane_3D.class.
		Parallel par = new Parallel(l,Line_3D.class);
		
		if(!TestStatistics.check(par.getFirstEntity(),l,1e17))
			fail();
	}

	/**
	 * Test for Parallel(Plane_3D, Class)
	 * This tests a Constructor of Parallel, which only stores
	 * the given Entity in a RelationalProperty.
	 * If you are looking for more tests of this Relation 
	 * look in RelationTests.testDecision()
	 * and the getJacobianOfRelation() tests.
	 */
	public void testParallelPlane_3DClass() {
		//construct Parallel
		Plane_3D p = new Plane_3D(1,2,4,7,
				LinearAlgebraFactory.getDiagMatrix(4,0.01));
		//second parameter has to be Line_3D.class or Plane_3D.class.
		Parallel par = new Parallel(p,Line_3D.class);
		
		if(!TestStatistics.check(par.getFirstEntity(),p,1e-17))
			fail();
	}

}
