/*
 * Created on 11.02.2004
 *
 * Contains test for Point_2D.
 */
package sugr.test;

import sugr.*;
import sugr.linalg.*;
import junit.framework.TestCase;

/**
 * This class tests the Point_2D class. The most important test is the test of 
 * the constructor Point_2D(Line_2D,Line_2D); 
 * Errorpropagation is tested with the help of the TestStatistics class, which 
 * calculates sigma in many iterations from gaussian random numbers.
 * @author Achim Knigs
 */
public class Point_2DTest extends TestCase {

	/**
	 * Constructor for Point_2DTest.
	 * @param arg0
	 */
	public Point_2DTest(String arg0) {
		super(arg0);
	}
	
	/**
	 * int MAXITER determines how many iterations there are for calculation of
	 * the statstic parts. It should be around 100 000 for good results, but
	 * this may mean much work for your maschine and some coffee for you,
	 * if you are waiting for the results.
	 */
	public static int MAXITER = 100000;
	
	/**
	 * Method to test the statistic reasoning of a point out of 
	 * two uncertain lines.
	 * @param line1 - first line to be crossed with second line
	 * @param line2 - second line
	 * @return - true if test is successful
	 */
	public boolean helpPoint_2DLine_2DLine_2DStatistics(Line_2D line1, Line_2D line2,double sigma){
		//Create Statistics Object
		TestStatistics stat = new TestStatistics(3);
		
		//Two Random Vectors for Random Line generation
		RandomVector vl1 = new RandomVector(line1.getV(),line1.getCov());
		RandomVector vl2 = new RandomVector(line2.getV(),line2.getCov());
		
		//a temp point
		Point_2D cross;
		
		//ProgressMonitor
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing statistic reasoning of a point",
														"out of two lines",0,MAXITER);	
		
		//iteration. should be close to calculated values after
		//100,000 iterations
		for(int i = 0; i < MAXITER; i++)
		{
			Line_2D tline1 = new Line_2D(vl1.getNextRandom(),line1.getCov());
			Line_2D tline2 = new Line_2D(vl2.getNextRandom(),line2.getCov());
			
			cross = new Point_2D(tline1,tline2);
			stat.addObservation(cross);
			
			//break if cancel button is pressed
			if(pm.isCanceled()) break;
			
			//update progress monitor.
			pm.setProgress(i);
		}
		//close progress monitor
		pm.close();
		
		//calculate values
		cross = new Point_2D(line1,line2);
		
		//normalize.
		Point_2D estimated = new Point_2D(stat.getMue(),stat.getSigma());
		cross.normalizeEuclidean();
		estimated.normalizeEuclidean();

		//compare and return true if equal
		boolean retval = TestStatistics.check(cross,estimated,sigma);
		
		if(!retval) {
			System.out.println("Failed Constructor: Point_2D(Line_2D,Line_2D)");
			System.out.println("SUGR Point: " + cross);
			System.out.println("Calculated Point: " + estimated);
 		}
 		return retval;
	}
	
	/**
	 * Test for getDegreesOfFreedom();
	 * Test's Degrees of Freedom, should be 2,
	 * and tries another Point, which should
	 * return the same value.
	 */	
	public void testGetDegreesOfFreedom() {
		//constructing points
		Point_2D punkt = new Point_2D(4,21,16);
		Point_2D punkt2 = new Point_2D(5,7);
		
		//using getDegreesOfFreedom
		int dof = punkt.getDegreesOfFreedom();
		int dof2 = punkt2.getDegreesOfFreedom();
		
		//test if it gets the right value and if the value is the same 
		//on every point
		if(dof != 2) fail();
		if(dof != dof2) fail();
	}

	/**
	 * Test for getDimension();
	 * Test dimension of one point, which should be three
	 * then it takes another one, and test if they have the 
	 * same dimensions, as proposed.
	 */
	public void testGetDimension() {
		//consructing points
		Point_2D punkt = new Point_2D(Double.MAX_VALUE,Double.MIN_VALUE);
		Point_2D punkt2 = new Point_2D(2,5.754,32.67);
		
		//using getDimension
		int dim = punkt.getDimension();
		int dim2 = punkt2.getDimension();
		
		//test the value and if the value is the same everytime
		if(dim != 3) fail();
		if(dim != dim2) fail();
	}

	/**
	 * Test for getEuclideanPart();
	 * Generates two Points. Get's the eucledean Part 
	 * from each and compares them two the values the should have.
	 */
	public void testGetEuclideanPart() {
		//constructing points
		Point_2D punkt = new Point_2D(2,7,3);
		Point_2D punkt2 = new Point_2D(4,14,7);
		Point_2D punkt3 = new Point_2D(3,4,0);
		
		//using getEuclideanPart
		Vector eucl = punkt.getEuclideanPart();
		Vector eucl2 = punkt2.getEuclideanPart();
		Vector eucl3 = punkt3.getEuclideanPart();
		
		//getting test Vector
		Vector euclTest = LinearAlgebraFactory.getVector(2,7);
		Vector euclTest2 = LinearAlgebraFactory.getVector(4,14);
		Vector euclTest3 = LinearAlgebraFactory.getVector(3,4);
		
		//test the values:
		if(!TestStatistics.check(eucl,euclTest,0.00000001)) fail();
		if(!TestStatistics.check(eucl2,euclTest2,0.00000001)) fail();
		if(!TestStatistics.check(eucl3,euclTest3,0.00000001)) fail();
	}

	/**
	 * Test for getHomogenousePart()
	 * Takes two Points and tests the returned 
	 * homogenous Parts.
	 */
	public void testGetHomogeneousPart() {
		//constructing points
		Point_2D punkt = new Point_2D(65,23);
		Point_2D punkt2 = new Point_2D(65,23,Double.MAX_VALUE);
		Point_2D punkt3 = new Point_2D(3,4,0);
		
		//using getHomogeneousPart
		Vector hom = punkt.getHomogeneousPart();
		Vector hom2 = punkt2.getHomogeneousPart();
		Vector hom3 = punkt3.getHomogeneousPart();
		
		//test
		if(hom.get(0) != 1) fail();
		if(hom2.get(0) != Double.MAX_VALUE) fail();
		if(hom3.get(0) != 0) fail();
	}

	 /**
	  * Test for Matrix getJacobianOfRelation(Class, Class)
	  * First tests the returned matrices if they fullfill the constraint 
	  * f(x)*y = 0.
	  * Then it tests if the matrices have the right values.
	  * At last it tests if getJacobianOfRelation throws an exception
	  * for every forbidden combination of classes.
	  */
	public void testGetJacobianOfRelationClassClass() {
		//constructing Points
		Point_2D geradenPunkt1 = new Point_2D(0,0,1);
		Point_2D geradenPunkt2 = new Point_2D(1,1,1);
		Point_2D nichtGeradenPunkt = new Point_2D(2,1,1);
		Point_2D glPunkt1 = new Point_2D(7,6,1);
		Point_2D glPunkt2 = new Point_2D(14,12,2);
		
		//constructing a line out of 2 points
		Line_2D gerade = new Line_2D(geradenPunkt1,geradenPunkt2);
		
		//getting the Jacobians
		Matrix geradePunkt1 = geradenPunkt1.getJacobianOfRelation(Incident.class, Line_2D.class);
		Matrix geradePunkt2 = geradenPunkt2.getJacobianOfRelation(Incident.class, Line_2D.class);
		Matrix gl = glPunkt1.getJacobianOfRelation(Equal.class, Point_2D.class);
		Matrix notIncP = nichtGeradenPunkt.getJacobianOfRelation(Incident.class, Line_2D.class); 
		
		//test the Jacobians, that means, test if they fullfill the constraint:
		Matrix inc = geradePunkt1.mult(gerade.getV().getMatrix());
		Matrix inc2 = geradePunkt2.mult(gerade.getV().getMatrix());
		Matrix eq = gl.mult(glPunkt2.getV().getMatrix());
		Matrix ninc = notIncP.mult(gerade.getV().getMatrix());
		Matrix neq = gl.mult(geradenPunkt1.getV().getMatrix());
		
		//fail if not null:
		if(inc.vec().getLength() != 0) fail();
		if(inc2.vec().getLength() != 0) fail();
		if(eq.vec().getLength() != 0) fail();
		
		//fail if null, because it shouldn't work:
		if(ninc.vec().getLength() == 0) fail();
		if(neq.vec().getLength() == 0) fail();
		
		//try every forbidden combination:
		Class[][] tests = {	{Equal.class,Point_2D.class},
							{Incident.class,Line_2D.class}}; 
		
		if(!TestTools.helpJacobianOfRelations(tests,glPunkt1)) fail();
		
		//calculate Jacobians and compare them to the ones form sugr
		//a new Point
		Point_2D p = new Point_2D(6,9,3);
		
		//let sugr calculate the matrices
		Matrix incP1 = p.getJacobianOfRelation(Incident.class, Line_2D.class);
		Matrix eqP1 = p.getJacobianOfRelation(Equal.class, Point_2D.class);
		
		//do it yourself:
		//Point and Line are incident, if x^T*l = 0 is true. So we need
		//x^T here
		Matrix incP2 = p.getV().getMatrix().trans();
		
		//two points are equal, when the cross produkt is zero.
		//so we can use the skewsym matrix of p to get the jacobian.
		Matrix eqP2 = p.getV().SkewSym();
		
		//take differenz between matrizes:
		Matrix diffInc = incP1.minus(incP2);
		Matrix diffEq = eqP1.minus(eqP2);
		
		//fail if not null
		if(diffInc.vec().getLength() != 0) fail();
		if(diffEq.vec().getLength() != 0) fail();
	}

	/**
	 * int getDofOfRelation(Class, Class)
	 * Tests the dimmensions of freedom for the Relations.
	 * Tests also if function throws exception for every forbidden combination.
	 */
	public void testGetDofOfRelation() {
		//construct point
		Point_2D irgendwas = new Point_2D(3,4,5);
		
		//use getDofOfRelation
		int point_line = irgendwas.getDofOfRelation(Incident.class, Line_2D.class);
		int point_point = irgendwas.getDofOfRelation(Equal.class, Point_2D.class);
		
		//test values
		if(point_line != 1) fail();
		if(point_point != 2) fail();
		
		//try unallowed combinations
		Class[][] test = {{Incident.class,Line_2D.class},{Equal.class,Point_2D.class}};
		if(!TestTools.helpDofOfRelations(test,irgendwas)) fail();
	}

	/**
	 * Test of normalizeEuclidean(),
	 * homogenous Part should be 1 here after
	 * tests over one million points without covariances.
	 * then tests some points with covariances to test 
	 * error propagation.
	 */
	public void testNormalizeEuclidean() {
		//point with homogenous part zero
		Point_2D x_axis  = new Point_2D(1,0,0);

		//doesn't throw an error, but 
		//all values are NaN. 
		x_axis.normalizeEuclidean(); 
		
		//ProgressMonitor
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing non statistic part",
														"of normalizeEuclidean",0,1000*1000);	
		
		//Test over 1 Million Points
		outer:
		for(int i = 0; i < 1000; i++) {
			for(int j = 0; j < 1000; j++){
				//new Point
				Point_2D punkt = new Point_2D(i-100,(j+100)%1000,j-500.5);
				
				//normalize
				punkt.normalizeEuclidean();
				
				//calculate normalization by myself.
				Vector v = LinearAlgebraFactory.getVector((i-100)/(j-500.5),((j+100)%1000)/(j-500.5),1);
				if(!TestStatistics.check(punkt.getV(),v,0.00000001)) {
					pm.close();
					fail();
				}
				
				//break if cancel button is pressed
				if(pm.isCanceled()) break outer;
				
				//update progress Monitor
				pm.setProgress(i*j);
			} 
		}
		//close progress monitor
		pm.close();
		
		//Test Statistics
		Point_2D p1 = new Point_2D(2,6,2.6,LinearAlgebraFactory.getDiagMatrix(3, 0.02));
		if(!TestTools.helpNormalizeEuclideanStatistics(p1,MAXITER)) fail();
		
		double[][] cov = {{0.04,0.01,0.02},{0.01,0.05,0.03},{0.02,0.03,0.07}};
		Point_2D p2 = new Point_2D(264,654,-67,LinearAlgebraFactory.getMatrix(cov));
		if(!TestTools.helpNormalizeEuclideanStatistics(p2,MAXITER)) fail();
		
		Point_2D p3 = new Point_2D(-54,76,24,LinearAlgebraFactory.getMatrix(cov));
		if(!TestTools.helpNormalizeEuclideanStatistics(p3,MAXITER)) fail();
	}

	/**
	 * Test of getDistance() 
	 * calculates the distance for some points at infinity
	 * then calculates the distance between two points 
	 * for one million times.
	 * Tests distance to a line.
	 */
	public void testGetDistance() {
		//construct points - a special distance
		Point_2D punkt1 = new Point_2D(2,8);
		Point_2D punkt2 = new Point_2D(7,5,0);
		
		//construct a line
		Line_2D l1 = new Line_2D(0,1,1);
		
		//use getDistance
		double infiniteDistance = punkt1.getDistance(punkt2);	
		double distanceToLine = punkt1.getDistance(l1);
		
		//do it yourself:
		double distanceToLineContr = Math.abs(l1.getV().scalarmult(punkt1.getV()))
									 / (l1.getHomogeneousPart().mult(punkt1.getHomogeneousPart().get(0)).getLength());
		
		
		//fail if wrong values
		if(infiniteDistance != Double.POSITIVE_INFINITY) fail();
		if(distanceToLine != distanceToLineContr) fail();

		//ProgressMonitor
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing distance between a point",
														"and a point and a line",0,1000*1000);	
		
		//Test over 1 Million Points
		outer:
		for(int i = 0; i < 1000; i++) {
			for(int j = 0; j < 1000; j++){
				Point_2D p1 = new Point_2D(i-100,(j+100)%1000,j-500.5);
 				Point_2D p2 = new Point_2D((i+j)-1000,j-500,i-500.5);
				double dist1 = p1.getDistance(p2);
				double dist2 = p2.getDistance(p1);

				//Yo * xh:
				Vector v = p2.getEuclideanPart().mult(p1.getHomogeneousPart().get(0));  
				//+ Xo * yh:
				v = v.minus(p1.getEuclideanPart().mult(p2.getHomogeneousPart().get(0)));
				// |v| / xh*yh:
				double dist3 = Math.abs(v.getLength()) / Math.abs(p1.getHomogeneousPart().scalarmult(p2.getHomogeneousPart()));
				if(dist1 != dist2 || dist1 != dist3 || dist2 != dist3) {
					pm.close();
					fail();
				}
				
				if(pm.isCanceled()) break outer;
				pm.setProgress(i*j);
			} 
		}
		pm.close();
	}

	/**
	 * Test for void Point_2D(Vector, Matrix)
	 */
	public void testPoint_2DVectorMatrix() {
		//constructing Point:
		Vector v = LinearAlgebraFactory.getVector(3, 5, 1);
		Matrix cov = LinearAlgebraFactory.getDiagMatrix(3, 0.009);
		Point_2D punkt = new Point_2D(v,cov);
		
		//constructing control values:
		Vector kontr = LinearAlgebraFactory.getVector(3, 5, 1);
		Matrix kontrCov = LinearAlgebraFactory.getDiagMatrix(3,0.009);
		
		//nulling: 
		Vector diff = punkt.getV().minus(kontr);
		Vector diffCov = (punkt.getCov().minus(kontrCov)).vec();
		
		//fail if not null:
		if(diff.getLength() != 0) fail();
		if(diffCov.getLength() != 0) fail();
	}

	/**
	 * Test for void Point_2D(double, double)
	 */
	public void testPoint_2Ddoubledouble() {
		//constructing point
		Point_2D punkt = new Point_2D(2,3);
		
		//control vector
		Vector v = LinearAlgebraFactory.getVector(2, 3, 1);
		
		//nulling:
		Vector diff = punkt.getV().minus(v);
		Vector covs = punkt.getCov().vec();
		
		//fail if not null 
		if(diff.getLength() != 0) fail();
		if(covs.getLength() != 0) fail();
	}

	/**
	 * Test for void Point_2D(double, double, Matrix)
	 */
	public void testPoint_2DdoubledoubleMatrix() {
		//constructing Point:
		Matrix cov = LinearAlgebraFactory.getDiagMatrix(3,0.009);
		Point_2D punkt = new Point_2D(2,4,cov);
		
		//control values:
		Vector v = LinearAlgebraFactory.getVector(2, 4, 1);
		
		//nulling:
		Vector diff = punkt.getV().minus(v);
		Vector diffCov = (punkt.getCov().minus(cov)).vec();
		
		//fail if not null:
		if(diff.getLength() != 0) fail();
		if(diffCov.getLength() != 0) fail();
	}

	/**
	 * Test for void Point_2D(double, double, double)
	 */
	public void testPoint_2Ddoubledoubledouble() {
		//constructing Point:
		Point_2D punkt = new Point_2D(6,3,5);
		
		//control values:
		Vector v = LinearAlgebraFactory.getVector(6, 3, 5);
		
		//nulling:
		Vector diff = punkt.getV().minus(v);
		
		//fail if not null
		if(diff.getLength() != 0) fail();
	}

	/**
	 * Test for void Point_2D(double, double, double, Matrix)
	 */
	public void testPoint_2DdoubledoubledoubleMatrix() {
		//constructing Point:
		Matrix cov = LinearAlgebraFactory.getDiagMatrix(3,0.009);
		Point_2D punkt = new Point_2D(2,4,3,cov);
		
		//control values:
		Vector v = LinearAlgebraFactory.getVector(2, 4, 3);
		
		//nulling:
		Vector diff = punkt.getV().minus(v);
		Vector diffCov = (punkt.getCov().minus(cov)).vec();
		
		//fail if not null:
		if(diff.getLength() != 0) fail();
		if(diffCov.getLength() != 0) fail();
	}
	
	/**
	 * Test for void Point_2D(x,y,sigma_x,sigma_y);
	 */
	public void testPoint_2Ddoubledoubledoubledouble() {
		//constructing Point:
		Point_2D punkt = new Point_2D(2,4, 0.1, 0.2);
		
		
		// Euclidean normalization must have no effect
		punkt.normalizeEuclidean();

		//control values:
		Vector v = LinearAlgebraFactory.getVector(2, 4, 1);
		Matrix cov = LinearAlgebraFactory.getMatrix(3,3);
		cov.set(0,0, 0.1*0.1);
		cov.set(1,1, 0.2*0.2);
		
		//nulling:
		Vector diff = punkt.getV().minus(v);
		Vector diffCov = (punkt.getCov().minus(cov)).vec();
		
		//fail if not null:
		if(diff.getLength() != 0) fail();
		if(diffCov.getLength() != 0) fail();
	}

	/**
	 * Test for void Point_2D(Line_2D, Line_2D)
	 * Includes test for the cov-matrices
	 */
	public void testPoint_2DLine_2DLine_2D() {
		//cov values
		double[][] values1 = {	{0.0155,0.0002,0.0003},
								{0.0002,0.0155,0.0004},
								{0.0003,0.0004,0.0155}};
		double[][] values2 = {	{0.0155,0,0},
								{0,0.0155,0},
								{0,0,0.0155}};
		double[][] values3 = {	{0.0155,0.0008,0.0009},
								{0.0008,0.0155,0.000001},
								{0.0009,0.000001,0.0155}};
				
		//construct lines
		Line_2D line11 = new Line_2D(9978,-8574,1456,LinearAlgebraFactory.getMatrix(values1));
		Line_2D line12 = new Line_2D(-2423,3154,7466,LinearAlgebraFactory.getMatrix(values3));
		
		
		Line_2D line21 = new Line_2D(2.6,8.8,1,LinearAlgebraFactory.getMatrix(values2));
		Line_2D line22 = new Line_2D(0.743,4.3,1.4423,LinearAlgebraFactory.getMatrix(values1));
		
		Line_2D line31 = new Line_2D(-64,-23,1.533,LinearAlgebraFactory.getMatrix(values3));
		Line_2D line32 = new Line_2D(12,4,LinearAlgebraFactory.getMatrix(values2));
		
		//fails.. are my values soooo bad?? 
		if(!helpPoint_2DLine_2DLine_2DStatistics(line21,line22,0.3)) fail();
		if(!helpPoint_2DLine_2DLine_2DStatistics(line31,line32,0.3)) fail();
		if(!helpPoint_2DLine_2DLine_2DStatistics(line11,line12,0.3)) fail();
	}

	/**
	 * Test for void Point_2D()
	 */
	public void testPoint_2D() {
		//conscruct an empty point
		Point_2D punkt = new Point_2D();
		
		//test if point exists
		if(punkt == null) fail();
	}

	/**
	 * Test for copy()
	 */
	public void testCopy() {
		//constructing point
		Point_2D p1 = new Point_2D(1,1,1,LinearAlgebraFactory.getDiagMatrix(3, 0.04));
		
		//copy point
		Point_2D p2 = p1.copy();
		
		//change first point
		p1.getV().set(0, 2);
		
		//nulling
		Vector diff = p1.getV().minus(p2.getV());
		Vector v = LinearAlgebraFactory.getVector(1, 1, 1);
		Vector diff2 = p2.getV().minus(v);
		Vector diff3 = (p2.getCov().minus(p1.getCov())).vec();
		
		//fail if null, because second point would be affected by changes after copy
		if(diff.getLength() == 0) fail();	
		//fail if not null, because copy would be wrong
		if(diff2.getLength() != 0) fail();
		//fail if not null, because copy of cov's would be wrong
		if(diff3.getLength() != 0) fail();
	}

	/**
	 * Test for Matrix SkewSym(Vector)
	 */
	public void testSkewSymVector() {
		//Static Method -> no Point ;)
		Vector v = LinearAlgebraFactory.getVector(3, 2, 1);
		Matrix skewV = Point_2D.SkewSym(v);
		
		//control value:
		double[][] values = {{0,-1,2},{1,0,-3},{-2,3,0}};
		Matrix kontr = LinearAlgebraFactory.getMatrix(values);
		
		//nulling:
		Vector diff = (kontr.minus(skewV)).vec();
		
		//fail if not null
		if(diff.getLength() != 0) fail();
	}

	/**
	 * Test for Matrix SkewSym()
	 */
	public void testSkewSym() {
		//constructing Point:
		Point_2D v = new Point_2D(3, 2, 1);
		Matrix skewV = v.SkewSym();
		
		//control value:
		double[][] values = {{0,-1,2},{1,0,-3},{-2,3,0}};
		Matrix kontr = LinearAlgebraFactory.getMatrix(values);
		
		//nulling:
		Vector diff = (kontr.minus(skewV)).vec();
		
		//fail if not null
		if(diff.getLength() != 0) fail();	
	}

	/**
	 * Test for getDistanceToOrigin()
	 */
	public void testGetDistanceToOrigin() {
		//ProgressMonitor
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing distance of points",
														"to origin",0,1000*1000);	
		
		//Test over 1 Million Points
		outer:
		for(int i = 0; i < 1000; i++) {
			for(int j = 0; j < 1000; j++){
				//constructing point
				Point_2D p1 = new Point_2D(i-100,(j+100)%1000,j-500.5);
				Point_2D p2 = new Point_2D((i+j)-1000,j-500,i-500.5);
		
				//using get Distance to Origin
				double dist1 = p1.getDistanceToOrigin();
				double dist2 = p2.getDistanceToOrigin();

				//calculate distance for myself
				double dist1Contr = p1.getEuclideanPart().mult(1/p1.getHomogeneousPart().get(0)).getLength();
				double dist2Contr = p2.getEuclideanPart().mult(1/p2.getHomogeneousPart().get(0)).getLength();
				
				//nulling: (needed because of exactness of double)
				double dist1val = Math.abs(dist1 - dist1Contr);
				double dist2val = Math.abs(dist2 - dist2Contr);
				
				//control values
				if(dist1val > 0.000000000001 || dist2val > 0.000000000001) {
					System.out.println("Dist1: " + dist1 + " \t\t Kontr: " + dist1Contr);
					System.out.println("Dist2: " + dist2 + " \t\t Kontr: " + dist2Contr);
					pm.close();
					fail();
				}
				
				if(pm.isCanceled()) break outer;
				pm.setProgress(i*j);
			} 
		}
		pm.close();
		
		//special distances:
		Point_2D p1 = new Point_2D(1,2,0);
		Point_2D p2 = new Point_2D(0,0,1);
		
		//distance to origin:
		double dist1 = p1.getDistanceToOrigin();
		double dist2 = p2.getDistanceToOrigin();
		
		//control values:
		if(dist1 != Double.POSITIVE_INFINITY) fail();
		if(dist2 != 0) fail();
	}
}
