package sugr.test;

import sugr.*;
import sugr.linalg.*;
import junit.framework.TestCase;

/**
 * Tests for class Point_3D. Most important are the constructors 
 * which construct a point out of other entities. 
 * Uses TestStatistics.class and TestTools.class.
 * 
 * @author Achim Knigs
 *  
 */
public class Point_3DTest extends TestCase {

	/**
	 * Constructor for Point_3DTest.
	 * 
	 * @param arg0
	 */
	public Point_3DTest(String arg0) {
		super(arg0);
	}

	/**
	 * int MAXITER determines how many iterations there are for calculation of
	 * the statstic parts. It should be around 1 000 000 for good results, but
	 * this may mean much work for your maschine and some coffee for you, if you
	 * are waiting for the results.
	 */
	public static int MAXITER = 100000;

	/**
	 * Method to test the statistic reasoning of a point out of a plane and a
	 * lines.
	 * 
	 * @param plane -
	 *            plane to cross with line
	 * @param line -
	 *            line to cross with plane
	 * @return - true if test is successful
	 */
	public boolean helpPoint_3DPlane_3DLine_3DStatistics(Plane_3D plane,
			Line_3D line, double sigma) {
		//Create Statistics Object
		TestStatistics stat = new TestStatistics(4);

		//Two Random Vectors for Random Line generation
		RandomVector vl1 = new RandomVector(plane.getV(), plane.getCov());
		RandomVector vl2 = new RandomVector(line.getV(), line.getCov());

		//a temp point
		Point_3D cross;

		//progress Monitor
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
				"Testing reasoning of a 3D point", "out of a plane and a line",
				0, MAXITER);

		//iteration. should be close to calculated values after
		//100,000 iterations
		for (int i = 0; i < MAXITER; i++) {
			//get "gaussian" lines
			Plane_3D tplane = new Plane_3D(vl1.getNextRandom(), plane.getCov());
			Line_3D tline = new Line_3D(vl2.getNextRandom(), line.getCov());

			//calculate point and add observation
			cross = new Point_3D(tplane, tline);
			stat.addObservation(cross);

			//update progress monitor
			pm.setProgress(i);

			//break if cancel button is pressed
			if (pm.isCanceled())
				break;
		}
		//close progress monitor
		pm.close();

		//calculate values
		cross = new Point_3D(plane, line);

		Point_3D estimated = new Point_3D(stat.getMue(), stat.getSigma());

		//normalize Points
		cross.normalizeEuclidean();
		estimated.normalizeEuclidean();

		//compare and return true if equal
		boolean retval = TestStatistics.check(cross, estimated, sigma);

		if (!retval) {
			System.out
					.println("Failed Constructor: Point_3D(Plane_3D,Line_3D)");
			System.out.println("SUGR Point: " + cross);
			System.out.println("Calculated Point: " + estimated);
		}
		return retval;
	}

	/**
	 * Method to test the statistic reasoning of a point out of a line and a
	 * plane.
	 * 
	 * @param line -
	 *            line to cross with plane
	 * @param plane -
	 *            plane to cross with line.
	 * @return - true if test is successful
	 */
	public boolean helpPoint_3DLine_3DPlane_3DStatistics(Line_3D line,
			Plane_3D plane, double sigma) {
		//Create Statistics Object
		TestStatistics stat = new TestStatistics(4);

		//Two Random Vectors for Random Line generation
		RandomVector vl1 = new RandomVector(line.getV(), line.getCov());
		RandomVector vl2 = new RandomVector(plane.getV(), plane.getCov());

		//a temp point
		Point_3D cross;

		//progress monitor
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
				"Testing reasoning of a point", "out of a line and a plane", 0,
				MAXITER);

		//iteration. should be close to calculated values after
		//100,000 iterations
		for (int i = 0; i < MAXITER; i++) {
			//get "gaussian" lines
			Line_3D tline = new Line_3D(vl1.getNextRandom(), line.getCov());
			Plane_3D tplane = new Plane_3D(vl2.getNextRandom(), plane.getCov());

			//calculate point and add observation
			cross = new Point_3D(tline, tplane);
			stat.addObservation(cross);

			//update progress monitor
			pm.setProgress(i);

			//break if cancel button is pressed
			if (pm.isCanceled())
				break;
		}
		//close progress monitor
		pm.close();

		//calculate values
		cross = new Point_3D(line, plane);

		Point_3D estimated = new Point_3D(stat.getMue(), stat.getSigma());

		//normalize points
		cross.normalizeEuclidean();
		estimated.normalizeEuclidean();

		//compare and return true if equal
		boolean retval = TestStatistics.check(cross, estimated, sigma);

		if (!retval) {
			System.out
					.println("Constructor failed: Point_3D(Line_3D,Plane_3D)");
			System.out.println("SUGR Point: " + cross);
			System.out.println("Calculated Point: " + estimated);
		}
		return retval;
	}

	/**
	 * Method to test the statistic reasoning of a point out of three uncertain
	 * planes.
	 * 
	 * @param plane1 -
	 *            first plane
	 * @param plane2 -
	 *            second plane
	 * @param plane3 -
	 *            third plane
	 * @return - true if test is successful
	 */
	public boolean helpPoint_3DPlane_3DPlane_3DPlane_3DStatistics(
			Plane_3D plane1, Plane_3D plane2, Plane_3D plane3, double sigma) {
		//Create Statistics Object
		TestStatistics stat = new TestStatistics(4);

		//Two Random Vectors for Random Line generation
		RandomVector vl1 = new RandomVector(plane1.getV(), plane1.getCov());
		RandomVector vl2 = new RandomVector(plane2.getV(), plane2.getCov());
		RandomVector vl3 = new RandomVector(plane3.getV(), plane3.getCov());

		//a temp point
		Point_3D cross;

		//progress monitor
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
				"Testing reasoning of a point", "out of three planes.", 0,
				MAXITER);

		//iteration. should be close to calculated values after
		//100,000 iterations
		for (int i = 0; i < MAXITER; i++) {
			//get "gaussian" lines
			Plane_3D tplane1 = new Plane_3D(vl1.getNextRandom(), plane1
					.getCov());
			Plane_3D tplane2 = new Plane_3D(vl2.getNextRandom(), plane2
					.getCov());
			Plane_3D tplane3 = new Plane_3D(vl3.getNextRandom(), plane3
					.getCov());

			//calculate point and add observation
			cross = new Point_3D(tplane1, tplane2, tplane3);
			stat.addObservation(cross);

			//update progress monitor
			pm.setProgress(i);

			//break if cancel button is pressed
			if (pm.isCanceled())
				break;
		}
		//close progress monitor
		pm.close();

		//calculate values
		cross = new Point_3D(plane1, plane2, plane3);

		Point_3D estimated = new Point_3D(stat.getMue(), stat.getSigma());

		//normalize points
		cross.normalizeEuclidean();
		estimated.normalizeEuclidean();

		//compare and return true if equal
		boolean retval = TestStatistics.check(cross, estimated, sigma);

		if (!retval) {
			System.out
					.println("Failed Constructor: Point_3D(Plane_3D,Plane_3D,Plane_3D)");
			System.out.println("SUGR Point: " + cross);
			System.out.println("Calculated Point: " + estimated);
		}
		return retval;
	}

	/**
	 * Test for getDegreesOfFreedom()
	 */
	public void testGetDegreesOfFreedom() {
		//construct points
		Point_3D p1 = new Point_3D(2, 3, 4, 1);
		Point_3D p2 = new Point_3D(1.5, 7.4, 6.4);

		//use getDegreesOfFreedom()
		double dof1 = p1.getDegreesOfFreedom();
		double dof2 = p2.getDegreesOfFreedom();

		//test values
		if (dof1 != 3)
			fail();
		if (dof1 != dof2)
			fail();
	}

	/**
	 * Test for getDimension()
	 */
	public void testGetDimension() {
		//construct points
		Point_3D p1 = new Point_3D(5, 4.3251, 3, 4, LinearAlgebraFactory
				.getDiagMatrix(4, 0.05));
		Point_3D p2 = new Point_3D(7, 5, 465);

		//use getDimension()
		int dim1 = p1.getDimension();
		int dim2 = p2.getDimension();

		//test values
		if (dim1 != 4)
			fail();
		if (dim1 != dim2)
			fail();
	}

	/**
	 * Test for getEuclideanPart()
	 */
	public void testGetEuclideanPart() {
		//construct points
		Point_3D p1 = new Point_3D(1, 2, 3, 4);
		Point_3D p2 = new Point_3D(-43, 21.6, 25 * 3);

		//use getEuclideanPart()
		Vector eucl1 = p1.getEuclideanPart();
		Vector eucl2 = p2.getEuclideanPart();

		//preparing test
		Vector kontr1 = LinearAlgebraFactory.getVector(1, 2, 3);
		Vector kontr2 = LinearAlgebraFactory.getVector(-43, 21.6, 25 * 3);
		Vector diff1 = eucl1.minus(kontr1);
		Vector diff2 = eucl2.minus(kontr2);

		//fail if not null
		if (diff1.getLength() != 0)
			fail();
		if (diff2.getLength() != 0)
			fail();
	}

	/**
	 * Test for getHomogeneousPart()
	 */
	public void testGetHomogeneousPart() {
		//construct points
		Point_3D p1 = new Point_3D(1, 2, 3, 4);
		Point_3D p2 = new Point_3D(-43, 21.6, 25 * 3, 0);
		Point_3D p3 = new Point_3D(4, 5, 2);

		//use getEuclideanPart()
		Vector hom1 = p1.getHomogeneousPart();
		Vector hom2 = p2.getHomogeneousPart();
		Vector hom3 = p3.getHomogeneousPart();

		//control values
		if (hom1.get(0) != 4)
			fail();
		if (hom2.get(0) != 0)
			fail();
		if (hom3.get(0) != 1)
			fail();
	}

	/**
	 * Test for Matrix getJacobianOfRelation(Class, Class). 
	 * Tests the calculation of Jacobians for some cases with the use of 
	 * the constraint f(x)*y = x * f(y) = 0. 
	 * Then calculates the jacobians and compares them to the ones
	 * given back from sugr. At last tests if all forbidden combinations throw
	 * an exception
	 */
	public void testGetJacobianOfRelationClassClass() {
		//constructing Points

		//two points which are on one line:
		Point_3D linePoint1 = new Point_3D(0, 0, 0, 1);
		Point_3D linePoint2 = new Point_3D(1, 1, 1, 1);
		//a third one which is not on that line
		Point_3D notOnLinePoint = new Point_3D(2, 1, 1, 1);

		//two points which are equal
		Point_3D eqPoint1 = new Point_3D(7, 6, 1, 1);
		Point_3D eqPoint2 = new Point_3D(14, 12, 2, 2);

		//three points on one plane
		Point_3D planePoint1 = new Point_3D(15, 0, 3, 1);
		Point_3D planePoint2 = new Point_3D(1, 0, 8, 1);
		Point_3D planePoint3 = new Point_3D(3, 0, 5, 1);
		//a fourth one which is not on that plane
		Point_3D notOnPlanePoint = new Point_3D(0, 1, 0, 1);

		//constructing the line out of 2 points and the plane out of three
		Line_3D line = new Line_3D(linePoint1, linePoint2);
		Plane_3D plane = new Plane_3D(planePoint1, planePoint2, planePoint3);

		//getting the Jacobians
		Matrix linePointMatrix = linePoint1.getJacobianOfRelation(
				Incident.class, Line_3D.class);
		Matrix planePointMatrix = planePoint1.getJacobianOfRelation(
				Incident.class, Plane_3D.class);
		Matrix equalMatrix = eqPoint1.getJacobianOfRelation(Equal.class,
				Point_3D.class);
		Matrix notOnLineMatrix = notOnLinePoint.getJacobianOfRelation(
				Incident.class, Line_3D.class);
		Matrix notOnPlaneMatrix = notOnPlanePoint.getJacobianOfRelation(
				Incident.class, Plane_3D.class);

		//test the Jacobians, that means, test if they fullfill the constraint:
		Matrix inc = linePointMatrix.mult(line.getV().getMatrix());
		Matrix inc2 = planePointMatrix.mult(plane.getV().getMatrix());
		Matrix eq = equalMatrix.mult(eqPoint2.getV().getMatrix());
		Matrix ninc = notOnLineMatrix.mult(line.getV().getMatrix());
		Matrix ninc2 = notOnPlaneMatrix.mult(plane.getV().getMatrix());
		Matrix neq = equalMatrix.mult(linePoint1.getV().getMatrix());

		//fail if not null:
		if (inc.vec().getLength() != 0)
			fail();
		if (inc2.vec().getLength() != 0)
			fail();
		if (eq.vec().getLength() != 0)
			fail();

		//fail if null, because it shouldn't work:
		if (ninc.vec().getLength() == 0)
			fail();
		if (ninc2.vec().getLength() == 0)
			fail();
		if (neq.vec().getLength() == 0)
			fail();

		//try every forbidden combination:
		Class[][] tests = { { Equal.class, Point_3D.class },
				{ Incident.class, Line_3D.class },
				{ Incident.class, Plane_3D.class } };

		if (!TestTools.helpJacobianOfRelations(tests, eqPoint1))
			fail();

		//calculate Jacobians and compare them to the ones form sugr
		//a new Point
		Point_3D p = new Point_3D(6, 9, 3);

		//let sugr calculate the matrices
		Matrix incLine1 = p
				.getJacobianOfRelation(Incident.class, Line_3D.class);
		Matrix eqP1 = p.getJacobianOfRelation(Equal.class, Point_3D.class);
		Matrix incPlane1 = p.getJacobianOfRelation(Incident.class,
				Plane_3D.class);

		//do it yourself:
		//Point and plane are incident, if x^T*l = 0 is true. So we need
		//x^T here
		Matrix incPlane2 = p.getV().getMatrix().trans();

		//two points are equal, if Pi(X)*Y = 0
		//so we need Pi(X) (see sugr handbook page 5).
		Vector v = p.getV();
		double[][] valuesPoint = { { v.get(3), 0, 0, -v.get(0) },
				{ 0, v.get(3), 0, -v.get(1) }, { 0, 0, v.get(3), -v.get(2) },
				{ 0, -v.get(2), v.get(1), 0 }, { v.get(2), 0, -v.get(0), 0 },
				{ -v.get(1), v.get(0), 0, 0 } };
		Matrix eqP2 = LinearAlgebraFactory.getMatrix(valuesPoint);

		//Line... Pi Dual Trans() (not other definition of dual!)
		double[][] valuesLine = { { 0, v.get(2), -v.get(1), 0 },
				{ -v.get(2), 0, v.get(0), 0 }, { v.get(1), -v.get(0), 0, 0 },
				{ -v.get(3), 0, 0, v.get(0) }, { 0, -v.get(3), 0, v.get(1) },
				{ 0, 0, -v.get(3), v.get(2) } };
		Matrix incLine2 = LinearAlgebraFactory.getMatrix(valuesLine).trans();
		

		//take differenz between matrizes:
		Matrix diffIncLine = incLine1.minus(incLine2);
		Matrix diffEq = eqP1.minus(eqP2);
		Matrix diffIncPlane = incPlane1.minus(incPlane2);

		//fail if not null
		if (diffIncLine.vec().getLength() != 0)
			fail();
		if (diffEq.vec().getLength() != 0)
			fail();
		if (diffIncPlane.vec().getLength() != 0)
			fail();
	}

	/**
	 * Test for getDofOfRelation. Tests degrees of freedom for all three
	 * allowed combinations and tests if exceptoins are thrown for all others.
	 */
	public void testGetDofOfRelation() {
		//construct point
		Point_3D point = new Point_3D(1, 2, 3);

		//getDofOfRelations
		int dofPoint = point.getDofOfRelation(Equal.class, Point_3D.class);
		int dofPlane = point.getDofOfRelation(Incident.class, Plane_3D.class);
		int dofLine = point.getDofOfRelation(Incident.class, Line_3D.class);

		//test values
		if (dofPoint != 3)
			fail();
		if (dofPlane != 1)
			fail();
		if (dofLine != 2)
			fail();

		//allowed values:
		Class[][] allowed = { { Equal.class, Point_3D.class },
				{ Incident.class, Plane_3D.class },
				{ Incident.class, Line_3D.class } };

		//test if all of the unallowed combinations throw an exception.
		if (!TestTools.helpDofOfRelations(allowed, point))
			fail();
	}

	/**
	 * Test for normalizeEuclidean, uses TestTools. Tests the normization of one
	 * million points without covariance then tests three normilizations with
	 * covariances.
	 */
	public void testNormalizeEuclidean() {
		//point with homogenous part zero
		Point_3D x_axis = new Point_3D(1, 0, 0, 0);

		//doesn't throw an error, but
		//all values are NaN.
		x_axis.normalizeEuclidean();

		//progress monitor:
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
				"Testing normalization of points", "without covariance.", 0,
				1000000);

		//Test over 1 Million Points
		outer: for (int i = 0; i < 1000; i++) {
			for (int j = 0; j < 1000; j++) {
				Point_3D punkt = new Point_3D(i - 100, (j + 100) % 1000, i - j,
						j - 500.5);
				punkt.normalizeEuclidean();
				Vector v = LinearAlgebraFactory.getVector((i - 100)
						/ (j - 500.5), ((j + 100) % 1000) / (j - 500.5),
						(i - j) / (j - 500.5), 1);
				if (!TestStatistics.check(punkt.getV(), v, 0.00000001)) {
					pm.close();
					fail();
				}
				//update progress:
				pm.setProgress(i * j);

				//break if canceld:
				if (pm.isCanceled())
					break outer;
			}
		}
		//close progress monitor
		pm.close();

		//Test Statistics
		Point_3D p1 = new Point_3D(2, 6, 5, 2.6, LinearAlgebraFactory
				.getDiagMatrix(4, 0.02));
		if (!TestTools.helpNormalizeEuclideanStatistics(p1, MAXITER))
			fail();

		double[][] cov = { { 0.2, 0.01, 0.01, 0.01 },
				{ 0.01, 0.3, 0.01, 0.01 }, { 0.01, 0.01, 0.4, 0.01 },
				{ 0.01, 0.01, 0.01, 0.10502 } };
		Point_3D p2 = new Point_3D(264, 654, 523, -67, LinearAlgebraFactory
				.getMatrix(cov));
		if (!TestTools.helpNormalizeEuclideanStatistics(p2, MAXITER))
			fail();

		Point_3D p3 = new Point_3D(-54, 76, 28, 24, LinearAlgebraFactory
				.getMatrix(cov));
		if (!TestTools.helpNormalizeEuclideanStatistics(p3, MAXITER))
			fail();
	}

	/**
	 * Test for getDistance()
	 */
	public void testGetDistance() {
		//ProgressMonitor
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
				"Testing getDistance between point",
				"and point, line and plane.", 0, 100 * 100 * 100);

		//test distance one million times:
		outer: for (int i = 0; i < 100; i++) {
			for (double j = 0; j < 50; j += 0.5) {
				for (int k = 100; k > 0; k--) {
					//construct points
					Point_3D p1 = new Point_3D(i, j, k, i);
					Point_3D p2 = new Point_3D(j, k, i, j);

					//calculate distance between points:
					Vector v = p2.getEuclideanPart().mult(
							p1.getHomogeneousPart().get(0));
					v = v.minus(p1.getEuclideanPart().mult(
							p2.getHomogeneousPart().get(0)));
					double myDist = v.getLength()
							/ Math.abs(p1.getHomogeneousPart().get(0)
									* p2.getHomogeneousPart().get(0));
					double sugrDist = p1.getDistance(p2);

					//fail is wrong
					if (Math.abs(myDist - sugrDist) > 0.00000001) {
						pm.close();
						fail();
					}

					//construct plane
					Plane_3D plane = new Plane_3D(k, i, j, k);

					//distance to first point:
					Vector z = p1.getV().getMatrix().trans().mult(plane.getV());
					Vector n = plane.getHomogeneousPart().mult(
							p1.getHomogeneousPart().get(0));
					double myDistPlane1 = z.normF() / n.normF();
					double sugrDistPlane1 = p1.getDistance(plane);

					//distance to second Point:
					z = p2.getV().getMatrix().trans().mult(plane.getV());
					n = plane.getHomogeneousPart().mult(
							p2.getHomogeneousPart().get(0));
					double myDistPlane2 = z.normF() / n.normF();
					double sugrDistPlane2 = p2.getDistance(plane);

					//test
					if (Math.abs(myDistPlane1 - sugrDistPlane1) > 0.00000001){
						pm.close();
						fail();
					}
					if (Math.abs(myDistPlane2 - sugrDistPlane2) > 0.00000001){
						pm.close();
						fail();
					}

					//construct line
					Line_3D line = new Line_3D(k, i, j, k, i, j);

					//distance to first point:
					Vector lo = line.getEuclideanPart();
					Vector lh = line.getHomogeneousPart();
					double myDistLine1 = lh.SkewSym().mult(
							p1.getEuclideanPart()).plus(lo.mult(i)).normF()
							/ lh.mult(p1.getHomogeneousPart().get(0)).normF();
					double sugrDistLine1 = p1.getDistance(line);

					//distance to second point:
					double myDistLine2 = lh.SkewSym().mult(
							p2.getEuclideanPart()).plus(lo.mult(j)).normF()
							/ lh.mult(p2.getHomogeneousPart().get(0)).normF();
					double sugrDistLine2 = p2.getDistance(line);

					//Test
					if (Math.abs(myDistLine1 - sugrDistLine1) > 0.00000001){
						pm.close();
						fail();
					}
					if (Math.abs(myDistLine2 - sugrDistLine2) > 0.00000001){
						pm.close();
						fail();
					}

					//update progress monitor
					pm.setProgress(i * (int) (2 * j) * k);

					//break if cancel button is pressed
					if (pm.isCanceled())
						break outer;
				}
			}
		}
		//close progress monitor
		pm.close();
	}

	/**
	 * Test for void Point_3D(Vector, Matrix).
	 */
	public void testPoint_3DVectorMatrix() {
		//get some matrix and vector with some values
		double[][] values = { { 0.1, 0.002, 0.002, 0.002 },
				{ 0.002, 0.100562, 0.002, 0.002 },
				{ 0.002, 0.002, 0.09954, 0.002 }, { 0.002, 0.002, 0.002, 0.13 } };
		Matrix cov = LinearAlgebraFactory.getMatrix(values);
		Vector v = LinearAlgebraFactory.getVector(7, Double.MAX_VALUE,
				Double.MIN_VALUE, 1);

		//contruct a point
		Point_3D punkt = new Point_3D(v, cov);

		//test if values are equal...
		if (!TestStatistics.check(v,punkt.getV(),
				cov,punkt.getCov(),0.00000001)) fail();
	}

	/**
	 * Test for void Point_3D(Plane_3D, Line_3D) tests the reasoning of a point
	 * out of a plane and a line three times with help of class TestStatistics
	 */
	public void testPoint_3DPlane_3DLine_3D() {
		//covariances for planes
		double[][] covP = { { 0.095, 0.004, 0.004, 0.004 },
				{ 0.004, 0.12, 0.004, 0.004 },
				{ 0.004, 0.004, 0.10004, 0.004 },
				{ 0.004, 0.004, 0.004, 0.10245 } };

		//covariances for lines
		double[][] covL = { { 0.1214, 0.002, 0.002, 0.002, 0.002, 0.002 },
				{ 0.002, 0.096423, 0.002, 0.002, 0.002, 0.002 },
				{ 0.002, 0.002, 0.11452, 0.002, 0.002, 0.002 },
				{ 0.002, 0.002, 0.002, 0.1244, 0.002, 0.002 },
				{ 0.002, 0.002, 0.002, 0.002, 0.09024, 0.002 },
				{ 0.002, 0.002, 0.002, 0.002, 0.002, 0.10123 } };

		//generate two planes and two lines
		Plane_3D plane1 = new Plane_3D(0.2123, -1.55232, -6.433, 4.777,
				LinearAlgebraFactory.getMatrix(covP));

		Plane_3D plane2 = new Plane_3D(-12211, 3122, 5553, -1223,
				LinearAlgebraFactory.getMatrix(covP).plus(
						LinearAlgebraFactory.getDiagMatrix(4, 0.15)));

		Line_3D line1 = new Line_3D(-1, 5.3, -6.7, 3.689, -4.55, 7.55,
				LinearAlgebraFactory.getMatrix(covL));

		Line_3D line2 = new Line_3D(1255.1, 5233.35, 2355.3, 2622.3, 2666.7,
				8965.66, LinearAlgebraFactory.getDiagMatrix(6, 0.09312));

		//test using helpfunction
		if (!helpPoint_3DPlane_3DLine_3DStatistics(plane1, line1, 0.2))
			fail();
		if (!helpPoint_3DPlane_3DLine_3DStatistics(plane2, line1, 0.3))
			fail();
		if (!helpPoint_3DPlane_3DLine_3DStatistics(plane1, line2, 0.2))
			fail();
		if (!helpPoint_3DPlane_3DLine_3DStatistics(plane2, line2, 0.2))
			fail();
	}

	/**
	 * Test for void Point_3D(Line_3D, Plane_3D) does the same as test above,
	 * but uses other values.
	 */
	public void testPoint_3DLine_3DPlane_3D() {
		//covariances for planes
		double[][] covP = { { 0.095, 0.004, 0.004, 0.004 },
				{ 0.004, 0.12, 0.004, 0.004 },
				{ 0.004, 0.004, 0.10004, 0.004 },
				{ 0.004, 0.004, 0.004, 0.10245 } };
		//covariances for lines
		double[][] covL = { { 0.1214, 0.002, 0.002, 0.002, 0.002, 0.002 },
				{ 0.002, 0.096423, 0.002, 0.002, 0.002, 0.002 },
				{ 0.002, 0.002, 0.11452, 0.002, 0.002, 0.002 },
				{ 0.002, 0.002, 0.002, 0.1244, 0.002, 0.002 },
				{ 0.002, 0.002, 0.002, 0.002, 0.09024, 0.002 },
				{ 0.002, 0.002, 0.002, 0.002, 0.002, 0.10123 } };

		//gelerate two planes and two lines
		Plane_3D plane1 = new Plane_3D(0.2123, -1.55232, -6.433, 4.777,
				LinearAlgebraFactory.getMatrix(covP));

		Plane_3D plane2 = new Plane_3D(-1711, -3122, 5593, -1223,
				LinearAlgebraFactory.getMatrix(covP).plus(
						LinearAlgebraFactory.getDiagMatrix(4, 0.05)));

		Line_3D line1 = new Line_3D(1.08, 5.3, -6.7, 3.689, -4.55, 7.55,
				LinearAlgebraFactory.getMatrix(covL));

		Line_3D line2 = new Line_3D(1255.1, 5233.35, 2355.3, 2622.3, 2666.7,
				8965.66, LinearAlgebraFactory.getDiagMatrix(6, 0.19312));

		//test using help function
		if (!helpPoint_3DLine_3DPlane_3DStatistics(line1, plane1, 0.2))
			fail();
		if (!helpPoint_3DLine_3DPlane_3DStatistics(line1, plane2, 0.2))
			fail();
		if (!helpPoint_3DLine_3DPlane_3DStatistics(line2, plane1, 0.2))
			fail();
		if (!helpPoint_3DLine_3DPlane_3DStatistics(line2, plane2, 0.2))
			fail();
	}

	/**
	 * Test for void Point_3D(Plane_3D, Plane_3D, Plane_3D) Tests reasoning of a
	 * point out of three planes with help of class TestStatistics.
	 */
	public void testPoint_3DPlane_3DPlane_3DPlane_3D() {

		// Test for the result (Vector) of the constructor
		for (double x = -100.1; x < 100; x = x + 1.5) {
			for (double y = -100.1; y < 100; y = y + 13.5)
				for (double z = -100.1; z < 100; z = z + 25.5) {
					//System.out.println(x+" "+y+" "+z);

					double sigma = 0.01;
					Matrix cov = LinearAlgebraFactory.getDiagMatrix(4, sigma
							* sigma);
					cov.set(3, 3, 0);

					Point_3D p1 = new Point_3D(x, y, z, cov);
					Point_3D p2 = new Point_3D(2.2, 5.333, 7.432, cov);
					Point_3D p3 = new Point_3D(3.15, 5.3, 8.99, cov);
					Point_3D p4 = new Point_3D(4.23, 6.65, 9.4, cov);
					Point_3D p5 = new Point_3D(5.12, 7.14, 10.6, cov);

					Plane_3D pl1 = new Plane_3D(p1, p2, p3);
					Plane_3D pl2 = new Plane_3D(p1, p2, p4);
					Plane_3D pl3 = new Plane_3D(p1, p3, p5);

					/*
					 * System.out.println("pl1: "+pl1); System.out.println("pl2
					 * "+pl2); System.out.println("pl3 "+pl3);
					 */

					Point_3D pinter = new Point_3D(pl1, pl2, pl3);
					pinter.normalizeEuclidean();

					Vector dist = p1.getV().minus(pinter.getV());
					if (dist.normF() > 1e-5) {
						System.out.println("Fail bei " + x + " " + y + " " + z
								+ " Distanz: " + dist.normF());
						System.out.println("Schnitt: " + pinter);
						fail();
					}
				}
		}

		// Test of covariance with statistics

		//cov values
		double[][] values = { { 0.00009, 0.000002, 0.000003, 0.0000013 },
				{ 0.000002, 0.00012, 0.000005, 0.0000011 },
				{ 0.000003, 0.000005, 0.00011, 0.0000015 },
				{ 0.0000013, 0.0000011, 0.0000015, 0.00013 } };

		double[][] values2 = { { 0.0135, 0.0001, 0.0002, 0.0003 },
				{ 0.0001, 0.0135, 0.0004, 0.0005 },
				{ 0.0002, 0.0004, 0.0135, 0.0006 },
				{ 0.0003, 0.0005, 0.0006, 0.0135 } };

		//generate six planes
		Plane_3D[] planes = new Plane_3D[6];

		planes[0] = new Plane_3D(5.56, -2.632, 7.2, 5.11, LinearAlgebraFactory
				.getMatrix(values));

		planes[1] = new Plane_3D(-7.22, 8.053, 9.232, 6.3, LinearAlgebraFactory
				.getMatrix(values));

		planes[2] = new Plane_3D(-4.2, 4.333, 12.3, 18, LinearAlgebraFactory
				.getMatrix(values2));

		planes[3] = new Plane_3D(1095.3, -5992, 5568, 1140,
				LinearAlgebraFactory.getMatrix(values));

		planes[4] = new Plane_3D(-4111.5, 1245.111, 6790, 1165.9,
				LinearAlgebraFactory.getMatrix(values));

		planes[5] = new Plane_3D(8884, 4752.36, -8854.223, 7453.85,
				LinearAlgebraFactory.getMatrix(values2));

		//test 2 times, using helpfunction
		if (!helpPoint_3DPlane_3DPlane_3DPlane_3DStatistics(planes[0],
				planes[1], planes[2], 0.2))
			fail();
		if (!helpPoint_3DPlane_3DPlane_3DPlane_3DStatistics(planes[3],
				planes[4], planes[5], 0.2))
			fail();
	}

	/**
	 * Test for void Point_3D(double, double, double, double)
	 */
	public void testPoint_3Ddoubledoubledoubledouble() {
		//generate point
		Point_3D punkt = new Point_3D(6, Double.MAX_VALUE, Double.MIN_VALUE,
				1.7);

		//help vector
		Vector v = punkt.getV();

		//test if all values are equal
		if (v.get(0) != 6 || v.get(1) != Double.MAX_VALUE
				|| v.get(2) != Double.MIN_VALUE || v.get(3) != 1.7)
			fail();
	}

	/** 
	 * Test for Point_3D(x,y,z,sigma_x,sigma_y,sigma_z)
	 *
	 */
	public void testPoint_3Ddoubledoubledoubledoubledoubledouble() {
		//constructing Point:
		Point_3D punkt = new Point_3D(2,4, 30, 0.1, 0.2, 0.3);
		
		// Euclidean normalization must have no effect
		punkt.normalizeEuclidean();

		//control values:
		Vector v = LinearAlgebraFactory.getVector(2, 4, 30, 1);
		Matrix cov = LinearAlgebraFactory.getMatrix(4,4);
		cov.set(0,0, 0.1*0.1);
		cov.set(1,1, 0.2*0.2);
		cov.set(2,2, 0.3*0.3);
		
		//nulling:
		Vector diff = punkt.getV().minus(v);
		Vector diffCov = (punkt.getCov().minus(cov)).vec();
		
		//fail if not null:
		if(diff.getLength() != 0) fail();
		if(diffCov.getLength() != 0) fail();
	}


	/**
	 * Test for void Point_3D(double, double, double, double, Matrix)
	 */
	public void testPoint_3DdoubledoubledoubledoubleMatrix() {
		//get cov values and Matrix
		double[][] values = { { 0.1, 0.002, 0.002, 0.002 },
				{ 0.002, 0.100562, 0.002, 0.002 },
				{ 0.002, 0.002, 0.09954, 0.002 }, { 0.002, 0.002, 0.002, 0.13 } };
		Matrix cov = LinearAlgebraFactory.getMatrix(values);

		//construct point
		Point_3D p = new Point_3D(1.7, 5.1, -5212, 3, cov);

		//help vector
		Vector v = LinearAlgebraFactory.getVector(1.7, 5.1, -5212, 3);

		//test if all values are equal:
		if (!TestStatistics.check(v, p.getV(), cov, p.getCov(), 0.0000001))
			fail();
	}

	/**
	 * Test for void Point_3D(double, double, double)
	 */
	public void testPoint_3Ddoubledoubledouble() {
		//construct point
		Point_3D p = new Point_3D(4.5, 12, 67);

		//help vector
		Vector v = LinearAlgebraFactory.getVector(4.5, 12, 67, 1);

		//test if all values are equal:
		if (!TestStatistics.check(v, p.getV(), 0.00001))
			fail();
	}

	/**
	 * Test for void Point_3D(double, double, double, Matrix)
	 */
	public void testPoint_3DdoubledoubledoubleMatrix() {
		//get cov values and Matrix
		double[][] values = { { 0.1, 0.002, 0.002, 0.002 },
				{ 0.002, 0.100562, 0.002, 0.002 },
				{ 0.002, 0.002, 0.09954, 0.002 }, { 0.002, 0.002, 0.002, 0.13 } };
		Matrix cov = LinearAlgebraFactory.getMatrix(values);

		//construct point
		Point_3D p = new Point_3D(Double.MAX_VALUE, 5l, Double.MIN_VALUE, cov);

		//help vector
		Vector v = LinearAlgebraFactory.getVector(Double.MAX_VALUE, 5l,
				Double.MIN_VALUE, 1);

		//test if all values are equal:
		if (!TestStatistics.check(v, p.getV(), cov, p.getCov(), 0.0000001))
			fail();
	}

	/**
	 * Test for void Point_3D()
	 */
	public void testPoint_3D() {
		Point_3D p = new Point_3D();

		if (p == null)
			fail();
	}

	/**
	 * Test for copy().
	 * Changes original to proof that copy is independent of it.
	 */
	public void testCopy() {
		//constructing point
		Point_3D p1 = new Point_3D(1, 1, 1, 1, LinearAlgebraFactory
				.getDiagMatrix(4, 0.04));

		//copy point
		Point_3D p2 = p1.copy();

		//change first point
		p1.getV().set(0, 2);

		//nulling
		Vector diff = p1.getV().minus(p2.getV());
		Vector v = LinearAlgebraFactory.getVector(1, 1, 1, 1);
		Vector diff2 = p2.getV().minus(v);
		Vector diff3 = (p2.getCov().minus(p1.getCov())).vec();

		//fail if null, because second point would be affected by changes after
		// copy
		if (diff.getLength() == 0)
			fail();
		//fail if not null, because copy would be wrong
		if (diff2.getLength() != 0)
			fail();
		//fail if not null, because copy of cov's would be wrong
		if (diff3.getLength() != 0)
			fail();
	}

	/**
	 * Test for Matrix Pi()
	 */
	public void testPi() {
		//construct point
		Point_3D p = new Point_3D(1, 2, 3, 4);

		//use Pi()
		Matrix sugrPi = p.Pi();

		//construct Pi "by foot"
		double[][] values = { { 4, 0, 0, -1 }, { 0, 4, 0, -2 },
				{ 0, 0, 4, -3 }, { 0, -3, 2, 0 }, { 3, 0, -1, 0 },
				{ -2, 1, 0, 0 } };
		Matrix myPi = LinearAlgebraFactory.getMatrix(values);

		//test if values are equal
		if (!TestStatistics.check(sugrPi,myPi,0.00000001))
			fail();
	}

	/**
	 * Test for Matrix Pi(Vector)
	 */
	public void testPiVector() {
		//construct Vector
		Vector v = LinearAlgebraFactory.getVector(1, 2, 3, 4);

		//use Pi()
		Matrix sugrPi = Point_3D.Pi(v);

		//construct Pi "by foot"
		double[][] values = { { 4, 0, 0, -1 }, { 0, 4, 0, -2 },
				{ 0, 0, 4, -3 }, { 0, -3, 2, 0 }, { 3, 0, -1, 0 },
				{ -2, 1, 0, 0 } };
		Matrix myPi = LinearAlgebraFactory.getMatrix(values);

		//test if values are equal
		if (!TestStatistics.check(sugrPi,myPi,0.00000001))
			fail();
	}

	/**
	 * Test for Matrix PiDual()
	 * Note that PiDual
	 * differs from that of the dissertation (sign) because of
	 * other definition of C  
	 */
	public void testPiDual() {
		//construct point
		Point_3D p = new Point_3D(1, 2, 3, 4);

		//use PiDual()
		Matrix sugrPiDual = p.PiDual();

		//construct Pi "by foot"
		double[][] values = { { 0, +3, -2, 0 }, { -3, 0, +1, 0 },
				{ +2, -1, 0, 0 }, { -4, 0, 0, +1 }, { 0, -4, 0, +2 },
				{ 0, 0, -4, +3 } };
		Matrix myPiDual = LinearAlgebraFactory.getMatrix(values);
		
		//test if values are equal
		if (!TestStatistics.check(sugrPiDual,myPiDual,0.00000001))
			fail();
	}

	/**
	 * Test for Matrix PiDual(Vector)
	 */
	public void testPiDualVector() {
		//construct point
		Vector v = LinearAlgebraFactory.getVector(1, 2, 3, 4);

		//use PiDual()
		Matrix sugrPiDual = Point_3D.PiDual(v);

		//construct Pi "by foot"
		double[][] values = { { 0, 3, -2, 0 }, { -3, 0, 1, 0 },
				{ 2, -1, 0, 0 }, { -4, 0, 0, 1 }, { 0, -4, 0, 2 },
				{ 0, 0, -4, 3 } };
		Matrix myPiDual = LinearAlgebraFactory.getMatrix(values);

		//test if values are equal
		if (!TestStatistics.check(sugrPiDual,myPiDual,0.00000001))
			fail();
	}

	/**
	 * Test for getDistanceToOrigin.
	 * Tests the distance to the origin for one million points.
	 */
	public void testGetDistanceToOrigin() {
		//progress Monitor
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
				"Testing distance between points", "and origin.", 0,
				1000 * 1000);

		//Test over 1 Million Points
		outer: for (int i = 0; i < 1000; i++) {
			for (int j = 0; j < 1000; j++) {
				//constructing point
				Point_3D p1 = new Point_3D(i - 100, (j + 100) % 1000, i,
						j - 500.5);
				Point_3D p2 = new Point_3D((i + j) - 1000, j - 500, j,
						i - 500.5);

				//using get Distance to Origin
				double dist1 = p1.getDistanceToOrigin();
				double dist2 = p2.getDistanceToOrigin();

				//calculate distance for myself
				double dist1Contr = p1.getEuclideanPart().mult(
						1 / p1.getHomogeneousPart().get(0)).getLength();
				double dist2Contr = p2.getEuclideanPart().mult(
						1 / p2.getHomogeneousPart().get(0)).getLength();

				//nulling: (needed because of exactness of double)
				double dist1val = Math.abs(dist1 - dist1Contr);
				double dist2val = Math.abs(dist2 - dist2Contr);

				//control values
				if (dist1val > 0.000000000001 || dist2val > 0.000000000001) {
					System.out.println("Dist1: " + dist1 + " \t\t Kontr: "
							+ dist1Contr);
					System.out.println("Dist2: " + dist2 + " \t\t Kontr: "
							+ dist2Contr);
					pm.close();
					fail();
				}

				//update progress monitor
				pm.setProgress(i * j);

				//break if cancel button is pressed
				if (pm.isCanceled())
					break outer;
			}
		}
		//close progress monitor
		pm.close();

		//special distances:
		Point_3D p1 = new Point_3D(1, 2, 1, 0);
		Point_3D p2 = new Point_3D(0, 0, 0, 1);

		//distance to origin:
		double dist1 = p1.getDistanceToOrigin();
		double dist2 = p2.getDistanceToOrigin();

		//control values:
		if (dist1 != Double.POSITIVE_INFINITY)
			fail();
		if (dist2 != 0)
			fail();
	}
}