/*
 * Created on 04.02.2004
 *
 */
package sugr.test;
import sugr.linalg.LinearAlgebraFactory;
import sugr.linalg.Matrix;
import sugr.linalg.Vector;
import cern.colt.matrix.DoubleMatrix2D;
import cern.colt.matrix.impl.DenseDoubleMatrix2D;
import cern.colt.matrix.linalg.CholeskyDecomposition;
 

/** Generate normally distributed random vectors from a given covariance matrix. This
 *  is done in the following way to support a covariance matrix
 *  with covariances
 * 
 *  Given: covariance matrix of vector v
 *  
 *  1) Cholesky decomposition: sigma=A*At
 *  2) Singular value decomposition from A : A=U*S*Vt
 *     ==> sigma=U*S^2*Ut
 *  3) Error propagation: Use random generator to get a random
 *                        vector y with y1sigma, y2sigma, ...
 *                        (of course without covariances !)
 *     Then v=U*y has covariance matrix sigma.
 * 
 * This class uses Jama for Cholesky decomposition directly.
 * 
 * @author Thomas Laebe, ipb
 *
 */
public class RandomVector {
	
	/** svd from A, sigma=A*At */
	private Matrix [] ULUT;
	
	/** A random number. */
	private static java.util.Random random = new java.util.Random();
	
	/** the offset */
	private Vector v;
	
	/** Initialize random generation with a given covariance.  
	 * @param sigma covariance matrix of the random vector
	 */ 
	public RandomVector(Matrix sigma) {
		v= LinearAlgebraFactory.getVector(sigma.getRowDimension());
		
		// Cholesky decompositions delivers sigma=At*A
		DoubleMatrix2D Jsigma=toColt(sigma);
		DoubleMatrix2D JA=(new CholeskyDecomposition(Jsigma)).getL();
		// Old code for Jama:
		//Jama.Matrix Jsigma=toJama(sigma);
		//Jama.Matrix JA=Jsigma.chol().getL();
		
		Matrix A=toSUGR(JA);
		
		// SVD delivers now sigma=U*S^2*Ut
		ULUT = A.svd();
	}
	
	/** Initialize random generation with a given covariance and vector.
	 * @param v the mean value.
	 * @param sigma covariance matrix of the random vector
	 */ 
	public RandomVector(Vector v, Matrix sigma) {
		this.v= (Vector) v.clone();
		
		// Cholesky decompositions delivers sigma=At*A
		
		// Old code for Jama:
		//Jama.Matrix Jsigma=toColt(sigma);
		//Jama.Matrix JA=Jsigma.chol().getL();
		DoubleMatrix2D Jsigma=toColt(sigma);
		DoubleMatrix2D JA=(new CholeskyDecomposition(Jsigma)).getL();

		Matrix A=toSUGR(JA);
		
		// SVD delivers now sigma=U*S^2*Ut
		ULUT = A.svd();
	}

	
	/** Get next random vector. 
	 * 
	 * @return the random vector
	 */
	public Vector getNextRandom() {
		int n = ULUT[1].getRowDimension();
		Vector dy = LinearAlgebraFactory.getVector(n);
		for (int i = 0; i < n; i++)
			dy.set(i, random.nextGaussian()*ULUT[1].get(i, i));

		return v.plus(ULUT[0].mult(dy));

	}

	/** Converting a sugr.linalg matrix to a jama matrix. 	
	private static Jama.Matrix toJama(Matrix A) {
		Jama.Matrix J =
			new Jama.Matrix(A.getRowDimension(), A.getColumnDimension());
		for (int i = 0; i < J.getRowDimension(); i++)
			for (int j = 0; j < J.getColumnDimension(); j++)
				J.set(i, j, A.get(i, j));
		return J;
	}
	
	/** Converting a Jama matrix to sugr matrix. 
	private static Matrix toSUGR(Jama.Matrix A) {
		Matrix S = LinearAlgebraFactory.getMatrix(A.getRowDimension(), A.getColumnDimension());
		for (int i = 0; i < S.getRowDimension(); i++)
			for (int j = 0; j < S.getColumnDimension(); j++)
				S.set(i, j, A.get(i, j));
		return S;
	}
	*/
	
	/** Converting a sugr.linalg matrix to a Colt matrix. */	
	private static DoubleMatrix2D toColt(Matrix A) {
		DoubleMatrix2D J =
			new DenseDoubleMatrix2D(A.getRowDimension(), A.getColumnDimension());
		for (int i = 0; i < J.rows(); i++)
			for (int j = 0; j < J.columns(); j++)
				J.set(i, j, A.get(i, j));
		return J;
	}
	
	/** Converting a Colt matrix to sugr matrix. */
	private static Matrix toSUGR(DoubleMatrix2D A) {
		Matrix S = LinearAlgebraFactory.getMatrix(A.rows(), A.columns());
		for (int i = 0; i < S.getRowDimension(); i++)
			for (int j = 0; j < S.getColumnDimension(); j++)
				S.set(i, j, A.get(i, j));
		return S;
	}
}
